@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: apic.cc,v 1.94 2006/06/20 16:51:03 sshwarts Exp $
+// $Id: apic.cc,v 1.105 2007/12/07 10:59:18 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (c) 2002 Zwane Mwaikambo, Stanislav Shwartsman
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +18,7 @@
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 //
+/////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
@@ -36,6 +31,9 @@
 
 #define APIC_BROADCAST_PHYSICAL_DESTINATION_MODE (APIC_MAX_ID)
 
+#define BX_LAPIC_FIRST_VECTOR	0x10
+#define BX_LAPIC_LAST_VECTOR	0xfe
+
 ///////////// APIC BUS /////////////
 
 int apic_bus_deliver_interrupt(Bit8u vector, Bit8u dest, Bit8u delivery_mode, Bit8u dest_mode, bx_bool level, bx_bool trig_mode)
@@ -91,9 +89,11 @@ int apic_bus_deliver_interrupt(Bit8u vector, Bit8u dest, Bit8u delivery_mode, Bi
 
 int apic_bus_deliver_lowest_priority(Bit8u vector, Bit8u dest, bx_bool trig_mode, bx_bool broadcast)
 {
+  int i;
+
 #ifndef BX_IMPLEMENT_XAPIC
   // search for if focus processor exists
-  for (int i=0; i<BX_NUM_LOCAL_APICS; i++) {
+  for (i=0; i<BX_NUM_LOCAL_APICS; i++) {
     if(BX_CPU_APIC(i)->is_focus(vector)) {
       BX_CPU_APIC(i)->deliver(vector, APIC_DM_LOWPRI, trig_mode);
       return 1;
@@ -104,7 +104,7 @@ int apic_bus_deliver_lowest_priority(Bit8u vector, Bit8u dest, bx_bool trig_mode
   // focus processor not found, looking for lowest priority agent
   int lowest_priority_agent = -1, lowest_priority = 0x100;
 
-  for (int i=0; i<BX_NUM_LOCAL_APICS; i++) {
+  for (i=0; i<BX_NUM_LOCAL_APICS; i++) {
     if(broadcast || BX_CPU_APIC(i)->match_logical_addr(dest)) {
 #ifndef BX_IMPLEMENT_XAPIC
       int priority = BX_CPU_APIC(i)->get_apr();
@@ -148,6 +148,22 @@ static void apic_bus_broadcast_eoi(Bit8u vector)
   bx_devices.ioapic->receive_eoi(vector);
 }
 
+#endif
+
+// available even if APIC is not compiled in
+void apic_bus_deliver_smi(void)
+{
+  BX_CPU(0)->deliver_SMI();
+}
+
+void apic_bus_broadcast_smi(void)
+{
+  for (unsigned i=0; i<BX_SMP_PROCESSORS; i++)
+    BX_CPU(i)->deliver_SMI();
+}
+
+#if BX_SUPPORT_APIC
+
 ////////////////////////////////////
 
 bx_generic_apic_c::bx_generic_apic_c(bx_phy_address base)
@@ -273,7 +289,7 @@ void bx_local_apic_c::init()
   log_dest = 0;
   task_priority = 0;
 
-  for(i=0; i<BX_LOCAL_APIC_MAX_INTS; i++) {
+  for(i=0; i<BX_LAPIC_MAX_INTS; i++) {
     irr[i] = isr[i] = tmr[i] = 0;
   }
 
@@ -508,11 +524,11 @@ void bx_local_apic_c::receive_EOI(Bit32u value)
 
 void bx_local_apic_c::startup_msg(Bit32u vector)
 {
-  if(cpu->debug_trap & BX_DEBUG_TRAP_HALT_STATE) {
-    cpu->debug_trap &= ~BX_DEBUG_TRAP_HALT_STATE;
-    cpu->dword.eip = 0;
+  if(cpu->debug_trap & BX_DEBUG_TRAP_SPECIAL) {
+    cpu->debug_trap &= ~BX_DEBUG_TRAP_SPECIAL;
+    cpu->eip_reg.dword.eip = 0;
     cpu->load_seg_reg(&cpu->sregs[BX_SEG_REG_CS], vector*0x100);
-    BX_INFO(("%s started up at %04X:%08X by APIC", cpu->name, vector*0x100, cpu->dword.eip));
+    BX_INFO(("%s started up at %04X:%08X by APIC", cpu->name, vector*0x100, cpu->eip_reg.dword.eip));
   } else {
     BX_INFO(("%s started up by APIC, but was not halted at the time", cpu->name));
   }
@@ -643,7 +659,7 @@ void bx_local_apic_c::read_aligned(bx_phy_address addr, Bit32u *data)
 
 int bx_local_apic_c::highest_priority_int(Bit8u *array)
 {
-  for(int i=BX_APIC_LAST_VECTOR; i>=BX_APIC_FIRST_VECTOR; i--)
+  for(int i=BX_LAPIC_LAST_VECTOR; i>=BX_LAPIC_FIRST_VECTOR; i--)
     if(array[i]) return i;
 
   return -1;
@@ -715,7 +731,7 @@ void bx_local_apic_c::trigger_irq(unsigned vector, unsigned trigger_mode, bx_boo
 {
   BX_DEBUG(("Local apic on %s: trigger interrupt vector=0x%x", cpu->name, vector));
   
-  if(vector > BX_APIC_LAST_VECTOR || vector < BX_APIC_FIRST_VECTOR) {
+  if(vector > BX_LAPIC_LAST_VECTOR || vector < BX_LAPIC_FIRST_VECTOR) {
     shadow_error_status |= APIC_ERR_RX_ILLEGAL_VEC;
     BX_INFO(("bogus vector %#x, ignoring ...", vector));
     return;
@@ -779,7 +795,7 @@ spurious:
 void bx_local_apic_c::print_status(void)
 {
   BX_INFO(("%s local apic: status is {:", cpu->name));
-  for(int vec=0; vec<BX_LOCAL_APIC_MAX_INTS; vec++) {
+  for(int vec=0; vec<BX_LAPIC_MAX_INTS; vec++) {
     if(irr[vec] || isr[vec]) {
       BX_INFO(("vec 0x%x: irr=%d, isr=%d", vec,(int)irr[vec],(int)isr[vec]));
     }
@@ -935,7 +951,6 @@ void bx_local_apic_c::set_initial_timer_count(Bit32u value)
   }
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_local_apic_c::register_state(bx_param_c *parent)
 {
   unsigned i;
@@ -953,10 +968,10 @@ void bx_local_apic_c::register_state(bx_param_c *parent)
   BXRS_HEX_PARAM_SIMPLE(lapic, log_dest);
   BXRS_HEX_PARAM_SIMPLE(lapic, dest_format);
   
-  bx_list_c *ISR = new bx_list_c(lapic, "isr", BX_LOCAL_APIC_MAX_INTS);
-  bx_list_c *TMR = new bx_list_c(lapic, "tmr", BX_LOCAL_APIC_MAX_INTS);
-  bx_list_c *IRR = new bx_list_c(lapic, "irr", BX_LOCAL_APIC_MAX_INTS);
-  for (i=0; i<BX_LOCAL_APIC_MAX_INTS; i++) {
+  bx_list_c *ISR = new bx_list_c(lapic, "isr", BX_LAPIC_MAX_INTS);
+  bx_list_c *TMR = new bx_list_c(lapic, "tmr", BX_LAPIC_MAX_INTS);
+  bx_list_c *IRR = new bx_list_c(lapic, "irr", BX_LAPIC_MAX_INTS);
+  for (i=0; i<BX_LAPIC_MAX_INTS; i++) {
     sprintf(name, "0x%02x", i);
     new bx_shadow_num_c(ISR, name, &isr[i]);
     new bx_shadow_num_c(TMR, name, &tmr[i]);
@@ -982,6 +997,5 @@ void bx_local_apic_c::register_state(bx_param_c *parent)
   BXRS_HEX_PARAM_SIMPLE(lapic, ticksInitial);
   BXRS_PARAM_BOOL(lapic, INTR, INTR);
 }
-#endif
 
 #endif /* if BX_SUPPORT_APIC */
