@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: flag_ctrl_pro.cc,v 1.24 2006/04/05 17:31:31 sshwarts Exp $
+// $Id: flag_ctrl_pro.cc,v 1.29 2007/11/20 21:22:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -35,12 +35,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::setEFlags(Bit32u val)
 {
   // VM flag could not be set from long mode
 #if BX_SUPPORT_X86_64
-  if (IsLongMode()) {
+  if (long_mode()) {
     if (BX_CPU_THIS_PTR get_VM()) BX_PANIC(("VM is set in long mode !"));
     val &= ~EFlagsVMMask;
   }
 #endif
 
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+  handleAlignmentCheck();
+#endif
+
   BX_CPU_THIS_PTR eflags.val32 = val;
   BX_CPU_THIS_PTR lf_flags_status = 0; // OSZAPC flags are known.
   BX_CPU_THIS_PTR set_VM(val & EFlagsVMMask);
@@ -49,22 +53,20 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::setEFlags(Bit32u val)
   void BX_CPP_AttrRegparmN(2)
 BX_CPU_C::writeEFlags(Bit32u flags, Bit32u changeMask)
 {
-  Bit32u supportMask, newEFlags;
-  
   // Build a mask of the non-reserved bits:
   // ID,VIP,VIF,AC,VM,RF,x,NT,IOPL,OF,DF,IF,TF,SF,ZF,x,AF,x,PF,x,CF
-  supportMask = 0x00037fd5;
+  Bit32u supportMask = 0x00037fd5;
 #if BX_CPU_LEVEL >= 4
   supportMask |= (EFlagsIDMask | EFlagsACMask); // ID/AC
 #endif
 #if BX_SUPPORT_VME
-  supportMask |= (EFlagsVPMask | EFlagsVFMask); // VIP/VIF
+  supportMask |= (EFlagsVIPMask | EFlagsVIFMask); // VIP/VIF
 #endif
 
   // Screen out changing of any unsupported bits.
   changeMask &= supportMask;
 
-  newEFlags = (BX_CPU_THIS_PTR eflags.val32 & ~changeMask) |
+  Bit32u newEFlags = (BX_CPU_THIS_PTR eflags.val32 & ~changeMask) |
               (flags & changeMask);
   setEFlags(newEFlags);
   // OSZAPC flags are known - done in setEFlags(newEFlags)
@@ -96,12 +98,16 @@ BX_CPU_C::write_flags(Bit16u flags, bx_bool change_IOPL, bx_bool change_IF)
 Bit32u BX_CPU_C::force_flags(void)
 {
   if (BX_CPU_THIS_PTR lf_flags_status) {
-    (void) get_CF();
-    (void) get_PF();
-    (void) get_AF();
-    (void) get_ZF();
-    (void) get_SF();
-    (void) get_OF();
+    Bit32u newflags;
+
+    newflags  = get_CF() ? EFlagsCFMask : 0;
+    newflags |= get_PF() ? EFlagsPFMask : 0;
+    newflags |= get_AF() ? EFlagsAFMask : 0;
+    newflags |= get_ZF() ? EFlagsZFMask : 0;
+    newflags |= get_SF() ? EFlagsSFMask : 0;
+    newflags |= get_OF() ? EFlagsOFMask : 0;
+
+    setEFlagsOSZAPC(newflags);
   }
 
   return BX_CPU_THIS_PTR eflags.val32;
