@@ -1,9 +1,9 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: i387.h,v 1.31 2005/06/09 17:42:34 vruppert Exp $
+// $Id: i387.h,v 1.33 2007/03/23 21:27:12 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2004 Stanislav Shwartsman
-//          Written by Stanislav Shwartsman <stl at fidonet.org.il>
+//          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -117,13 +117,13 @@ BX_CPP_INLINE void i387_t::FPU_settagi(int tag, int stnr)
 
 BX_CPP_INLINE void i387_t::FPU_push(void)
 {
-  tos--;
+  tos = (tos - 1) & 7;
 }
 
 BX_CPP_INLINE void i387_t::FPU_pop(void)
 {
-  twd |= 3 << ((tos & 7)*2);
-  tos++;
+  twd |= 3 << (tos*2);
+  tos = (tos + 1) & 7;
 }
 
 BX_CPP_INLINE void i387_t::FPU_save_regi(floatx80 reg, int tag, int stnr)
