@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: init.cc,v 1.120 2006/08/26 07:35:27 vruppert Exp $
+// $Id: init.cc,v 1.145 2007/11/24 14:22:34 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -23,6 +23,8 @@
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//
+/////////////////////////////////////////////////////////////////////////
 
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -35,7 +37,6 @@
 #define RIP EIP
 #endif
 
-
 BX_CPU_C::BX_CPU_C(unsigned id): bx_cpuid(id)
 #if BX_SUPPORT_APIC
    ,local_apic (this)
@@ -162,111 +163,34 @@ void BX_CPU_C::initialize(BX_MEM_C *addrspace)
 
   // in SMP mode, the prefix of the CPU will be changed to [CPUn] in 
   // bx_local_apic_c::set_id as soon as the apic ID is assigned.
-
-// <TAG-INIT-CPU-START>
-  // for decoding instructions: access to seg reg's via index number
-  sreg_mod00_rm16[0] = BX_SEG_REG_DS;
-  sreg_mod00_rm16[1] = BX_SEG_REG_DS;
-  sreg_mod00_rm16[2] = BX_SEG_REG_SS;
-  sreg_mod00_rm16[3] = BX_SEG_REG_SS;
-  sreg_mod00_rm16[4] = BX_SEG_REG_DS;
-  sreg_mod00_rm16[5] = BX_SEG_REG_DS;
-  sreg_mod00_rm16[6] = BX_SEG_REG_DS;
-  sreg_mod00_rm16[7] = BX_SEG_REG_DS;
-
-  sreg_mod01or10_rm16[0] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm16[1] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm16[2] = BX_SEG_REG_SS;
-  sreg_mod01or10_rm16[3] = BX_SEG_REG_SS;
-  sreg_mod01or10_rm16[4] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm16[5] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm16[6] = BX_SEG_REG_SS;
-  sreg_mod01or10_rm16[7] = BX_SEG_REG_DS;
-
-  // the default segment to use for a one-byte modrm with 
-  // mod==01b or mod==10b and rm == i
-  sreg_mod01or10_rm32[0] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[1] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[2] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[3] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[4] = BX_SEG_REG_NULL;
-    // this entry should never be accessed
-    // (escape to 2-byte)
-  sreg_mod01or10_rm32[5] = BX_SEG_REG_SS;
-  sreg_mod01or10_rm32[6] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[7] = BX_SEG_REG_DS;
-#if BX_SUPPORT_X86_64
-  sreg_mod01or10_rm32[8] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[9] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[10] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[11] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[12] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[13] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[14] = BX_SEG_REG_DS;
-  sreg_mod01or10_rm32[15] = BX_SEG_REG_DS;
-#endif
-
-  // the default segment to use for a two-byte modrm with
-  // mod==00b and base == i
-  sreg_mod0_base32[0] = BX_SEG_REG_DS;
-  sreg_mod0_base32[1] = BX_SEG_REG_DS;
-  sreg_mod0_base32[2] = BX_SEG_REG_DS;
-  sreg_mod0_base32[3] = BX_SEG_REG_DS;
-  sreg_mod0_base32[4] = BX_SEG_REG_SS;
-  sreg_mod0_base32[5] = BX_SEG_REG_DS;
-  sreg_mod0_base32[6] = BX_SEG_REG_DS;
-  sreg_mod0_base32[7] = BX_SEG_REG_DS;
-#if BX_SUPPORT_X86_64
-  sreg_mod0_base32[8] = BX_SEG_REG_DS;
-  sreg_mod0_base32[9] = BX_SEG_REG_DS;
-  sreg_mod0_base32[10] = BX_SEG_REG_DS;
-  sreg_mod0_base32[11] = BX_SEG_REG_DS;
-  sreg_mod0_base32[12] = BX_SEG_REG_DS;
-  sreg_mod0_base32[13] = BX_SEG_REG_DS;
-  sreg_mod0_base32[14] = BX_SEG_REG_DS;
-  sreg_mod0_base32[15] = BX_SEG_REG_DS;
-#endif
-
-  // the default segment to use for a two-byte modrm with
-  // mod==01b or mod==10b and base == i
-  sreg_mod1or2_base32[0] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[1] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[2] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[3] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[4] = BX_SEG_REG_SS;
-  sreg_mod1or2_base32[5] = BX_SEG_REG_SS;
-  sreg_mod1or2_base32[6] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[7] = BX_SEG_REG_DS;
-#if BX_SUPPORT_X86_64
-  sreg_mod1or2_base32[8] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[9] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[10] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[11] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[12] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[13] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[14] = BX_SEG_REG_DS;
-  sreg_mod1or2_base32[15] = BX_SEG_REG_DS;
-#endif
-// <TAG-INIT-CPU-END>
-
-  mem = addrspace;
   sprintf(name, "CPU %d", BX_CPU_ID);
 
+  mem = addrspace;
+
 #if BX_WITH_WX
+  register_wx_state();
+#endif
+}
+
+#if BX_WITH_WX
+void BX_CPU_C::register_wx_state(void)
+{
   if (SIM->get_param(BXPN_WX_CPU_STATE) != NULL) {
     // Register some of the CPUs variables as shadow parameters so that
     // they can be visible in the config interface.
     // (Experimental, obviously not a complete list)
     bx_param_num_c *param;
-    char cpu_name[10], cpu_title[10];
+    char cpu_name[10], cpu_title[10], cpu_pname[16];
     const char *fmt16 = "%04X";
     const char *fmt32 = "%08X";
     Bit32u oldbase = bx_param_num_c::set_default_base(16);
     const char *oldfmt = bx_param_num_c::set_default_format(fmt32);
     sprintf(cpu_name, "%d", BX_CPU_ID);
     sprintf(cpu_title, "CPU %d", BX_CPU_ID);
-    bx_list_c *list = new bx_list_c(SIM->get_param(BXPN_WX_CPU_STATE), 
-       cpu_name, cpu_title, 60);
+    sprintf(cpu_pname, "%s.%d", BXPN_WX_CPU_STATE, BX_CPU_ID);
+    if (SIM->get_param(cpu_pname) == NULL) {
+      bx_list_c *list = new bx_list_c(SIM->get_param(BXPN_WX_CPU_STATE), 
+         cpu_name, cpu_title, 60);
 
 #define DEFPARAM_NORMAL(name,field) \
     new bx_shadow_num_c(list, #name, &(field))
@@ -291,11 +215,11 @@ void BX_CPU_C::initialize(BX_MEM_C *addrspace)
       DEFPARAM_NORMAL(CR2, cr2);
       DEFPARAM_NORMAL(CR3, cr3);
 #if BX_CPU_LEVEL >= 4
-      DEFPARAM_NORMAL(CR4, cr4.registerValue);
+      DEFPARAM_NORMAL(CR4, cr4.val32);
 #endif
 
-    // segment registers require a handler function because they have
-    // special get/set requirements.
+      // segment registers require a handler function because they have
+      // special get/set requirements.
 #define DEFPARAM_SEG_REG(x) \
     param = new bx_param_num_c(list, \
       #x, #x, "", 0, 0xffff, 0); \
@@ -309,25 +233,25 @@ void BX_CPU_C::initialize(BX_MEM_C *addrspace)
         #name"_limit", \
         & BX_CPU_THIS_PTR field.limit);
 
-    DEFPARAM_SEG_REG(CS);
-    DEFPARAM_SEG_REG(DS);
-    DEFPARAM_SEG_REG(SS);
-    DEFPARAM_SEG_REG(ES);
-    DEFPARAM_SEG_REG(FS);
-    DEFPARAM_SEG_REG(GS);
-    DEFPARAM_SEG_REG(LDTR);
-    DEFPARAM_SEG_REG(TR);
-    DEFPARAM_GLOBAL_SEG_REG(GDTR, gdtr);
-    DEFPARAM_GLOBAL_SEG_REG(IDTR, idtr);
+      DEFPARAM_SEG_REG(CS);
+      DEFPARAM_SEG_REG(DS);
+      DEFPARAM_SEG_REG(SS);
+      DEFPARAM_SEG_REG(ES);
+      DEFPARAM_SEG_REG(FS);
+      DEFPARAM_SEG_REG(GS);
+      DEFPARAM_SEG_REG(LDTR);
+      DEFPARAM_SEG_REG(TR);
+      DEFPARAM_GLOBAL_SEG_REG(GDTR, gdtr);
+      DEFPARAM_GLOBAL_SEG_REG(IDTR, idtr);
 #undef DEFPARAM_NORMAL
 #undef DEFPARAM_SEG_REG
 #undef DEFPARAM_GLOBAL_SEG_REG
 
-    param = new bx_shadow_num_c(list, "EFLAGS",
-        &BX_CPU_THIS_PTR eflags.val32);
+      param = new bx_shadow_num_c(list, "EFLAGS",
+          &BX_CPU_THIS_PTR eflags.val32);
 
-    // flags implemented in lazy_flags.cc must be done with a handler
-    // that calls their get function, to force them to be computed.
+      // flags implemented in lazy_flags.cc must be done with a handler
+      // that calls their get function, to force them to be computed.
 #define DEFPARAM_EFLAG(name) \
     param = new bx_param_bool_c(list, \
             #name, #name, "", get_##name()); \
@@ -338,108 +262,109 @@ void BX_CPU_C::initialize(BX_MEM_C *addrspace)
     param->set_handler(cpu_param_handler);
 
 #if BX_CPU_LEVEL >= 4
-    DEFPARAM_EFLAG(ID);
-    DEFPARAM_EFLAG(VIP);
-    DEFPARAM_EFLAG(VIF);
-    DEFPARAM_EFLAG(AC);
+      DEFPARAM_EFLAG(ID);
+      DEFPARAM_EFLAG(VIP);
+      DEFPARAM_EFLAG(VIF);
+      DEFPARAM_EFLAG(AC);
 #endif
 #if BX_CPU_LEVEL >= 3
-    DEFPARAM_EFLAG(VM);
-    DEFPARAM_EFLAG(RF);
+      DEFPARAM_EFLAG(VM);
+      DEFPARAM_EFLAG(RF);
 #endif
 #if BX_CPU_LEVEL >= 2
-    DEFPARAM_EFLAG(NT);
-    // IOPL is a special case because it is 2 bits wide.
-    param = new bx_shadow_num_c(
-            list,
-            "IOPL",
-            &eflags.val32, 10,
-            12, 13);
-    param->set_range(0, 3);
-    param->set_format("%d");
+      DEFPARAM_EFLAG(NT);
+      // IOPL is a special case because it is 2 bits wide.
+      param = new bx_shadow_num_c(
+              list,
+              "IOPL",
+              &eflags.val32, 10,
+              12, 13);
+      param->set_range(0, 3);
+      param->set_format("%d");
 #endif
-    DEFPARAM_LAZY_EFLAG(OF);
-    DEFPARAM_EFLAG(DF);
-    DEFPARAM_EFLAG(IF);
-    DEFPARAM_EFLAG(TF);
-    DEFPARAM_LAZY_EFLAG(SF);
-    DEFPARAM_LAZY_EFLAG(ZF);
-    DEFPARAM_LAZY_EFLAG(AF);
-    DEFPARAM_LAZY_EFLAG(PF);
-    DEFPARAM_LAZY_EFLAG(CF);
+      DEFPARAM_LAZY_EFLAG(OF);
+      DEFPARAM_EFLAG(DF);
+      DEFPARAM_EFLAG(IF);
+      DEFPARAM_EFLAG(TF);
+      DEFPARAM_LAZY_EFLAG(SF);
+      DEFPARAM_LAZY_EFLAG(ZF);
+      DEFPARAM_LAZY_EFLAG(AF);
+      DEFPARAM_LAZY_EFLAG(PF);
+      DEFPARAM_LAZY_EFLAG(CF);
 
-    // restore defaults
-    bx_param_num_c::set_default_base(oldbase);
-    bx_param_num_c::set_default_format(oldfmt);
+      // restore defaults
+      bx_param_num_c::set_default_base(oldbase);
+      bx_param_num_c::set_default_format(oldfmt);
+    }
   }
-#endif
 }
+#endif
 
-#if BX_SUPPORT_SAVE_RESTORE
+// save/restore functionality
 void BX_CPU_C::register_state(void)
 {
   unsigned i;
-  char cpu_name[10], cpu_title[10], name[10];
+  char name[10];
 
-  sprintf(cpu_name, "%d", BX_CPU_ID);
-  sprintf(cpu_title, "CPU %d", BX_CPU_ID);
-  bx_list_c *list = new bx_list_c(SIM->get_param("save_restore.cpu"), 
-           cpu_name, cpu_title, 60);
+  sprintf(name, "cpu%d", BX_CPU_ID);
+  
+  bx_list_c *cpu = new bx_list_c(SIM->get_bochs_root(), name, name, 60);
 
-  BXRS_PARAM_SPECIAL32(list, cpu_version, param_save_handler, param_restore_handler);
-  BXRS_PARAM_SPECIAL32(list, cpuid_std,   param_save_handler, param_restore_handler);
-  BXRS_PARAM_SPECIAL32(list, cpuid_ext,   param_save_handler, param_restore_handler);
-  BXRS_DEC_PARAM_SIMPLE(list, cpu_mode);
-  BXRS_HEX_PARAM_SIMPLE(list, inhibit_mask);
+  BXRS_PARAM_SPECIAL32(cpu, cpu_version, param_save_handler, param_restore_handler);
+  BXRS_PARAM_SPECIAL32(cpu, cpuid_std,   param_save_handler, param_restore_handler);
+  BXRS_PARAM_SPECIAL32(cpu, cpuid_ext,   param_save_handler, param_restore_handler);
+  BXRS_DEC_PARAM_SIMPLE(cpu, cpu_mode);
+  BXRS_HEX_PARAM_SIMPLE(cpu, inhibit_mask);
+  BXRS_HEX_PARAM_SIMPLE(cpu, debug_trap);
 #if BX_SUPPORT_X86_64
-  BXRS_HEX_PARAM_SIMPLE(list, RAX);
-  BXRS_HEX_PARAM_SIMPLE(list, RBX);
-  BXRS_HEX_PARAM_SIMPLE(list, RCX);
-  BXRS_HEX_PARAM_SIMPLE(list, RDX);
-  BXRS_HEX_PARAM_SIMPLE(list, RSP);
-  BXRS_HEX_PARAM_SIMPLE(list, RBP);
-  BXRS_HEX_PARAM_SIMPLE(list, RSI);
-  BXRS_HEX_PARAM_SIMPLE(list, RDI);
-  BXRS_HEX_PARAM_SIMPLE(list, R8);
-  BXRS_HEX_PARAM_SIMPLE(list, R9);
-  BXRS_HEX_PARAM_SIMPLE(list, R10);
-  BXRS_HEX_PARAM_SIMPLE(list, R11);
-  BXRS_HEX_PARAM_SIMPLE(list, R12);
-  BXRS_HEX_PARAM_SIMPLE(list, R13);
-  BXRS_HEX_PARAM_SIMPLE(list, R14);
-  BXRS_HEX_PARAM_SIMPLE(list, R15);
-  BXRS_HEX_PARAM_SIMPLE(list, RIP);
+  BXRS_HEX_PARAM_SIMPLE(cpu, RAX);
+  BXRS_HEX_PARAM_SIMPLE(cpu, RBX);
+  BXRS_HEX_PARAM_SIMPLE(cpu, RCX);
+  BXRS_HEX_PARAM_SIMPLE(cpu, RDX);
+  BXRS_HEX_PARAM_SIMPLE(cpu, RSP);
+  BXRS_HEX_PARAM_SIMPLE(cpu, RBP);
+  BXRS_HEX_PARAM_SIMPLE(cpu, RSI);
+  BXRS_HEX_PARAM_SIMPLE(cpu, RDI);
+  BXRS_HEX_PARAM_SIMPLE(cpu, R8);
+  BXRS_HEX_PARAM_SIMPLE(cpu, R9);
+  BXRS_HEX_PARAM_SIMPLE(cpu, R10);
+  BXRS_HEX_PARAM_SIMPLE(cpu, R11);
+  BXRS_HEX_PARAM_SIMPLE(cpu, R12);
+  BXRS_HEX_PARAM_SIMPLE(cpu, R13);
+  BXRS_HEX_PARAM_SIMPLE(cpu, R14);
+  BXRS_HEX_PARAM_SIMPLE(cpu, R15);
+  BXRS_HEX_PARAM_SIMPLE(cpu, RIP);
 #else
-  BXRS_HEX_PARAM_SIMPLE(list, EAX);
-  BXRS_HEX_PARAM_SIMPLE(list, EBX);
-  BXRS_HEX_PARAM_SIMPLE(list, ECX);
-  BXRS_HEX_PARAM_SIMPLE(list, EDX);
-  BXRS_HEX_PARAM_SIMPLE(list, ESP);
-  BXRS_HEX_PARAM_SIMPLE(list, EBP);
-  BXRS_HEX_PARAM_SIMPLE(list, ESI);
-  BXRS_HEX_PARAM_SIMPLE(list, EDI);
-  BXRS_HEX_PARAM_SIMPLE(list, EIP);
+  BXRS_HEX_PARAM_SIMPLE(cpu, EAX);
+  BXRS_HEX_PARAM_SIMPLE(cpu, EBX);
+  BXRS_HEX_PARAM_SIMPLE(cpu, ECX);
+  BXRS_HEX_PARAM_SIMPLE(cpu, EDX);
+  BXRS_HEX_PARAM_SIMPLE(cpu, ESP);
+  BXRS_HEX_PARAM_SIMPLE(cpu, EBP);
+  BXRS_HEX_PARAM_SIMPLE(cpu, ESI);
+  BXRS_HEX_PARAM_SIMPLE(cpu, EDI);
+  BXRS_HEX_PARAM_SIMPLE(cpu, EIP);
 #endif
-  BXRS_PARAM_SPECIAL32(list, EFLAGS, 
+  BXRS_PARAM_SPECIAL32(cpu, EFLAGS, 
          param_save_handler, param_restore_handler);
 #if BX_CPU_LEVEL >= 3
-  BXRS_HEX_PARAM_FIELD(list, DR0, dr0);
-  BXRS_HEX_PARAM_FIELD(list, DR1, dr1);
-  BXRS_HEX_PARAM_FIELD(list, DR2, dr2);
-  BXRS_HEX_PARAM_FIELD(list, DR3, dr3);
-  BXRS_HEX_PARAM_FIELD(list, DR6, dr6);
-  BXRS_HEX_PARAM_FIELD(list, DR7, dr7);
+  BXRS_HEX_PARAM_FIELD(cpu, DR0, dr0);
+  BXRS_HEX_PARAM_FIELD(cpu, DR1, dr1);
+  BXRS_HEX_PARAM_FIELD(cpu, DR2, dr2);
+  BXRS_HEX_PARAM_FIELD(cpu, DR3, dr3);
+  BXRS_HEX_PARAM_FIELD(cpu, DR6, dr6);
+  BXRS_HEX_PARAM_FIELD(cpu, DR7, dr7);
 #endif
-  BXRS_HEX_PARAM_FIELD(list, cR0, cr0.val32);
-  BXRS_HEX_PARAM_FIELD(list, CR2, cr2);
-  BXRS_HEX_PARAM_FIELD(list, CR3, cr3);
+  BXRS_HEX_PARAM_FIELD(cpu, CR0, cr0.val32);
+  BXRS_HEX_PARAM_FIELD(cpu, CR2, cr2);
+  BXRS_HEX_PARAM_FIELD(cpu, CR3, cr3);
 #if BX_CPU_LEVEL >= 4
-  BXRS_HEX_PARAM_FIELD(list, CR4, cr4.registerValue);
+  BXRS_HEX_PARAM_FIELD(cpu, CR4, cr4.val32);
 #endif
 
   for(i=0; i<6; i++) {
     bx_segment_reg_t *segment = &BX_CPU_THIS_PTR sregs[i];
-    bx_list_c *sreg = new bx_list_c(list, strseg(segment), 9);
+    bx_list_c *sreg = new bx_list_c(cpu, strseg(segment), 9);
     BXRS_PARAM_SPECIAL16(sreg, selector, 
            param_save_handler, param_restore_handler);
     BXRS_HEX_PARAM_FIELD(sreg, base, segment->cache.u.segment.base);
@@ -456,34 +381,34 @@ void BX_CPU_C::register_state(void)
   }
 
 #if BX_CPU_LEVEL >= 2
-  BXRS_HEX_PARAM_FIELD(list, GDTR_BASE, BX_CPU_THIS_PTR gdtr.base);
-  BXRS_HEX_PARAM_FIELD(list, GDTR_LIMIT, BX_CPU_THIS_PTR gdtr.limit);
-  BXRS_HEX_PARAM_FIELD(list, IDTR_BASE, BX_CPU_THIS_PTR idtr.base);
-  BXRS_HEX_PARAM_FIELD(list, IDTR_LIMIT, BX_CPU_THIS_PTR idtr.limit);
+  BXRS_HEX_PARAM_FIELD(cpu, gdtr_base, gdtr.base);
+  BXRS_HEX_PARAM_FIELD(cpu, gdtr_limit, gdtr.limit);
+  BXRS_HEX_PARAM_FIELD(cpu, idtr_base, idtr.base);
+  BXRS_HEX_PARAM_FIELD(cpu, idtr_limit, idtr.limit);
 #endif
 
-  bx_list_c *LDTR = new bx_list_c (list, "LDTR", 7);
+  bx_list_c *LDTR = new bx_list_c (cpu, "LDTR", 7);
   BXRS_PARAM_SPECIAL16(LDTR, selector, param_save_handler, param_restore_handler);
-  BXRS_HEX_PARAM_FIELD(LDTR, base,  ldtr.cache.u.ldt.base);
-  BXRS_HEX_PARAM_FIELD(LDTR, limit, ldtr.cache.u.ldt.limit);
-  BXRS_HEX_PARAM_FIELD(LDTR, limit_scaled, ldtr.cache.u.ldt.limit);
+  BXRS_HEX_PARAM_FIELD(LDTR, base,  ldtr.cache.u.system.base);
+  BXRS_HEX_PARAM_FIELD(LDTR, limit, ldtr.cache.u.system.limit);
+  BXRS_HEX_PARAM_FIELD(LDTR, limit_scaled, ldtr.cache.u.system.limit);
   BXRS_PARAM_SPECIAL8 (LDTR, ar_byte, param_save_handler, param_restore_handler);
-  BXRS_PARAM_BOOL(LDTR, granularity, ldtr.cache.u.ldt.g);
-  BXRS_PARAM_BOOL(LDTR, avl, ldtr.cache.u.ldt.avl);
+  BXRS_PARAM_BOOL(LDTR, granularity, ldtr.cache.u.system.g);
+  BXRS_PARAM_BOOL(LDTR, avl, ldtr.cache.u.system.avl);
 
-  bx_list_c *TR = new bx_list_c (list, "TR", 7);
+  bx_list_c *TR = new bx_list_c (cpu, "TR", 7);
   BXRS_PARAM_SPECIAL16(TR, selector, param_save_handler, param_restore_handler);
-  BXRS_HEX_PARAM_FIELD(TR, base,  tr.cache.u.tss.base);
-  BXRS_HEX_PARAM_FIELD(TR, limit, tr.cache.u.tss.limit);
-  BXRS_HEX_PARAM_FIELD(TR, limit_scaled, tr.cache.u.tss.limit_scaled);
+  BXRS_HEX_PARAM_FIELD(TR, base,  tr.cache.u.system.base);
+  BXRS_HEX_PARAM_FIELD(TR, limit, tr.cache.u.system.limit);
+  BXRS_HEX_PARAM_FIELD(TR, limit_scaled, tr.cache.u.system.limit_scaled);
   BXRS_PARAM_SPECIAL8 (TR, ar_byte, param_save_handler, param_restore_handler);
-  BXRS_PARAM_BOOL(TR, granularity, tr.cache.u.tss.g);
-  BXRS_PARAM_BOOL(TR, avl, tr.cache.u.tss.avl);
+  BXRS_PARAM_BOOL(TR, granularity, tr.cache.u.system.g);
+  BXRS_PARAM_BOOL(TR, avl, tr.cache.u.system.avl);
 
-  BXRS_HEX_PARAM_SIMPLE(list, smbase);
+  BXRS_HEX_PARAM_SIMPLE(cpu, smbase);
 
 #if BX_CPU_LEVEL >= 5
-  bx_list_c *MSR = new bx_list_c(list, "msr", 12);
+  bx_list_c *MSR = new bx_list_c(cpu, "MSR", 45);
 
 #if BX_SUPPORT_APIC
   BXRS_HEX_PARAM_FIELD(MSR, apicbase, msr.apicbase);
@@ -503,11 +428,44 @@ void BX_CPU_C::register_state(void)
   BXRS_HEX_PARAM_FIELD(MSR, sysenter_esp_msr, msr.sysenter_esp_msr);
   BXRS_HEX_PARAM_FIELD(MSR, sysenter_eip_msr, msr.sysenter_eip_msr);
 #endif
+#if BX_SUPPORT_MTRR
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysbase0, msr.mtrrphys[0]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysmask0, msr.mtrrphys[1]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysbase1, msr.mtrrphys[2]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysmask1, msr.mtrrphys[3]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysbase2, msr.mtrrphys[4]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysmask2, msr.mtrrphys[5]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysbase3, msr.mtrrphys[6]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysmask3, msr.mtrrphys[7]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysbase4, msr.mtrrphys[8]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysmask4, msr.mtrrphys[9]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysbase5, msr.mtrrphys[10]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysmask5, msr.mtrrphys[11]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysbase6, msr.mtrrphys[12]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysmask6, msr.mtrrphys[13]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysbase7, msr.mtrrphys[14]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrphysmask7, msr.mtrrphys[15]);
 
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix64k_00000, msr.mtrrfix64k_00000);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix16k_80000, msr.mtrrfix16k_80000);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix16k_a0000, msr.mtrrfix16k_a0000);
+
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_c0000, msr.mtrrfix4k[0]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_c8000, msr.mtrrfix4k[1]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_d0000, msr.mtrrfix4k[2]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_d8000, msr.mtrrfix4k[3]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_e0000, msr.mtrrfix4k[4]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_e8000, msr.mtrrfix4k[5]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_f0000, msr.mtrrfix4k[6]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_f8000, msr.mtrrfix4k[7]);
+
+  BXRS_HEX_PARAM_FIELD(MSR, pat, msr.pat);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrr_deftype, msr.mtrr_deftype);
+#endif
 #endif
 
 #if BX_SUPPORT_FPU || BX_SUPPORT_MMX
-  bx_list_c *fpu = new bx_list_c(list, "FPU", 17);
+  bx_list_c *fpu = new bx_list_c(cpu, "FPU", 17);
   BXRS_HEX_PARAM_FIELD(fpu, cwd, the_i387.cwd);
   BXRS_HEX_PARAM_FIELD(fpu, swd, the_i387.swd);
   BXRS_HEX_PARAM_FIELD(fpu, twd, the_i387.twd);
@@ -526,28 +484,34 @@ void BX_CPU_C::register_state(void)
 #endif
 
 #if BX_SUPPORT_SSE
-  bx_list_c *sse = new bx_list_c(list, "SSE", 2*BX_XMM_REGISTERS+1);
+  bx_list_c *sse = new bx_list_c(cpu, "SSE", 2*BX_XMM_REGISTERS+1);
   BXRS_HEX_PARAM_FIELD(sse, mxcsr, mxcsr.mxcsr);
   for (i=0; i<BX_XMM_REGISTERS; i++) {
     sprintf(name, "xmm%02d_hi", i);
-    new bx_shadow_num_c(sse, name, &BX_CPU_THIS_PTR xmm[i].xmm64u(1), BASE_HEX);
+    new bx_shadow_num_c(sse, name, &xmm[i].xmm64u(1), BASE_HEX);
     sprintf(name, "xmm%02d_lo", i);
-    new bx_shadow_num_c(sse, name, &BX_CPU_THIS_PTR xmm[i].xmm64u(0), BASE_HEX);
+    new bx_shadow_num_c(sse, name, &xmm[i].xmm64u(0), BASE_HEX);
   }
 #endif
 
-#if BX_SUPPORT_APIC
-  local_apic.register_state(list);
+#if BX_SUPPORT_MONITOR_MWAIT
+  bx_list_c *monitor_list = new bx_list_c(cpu, "MONITOR", 2);
+  BXRS_HEX_PARAM_FIELD(monitor_list, begin_addr, monitor.monitor_begin);
+  BXRS_HEX_PARAM_FIELD(monitor_list, end_addr,   monitor.monitor_end);
 #endif
 
-  BXRS_PARAM_BOOL(list, EXT, EXT);
-  BXRS_PARAM_BOOL(list, async_event, async_event);
-  BXRS_PARAM_BOOL(list, INTR, INTR);
-  BXRS_PARAM_BOOL(list, smi_pending, smi_pending);
-  BXRS_PARAM_BOOL(list, nmi_pending, nmi_pending);
-  BXRS_PARAM_BOOL(list, in_smm, in_smm);
-  BXRS_PARAM_BOOL(list, nmi_disable, nmi_disable);
-  BXRS_PARAM_BOOL(list, trace, trace);
+#if BX_SUPPORT_APIC
+  local_apic.register_state(cpu);
+#endif
+
+  BXRS_PARAM_BOOL(cpu, EXT, EXT);
+  BXRS_PARAM_BOOL(cpu, async_event, async_event);
+  BXRS_PARAM_BOOL(cpu, INTR, INTR);
+  BXRS_PARAM_BOOL(cpu, smi_pending, smi_pending);
+  BXRS_PARAM_BOOL(cpu, nmi_pending, nmi_pending);
+  BXRS_PARAM_BOOL(cpu, in_smm, in_smm);
+  BXRS_PARAM_BOOL(cpu, nmi_disable, nmi_disable);
+  BXRS_PARAM_BOOL(cpu, trace, trace);
 }
 
 Bit64s BX_CPU_C::param_save_handler(void *devptr, bx_param_c *param, Bit64s val)
@@ -641,14 +605,14 @@ Bit64s BX_CPU_C::param_restore(bx_param_c *param, Bit64s val)
       BX_PANIC(("save/restore: CPUID mismatch"));
     }
   } else if (!strcmp(pname, "EFLAGS")) {
-    BX_CPU_THIS_PTR setEFlags(val);
+    BX_CPU_THIS_PTR setEFlags((Bit32u)val);
 #if BX_SUPPORT_X86_64
   } else if (!strcmp(pname, "EFER")) {
-    BX_CPU_THIS_PTR msr.sce   = (val >> 0)  & 1;
-    BX_CPU_THIS_PTR msr.lme   = (val >> 8)  & 1;
-    BX_CPU_THIS_PTR msr.lma   = (val >> 10) & 1;
-    BX_CPU_THIS_PTR msr.nxe   = (val >> 11) & 1;
-    BX_CPU_THIS_PTR msr.ffxsr = (val >> 14) & 1;
+    BX_CPU_THIS_PTR efer.sce   = (bx_bool)((val >> 0)  & 1);
+    BX_CPU_THIS_PTR efer.lme   = (bx_bool)((val >> 8)  & 1);
+    BX_CPU_THIS_PTR efer.lma   = (bx_bool)((val >> 10) & 1);
+    BX_CPU_THIS_PTR efer.nxe   = (bx_bool)((val >> 11) & 1);
+    BX_CPU_THIS_PTR efer.ffxsr = (bx_bool)((val >> 14) & 1);
 #endif
   } else if (!strcmp(pname, "ar_byte") || !strcmp(pname, "selector")) {
     segname = param->get_parent()->get_name();
@@ -673,10 +637,10 @@ Bit64s BX_CPU_C::param_restore(bx_param_c *param, Bit64s val)
       bx_descriptor_t *d = &(segment->cache);
       bx_selector_t *selector = &(segment->selector);
       if (!strcmp(pname, "ar_byte")) {
-        set_ar_byte(d, val);
+        set_ar_byte(d, (Bit8u)val);
       }
       else if (!strcmp(pname, "selector")) {
-        parse_selector(val, selector);
+        parse_selector((Bit16u)val, selector);
         // validate the selector
         if ((selector->value & 0xfffc) != 0) d->valid = 1;
         else d->valid = 0;
@@ -698,7 +662,7 @@ void BX_CPU_C::after_restore_state(void)
   invalidate_prefetch_q();
   debug(RIP);
 }
-#endif
+// end of save/restore functionality
 
 BX_CPU_C::~BX_CPU_C()
 {
@@ -708,7 +672,14 @@ BX_CPU_C::~BX_CPU_C()
 
 void BX_CPU_C::reset(unsigned source)
 {
-  UNUSED(source); // either BX_RESET_HARDWARE or BX_RESET_SOFTWARE
+  unsigned i;
+
+  if (source == BX_RESET_HARDWARE)
+    BX_INFO(("cpu hardware reset"));
+  else if (source == BX_RESET_SOFTWARE)
+    BX_INFO(("cpu software reset"));
+  else
+    BX_INFO(("cpu reset"));
 
 #if BX_SUPPORT_X86_64
   RAX = 0; // processor passed test :-)
@@ -739,33 +710,17 @@ void BX_CPU_C::reset(unsigned source)
   ESP = 0;
 #endif
 
-  // all status flags at known values, use BX_CPU_THIS_PTR eflags structure
-  BX_CPU_THIS_PTR lf_flags_status = 0x000000;
-
   // status and control flags register set
   BX_CPU_THIS_PTR setEFlags(0x2); // Bit1 is always set
-  BX_CPU_THIS_PTR clear_IF ();
-#if BX_CPU_LEVEL >= 3
-  BX_CPU_THIS_PTR clear_RF ();
-  BX_CPU_THIS_PTR clear_VM ();
-#endif
-#if BX_CPU_LEVEL >= 4
-  BX_CPU_THIS_PTR clear_AC ();
-#endif
 
   BX_CPU_THIS_PTR inhibit_mask = 0;
   BX_CPU_THIS_PTR debug_trap = 0;
 
   /* instruction pointer */
 #if BX_CPU_LEVEL < 2
-  BX_CPU_THIS_PTR prev_eip = EIP = 0x00000000;
+  BX_CPU_THIS_PTR prev_rip = EIP = 0x00000000;
 #else /* from 286 up */
-  BX_CPU_THIS_PTR prev_eip =
-#if BX_SUPPORT_X86_64
-  RIP = 0x0000FFF0;
-#else
-  EIP = 0x0000FFF0;
-#endif
+  BX_CPU_THIS_PTR prev_rip = RIP = 0x0000FFF0;
 #endif
 
   /* CS (Code Segment) and descriptor cache */
@@ -859,12 +814,12 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR ldtr.cache.dpl      = 0; /* field not used */
   BX_CPU_THIS_PTR ldtr.cache.segment  = 0; /* system segment */
   BX_CPU_THIS_PTR ldtr.cache.type     = BX_SYS_SEGMENT_LDT;
-  BX_CPU_THIS_PTR ldtr.cache.u.ldt.base       = 0x00000000;
-  BX_CPU_THIS_PTR ldtr.cache.u.ldt.limit      =     0xFFFF;
+  BX_CPU_THIS_PTR ldtr.cache.u.system.base       = 0x00000000;
+  BX_CPU_THIS_PTR ldtr.cache.u.system.limit      =     0xFFFF;
 #if BX_CPU_LEVEL >= 3
-  BX_CPU_THIS_PTR tr.cache.u.ldt.limit_scaled =     0xFFFF;
-  BX_CPU_THIS_PTR tr.cache.u.ldt.avl = 0;
-  BX_CPU_THIS_PTR tr.cache.u.ldt.g   = 0;  /* byte granular */
+  BX_CPU_THIS_PTR ldtr.cache.u.system.limit_scaled =   0xFFFF;
+  BX_CPU_THIS_PTR ldtr.cache.u.system.avl = 0;
+  BX_CPU_THIS_PTR ldtr.cache.u.system.g   = 0;  /* byte granular */
 #endif
 
   /* TR (Task Register) */
@@ -878,12 +833,12 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR tr.cache.dpl      = 0; /* field not used */
   BX_CPU_THIS_PTR tr.cache.segment  = 0; /* system segment */
   BX_CPU_THIS_PTR tr.cache.type     = BX_SYS_SEGMENT_BUSY_286_TSS;
-  BX_CPU_THIS_PTR tr.cache.u.tss.base         = 0x00000000;
-  BX_CPU_THIS_PTR tr.cache.u.tss.limit        =     0xFFFF;
+  BX_CPU_THIS_PTR tr.cache.u.system.base         = 0x00000000;
+  BX_CPU_THIS_PTR tr.cache.u.system.limit        =     0xFFFF;
 #if BX_CPU_LEVEL >= 3
-  BX_CPU_THIS_PTR tr.cache.u.tss.limit_scaled =     0xFFFF;
-  BX_CPU_THIS_PTR tr.cache.u.tss.avl = 0;
-  BX_CPU_THIS_PTR tr.cache.u.tss.g   = 0;  /* byte granular */
+  BX_CPU_THIS_PTR tr.cache.u.system.limit_scaled =     0xFFFF;
+  BX_CPU_THIS_PTR tr.cache.u.system.avl = 0;
+  BX_CPU_THIS_PTR tr.cache.u.system.g   = 0;  /* byte granular */
 #endif
 #endif
 
@@ -914,31 +869,13 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR nmi_pending = 0;
   BX_CPU_THIS_PTR in_smm = 0;
   BX_CPU_THIS_PTR nmi_disable = 0;
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+  BX_CPU_THIS_PTR alignment_check = 0;
+#endif
 
   BX_CPU_THIS_PTR smbase = 0x30000;
 
-#if BX_CPU_LEVEL >= 2
-  // MSW (Machine Status Word), so called on 286
-  // CR0 (Control Register 0), so called on 386+
-  BX_CPU_THIS_PTR cr0.ts = 0; // no task switch
-  BX_CPU_THIS_PTR cr0.em = 0; // emulate math coprocessor
-  BX_CPU_THIS_PTR cr0.mp = 0; // wait instructions not trapped
-  BX_CPU_THIS_PTR cr0.pe = 0; // real mode
-  BX_CPU_THIS_PTR cr0.val32 = 0;
-
-#if BX_CPU_LEVEL >= 3
-  BX_CPU_THIS_PTR cr0.pg = 0; // paging disabled
-  // no change to cr0.val32
-#endif
-
-#if BX_CPU_LEVEL >= 4
-  BX_CPU_THIS_PTR cr0.cd = 0;
-  BX_CPU_THIS_PTR cr0.nw = 0;
-  BX_CPU_THIS_PTR cr0.am = 0; // disable alignment check
-  BX_CPU_THIS_PTR cr0.wp = 0; // disable write-protect
-  BX_CPU_THIS_PTR cr0.ne = 0; // np exceptions through int 13H, DOS compat
-  BX_CPU_THIS_PTR cr0.val32 |= 0x00000000;
-#endif
+  BX_CPU_THIS_PTR cr0.setRegister(0);
 
   // handle reserved bits
 #if BX_CPU_LEVEL == 3
@@ -948,10 +885,6 @@ void BX_CPU_C::reset(unsigned source)
   // bit 4 is hardwired to 1 on all x86
   BX_CPU_THIS_PTR cr0.val32 |= 0x00000010;
 #endif
-#endif // CPU >= 2
-
-  // CR0 paging might be modified
-  TLB_flush(1);
 
 #if BX_CPU_LEVEL >= 3
   BX_CPU_THIS_PTR cr1 = 0;
@@ -959,10 +892,14 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR cr3 = 0;
   BX_CPU_THIS_PTR cr3_masked = 0;
 #endif
+
 #if BX_CPU_LEVEL >= 4
   BX_CPU_THIS_PTR cr4.setRegister(0);
 #endif
 
+  // CR0/CR4 paging might be modified
+  TLB_flush(1);
+
 /* initialise MSR registers to defaults */
 #if BX_CPU_LEVEL >= 5
 #if BX_SUPPORT_APIC
@@ -972,9 +909,10 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR msr.apicbase |= 0x900;
 #endif
 #if BX_SUPPORT_X86_64
-  BX_CPU_THIS_PTR msr.lme = BX_CPU_THIS_PTR msr.lma = 0;
-  BX_CPU_THIS_PTR msr.sce = BX_CPU_THIS_PTR msr.nxe = 0;
-  BX_CPU_THIS_PTR msr.ffxsr = 0;
+  BX_CPU_THIS_PTR efer.lme = BX_CPU_THIS_PTR efer.lma = 0;
+  BX_CPU_THIS_PTR efer.sce = BX_CPU_THIS_PTR efer.nxe = 0;
+  BX_CPU_THIS_PTR efer.ffxsr = 0;
+
   BX_CPU_THIS_PTR msr.star  = 0;
   BX_CPU_THIS_PTR msr.lstar = 0;
   BX_CPU_THIS_PTR msr.cstar = 0;
@@ -991,13 +929,26 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR msr.sysenter_eip_msr = 0;
 #endif
 
+#if BX_SUPPORT_MTRR
+  for (i=0; i<16; i++)
+    BX_CPU_THIS_PTR msr.mtrrphys[i] = 0;
+
+  BX_CPU_THIS_PTR msr.mtrrfix64k_00000 = 0; // all fix range MTRRs undefined according to manual
+  BX_CPU_THIS_PTR msr.mtrrfix16k_80000 = 0;
+  BX_CPU_THIS_PTR msr.mtrrfix16k_a0000 = 0;
+
+  for (i=0; i<8; i++)
+    BX_CPU_THIS_PTR msr.mtrrfix4k[i] = 0;
+
+  BX_CPU_THIS_PTR msr.pat = BX_CONST64(0x0007040600070406);
+  BX_CPU_THIS_PTR msr.mtrr_deftype = 0;
+#endif
+
   BX_CPU_THIS_PTR EXT = 0;
 
-#if BX_SUPPORT_PAGING
 #if BX_USE_TLB
   TLB_init();
-#endif // BX_USE_TLB
-#endif // BX_SUPPORT_PAGING
+#endif
 
   // invalidate the prefetch queue
   BX_CPU_THIS_PTR eipPageBias = 0;
@@ -1022,10 +973,10 @@ void BX_CPU_C::reset(unsigned source)
 
   // Reset XMM state
 #if BX_SUPPORT_SSE >= 1  // unchanged on #INIT
-  for(unsigned index=0; index < BX_XMM_REGISTERS; index++)
+  for(i=0; i<BX_XMM_REGISTERS; i++)
   {
-    BX_CPU_THIS_PTR xmm[index].xmm64u(0) = 0;
-    BX_CPU_THIS_PTR xmm[index].xmm64u(1) = 0;
+    BX_CPU_THIS_PTR xmm[i].xmm64u(0) = 0;
+    BX_CPU_THIS_PTR xmm[i].xmm64u(1) = 0;
   }
 
   BX_CPU_THIS_PTR mxcsr.mxcsr = MXCSR_RESET;
@@ -1043,11 +994,14 @@ void BX_CPU_C::reset(unsigned source)
     // it's an application processor, halt until IPI is heard.
     BX_CPU_THIS_PTR msr.apicbase &= ~0x0100; /* clear bit 8 BSP */
     BX_INFO(("CPU[%d] is an application processor. Halting until IPI.", apic_id));
-    debug_trap |= BX_DEBUG_TRAP_HALT_STATE;
+    debug_trap |= BX_DEBUG_TRAP_WAIT_FOR_SIPI;
     async_event = 1;
   }
 #endif
 
+  // initialize CPUID values - make sure apicbase already initialized
+  set_cpuid_defaults();
+
   BX_INSTR_RESET(BX_CPU_ID);
 }
 
@@ -1133,8 +1087,8 @@ void BX_CPU_C::assert_checks(void)
 {
   // check CPU mode consistency
 #if BX_SUPPORT_X86_64
-  if (BX_CPU_THIS_PTR msr.lma) {
-    if (! BX_CPU_THIS_PTR cr0.pe) {
+  if (BX_CPU_THIS_PTR efer.lma) {
+    if (! BX_CPU_THIS_PTR cr0.get_PE()) {
       BX_PANIC(("assert_checks: EFER.LMA is set when CR0.PE=0 !"));
     }
     if (BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.l) {
@@ -1149,7 +1103,7 @@ void BX_CPU_C::assert_checks(void)
   else 
 #endif
   {
-    if (BX_CPU_THIS_PTR cr0.pe) {
+    if (BX_CPU_THIS_PTR cr0.get_PE()) {
       if (BX_CPU_THIS_PTR get_VM()) {
         if (BX_CPU_THIS_PTR cpu_mode != BX_MODE_IA32_V8086)
           BX_PANIC(("assert_checks: unconsistent cpu_mode BX_MODE_IA32_V8086 !"));
@@ -1165,9 +1119,18 @@ void BX_CPU_C::assert_checks(void)
     }
   }
 
+  // check CR0 consistency
+  if (BX_CPU_THIS_PTR cr0.get_PG() && ! BX_CPU_THIS_PTR cr0.get_PE())
+    BX_PANIC(("assert_checks: CR0.PG=1 with CR0.PE=0 !"));
+#if BX_CPU_LEVEL >= 4
+  if (BX_CPU_THIS_PTR cr0.get_NW() && ! BX_CPU_THIS_PTR cr0.get_CD())
+    BX_PANIC(("assert_checks: CR0.NW=1 with CR0.CD=0 !"));
+#endif
+
+
 #if BX_SUPPORT_X86_64
   // VM should be OFF in long mode
-  if (IsLongMode()) {
+  if (long_mode()) {
     if (BX_CPU_THIS_PTR get_VM()) BX_PANIC(("assert_checks: VM is set in long mode !"));
   }
 
@@ -1196,9 +1159,9 @@ void BX_CPU_C::assert_checks(void)
       case BX_SYS_SEGMENT_BUSY_286_TSS:
       case BX_SYS_SEGMENT_AVAIL_286_TSS:
 #if BX_CPU_LEVEL >= 3
-        if (BX_CPU_THIS_PTR tr.cache.u.tss.g != 0)
+        if (BX_CPU_THIS_PTR tr.cache.u.system.g != 0)
           BX_PANIC(("assert_checks: tss286.g != 0 !"));
-        if (BX_CPU_THIS_PTR tr.cache.u.tss.avl != 0)
+        if (BX_CPU_THIS_PTR tr.cache.u.system.avl != 0)
           BX_PANIC(("assert_checks: tss286.avl != 0 !"));
 #endif
         break;
@@ -1210,35 +1173,9 @@ void BX_CPU_C::assert_checks(void)
     }
   }
 
-  // validate CR0 register
-  if (BX_CPU_THIS_PTR cr0.pe != (BX_CPU_THIS_PTR cr0.val32 & 1))
-    BX_PANIC(("assert_checks: inconsistent CR0.PE !"));
-  if (BX_CPU_THIS_PTR cr0.mp != ((BX_CPU_THIS_PTR cr0.val32 >>  1) & 1))
-    BX_PANIC(("assert_checks: inconsistent CR0.MP !"));
-  if (BX_CPU_THIS_PTR cr0.em != ((BX_CPU_THIS_PTR cr0.val32 >>  2) & 1))
-    BX_PANIC(("assert_checks: inconsistent CR0.EM !"));
-  if (BX_CPU_THIS_PTR cr0.ts != ((BX_CPU_THIS_PTR cr0.val32 >>  3) & 1))
-    BX_PANIC(("assert_checks: inconsistent CR0.TS !"));
-#if BX_CPU_LEVEL >= 4
-  if (BX_CPU_THIS_PTR cr0.ne != ((BX_CPU_THIS_PTR cr0.val32 >>  5) & 1))
-    BX_PANIC(("assert_checks: inconsistent CR0.NE !"));
-  if (BX_CPU_THIS_PTR cr0.wp != ((BX_CPU_THIS_PTR cr0.val32 >> 16) & 1))
-    BX_PANIC(("assert_checks: inconsistent CR0.WP !"));
-  if (BX_CPU_THIS_PTR cr0.am != ((BX_CPU_THIS_PTR cr0.val32 >> 18) & 1))
-    BX_PANIC(("assert_checks: inconsistent CR0.AM !"));
-  if (BX_CPU_THIS_PTR cr0.nw != ((BX_CPU_THIS_PTR cr0.val32 >> 29) & 1))
-    BX_PANIC(("assert_checks: inconsistent CR0.NW !"));
-  if (BX_CPU_THIS_PTR cr0.cd != ((BX_CPU_THIS_PTR cr0.val32 >> 30) & 1))
-    BX_PANIC(("assert_checks: inconsistent CR0.CD !"));
-#endif
-  if (BX_CPU_THIS_PTR cr0.pg != ((BX_CPU_THIS_PTR cr0.val32 >> 31) & 1))
-    BX_PANIC(("assert_checks: inconsistent CR0.PG !"));
-
-  if (BX_CPU_THIS_PTR cr0.pg && ! BX_CPU_THIS_PTR cr0.pe)
-    BX_PANIC(("assert_checks: CR0.PG=1 with CR0.PE=0 !"));
-#if BX_CPU_LEVEL >= 4
-  if (BX_CPU_THIS_PTR cr0.nw && ! BX_CPU_THIS_PTR cr0.cd)
-    BX_PANIC(("assert_checks: CR0.NW=1 with CR0.CD=0 !"));
+#if BX_SUPPORT_MONITOR_MWAIT
+  if (BX_CPU_THIS_PTR monitor.monitor_end < BX_CPU_THIS_PTR monitor.monitor_begin)
+    BX_PANIC(("assert_checks: MONITOR range is not set correctly !"));
 #endif
 }
 
