@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: io_pro.cc,v 1.17 2005/06/22 18:13:45 sshwarts Exp $
+// $Id: io_pro.cc,v 1.26 2007/12/17 21:13:55 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -27,38 +27,29 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
-#include "iodev/iodev.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#include "iodev/iodev.h"
 
   Bit16u BX_CPP_AttrRegparmN(1)
 BX_CPU_C::inp16(Bit16u addr)
 {
-  Bit16u ret16;
-
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM () || (CPL>BX_CPU_THIS_PTR get_IOPL ()))) {
-    if ( !BX_CPU_THIS_PTR allow_io(addr, 2) ) {
-      exception(BX_GP_EXCEPTION, 0, 0);
-      return(0);
-    }
+  if (! BX_CPU_THIS_PTR allow_io(addr, 2)) {
+    BX_DEBUG(("inp16(): I/O access not allowed !"));
+    exception(BX_GP_EXCEPTION, 0, 0);
   }
 
-  ret16 = BX_INP(addr, 2);
-  return( ret16 );
+  Bit16u ret16 = BX_INP(addr, 2);
+  return ret16;
 }
 
   void BX_CPP_AttrRegparmN(2)
 BX_CPU_C::outp16(Bit16u addr, Bit16u value)
 {
-  /* If CPL <= IOPL, then all IO addresses are accessible.
-   * Otherwise, must check the IO permission map on >286.
-   * On the 286, there is no IO permissions map */
-
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
-    if ( !BX_CPU_THIS_PTR allow_io(addr, 2) ) {
-      exception(BX_GP_EXCEPTION, 0, 0);
-      return;
-    }
+  if (! BX_CPU_THIS_PTR allow_io(addr, 2)) {
+    BX_DEBUG(("outp16(): I/O access not allowed !"));
+    exception(BX_GP_EXCEPTION, 0, 0);
   }
 
   BX_OUTP(addr, value, 2);
@@ -67,31 +58,21 @@ BX_CPU_C::outp16(Bit16u addr, Bit16u value)
   Bit32u BX_CPP_AttrRegparmN(1)
 BX_CPU_C::inp32(Bit16u addr)
 {
-  Bit32u ret32;
-
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
-    if ( !BX_CPU_THIS_PTR allow_io(addr, 4) ) {
-      exception(BX_GP_EXCEPTION, 0, 0);
-      return(0);
-    }
+  if (! BX_CPU_THIS_PTR allow_io(addr, 4)) {
+    BX_DEBUG(("inp32(): I/O access not allowed !"));
+    exception(BX_GP_EXCEPTION, 0, 0);
   }
 
-  ret32 = BX_INP(addr, 4);
-  return( ret32 );
+  Bit32u ret32 = BX_INP(addr, 4);
+  return ret32;
 }
 
   void BX_CPP_AttrRegparmN(2)
 BX_CPU_C::outp32(Bit16u addr, Bit32u value)
 {
-  /* If CPL <= IOPL, then all IO addresses are accessible.
-   * Otherwise, must check the IO permission map on >286.
-   * On the 286, there is no IO permissions map */
-
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
-    if ( !BX_CPU_THIS_PTR allow_io(addr, 4) ) {
-      exception(BX_GP_EXCEPTION, 0, 0);
-      return;
-    }
+  if (! BX_CPU_THIS_PTR allow_io(addr, 4)) {
+    BX_DEBUG(("outp32(): I/O access not allowed !"));
+    exception(BX_GP_EXCEPTION, 0, 0);
   }
 
   BX_OUTP(addr, value, 4);
@@ -100,32 +81,21 @@ BX_CPU_C::outp32(Bit16u addr, Bit32u value)
   Bit8u BX_CPP_AttrRegparmN(1)
 BX_CPU_C::inp8(Bit16u addr)
 {
-  Bit8u ret8;
-
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
-    if ( !BX_CPU_THIS_PTR allow_io(addr, 1) ) {
-      exception(BX_GP_EXCEPTION, 0, 0);
-      return(0);
-    }
+  if (! BX_CPU_THIS_PTR allow_io(addr, 1)) {
+    BX_DEBUG(("inp8(): I/O access not allowed !"));
+    exception(BX_GP_EXCEPTION, 0, 0);
   }
 
-  ret8 = BX_INP(addr, 1);
-  return( ret8 );
+  Bit8u ret8 = BX_INP(addr, 1);
+  return ret8;
 }
 
-
   void BX_CPP_AttrRegparmN(2)
 BX_CPU_C::outp8(Bit16u addr, Bit8u value)
 {
-  /* If CPL <= IOPL, then all IO addresses are accessible.
-   * Otherwise, must check the IO permission map on >286.
-   * On the 286, there is no IO permissions map */
-
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
-    if ( !BX_CPU_THIS_PTR allow_io(addr, 1) ) {
-      exception(BX_GP_EXCEPTION, 0, 0);
-      return;
-    }
+  if (! BX_CPU_THIS_PTR allow_io(addr, 1)) {
+    BX_DEBUG(("outp8(): I/O access not allowed !"));
+    exception(BX_GP_EXCEPTION, 0, 0);
   }
 
   BX_OUTP(addr, value, 1);
@@ -134,43 +104,49 @@ BX_CPU_C::outp8(Bit16u addr, Bit8u value)
 bx_bool BX_CPU_C::allow_io(Bit16u addr, unsigned len)
 {
   Bit16u io_base, permission16;
-  unsigned bit_index, i;
 
-  if (BX_CPU_THIS_PTR tr.cache.valid==0 || 
-      BX_CPU_THIS_PTR tr.cache.type != BX_SYS_SEGMENT_AVAIL_386_TSS)
+  /* If CPL <= IOPL, then all IO addresses are accessible.
+   * Otherwise, must check the IO permission map on >286.
+   * On the 286, there is no IO permissions map */
+
+  if (BX_CPU_THIS_PTR cr0.get_PE() && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL())))
   {
-    BX_ERROR(("allow_io(): TR doesn't point to a valid 32bit TSS"));
-    return(0);
-  }
+    if (BX_CPU_THIS_PTR tr.cache.valid==0 || 
+        BX_CPU_THIS_PTR tr.cache.type != BX_SYS_SEGMENT_AVAIL_386_TSS)
+    {
+      BX_ERROR(("allow_io(): TR doesn't point to a valid 32bit TSS"));
+      return(0);
+    }
 
-  if (BX_CPU_THIS_PTR tr.cache.u.tss386.limit_scaled < 103) {
-    BX_ERROR(("allow_io(): TR.limit < 103"));
-    return(0);
-  }
+    if (BX_CPU_THIS_PTR tr.cache.u.system.limit_scaled < 103) {
+      BX_ERROR(("allow_io(): TR.limit < 103"));
+      return(0);
+    }
 
-  access_linear(BX_CPU_THIS_PTR tr.cache.u.tss386.base + 102, 
+    access_linear(BX_CPU_THIS_PTR tr.cache.u.system.base + 102, 
                    2, 0, BX_READ, &io_base);
 
-  if (io_base <= 103) {
-     BX_PANIC(("allow_io(): TR:io_base (%u) <= 103",io_base));
-     return(0);
-  }
+    if (io_base <= 103) {
+      BX_ERROR(("allow_io(): TR:io_base (%u) <= 103", io_base));
+      return(0);
+    }
 
-  if ( (Bit32s) (addr/8) >= (Bit32s) (BX_CPU_THIS_PTR tr.cache.u.tss386.limit_scaled - io_base)) {
-    BX_INFO(("allow_io(): IO addr %x (len %d) outside TSS IO permission map (base=%x, limit=%x) #GP(0)",
-      addr, len, io_base, BX_CPU_THIS_PTR tr.cache.u.tss386.limit_scaled));
-    return(0);
-  }
+    if ( (Bit32s) (addr/8) >= (Bit32s) (BX_CPU_THIS_PTR tr.cache.u.system.limit_scaled - io_base)) {
+      BX_ERROR(("allow_io(): IO addr %x (len %d) outside TSS IO permission map (base=%x, limit=%x) #GP(0)",
+        addr, len, io_base, BX_CPU_THIS_PTR tr.cache.u.system.limit_scaled));
+      return(0);
+    }
 
-  access_linear(BX_CPU_THIS_PTR tr.cache.u.tss386.base + io_base + addr/8,
+    access_linear(BX_CPU_THIS_PTR tr.cache.u.system.base + io_base + addr/8,
                    2, 0, BX_READ, &permission16);
 
-  bit_index = addr & 0x07;
-  permission16 >>= bit_index;
-  for (i=0; i<len; i++) {
-    if (permission16 & 0x01)
-      return(0);
-    permission16 >>= 1;
+    unsigned bit_index = addr & 0x07;
+    permission16 >>= bit_index;
+    for (unsigned i=0; i<len; i++) {
+      if (permission16 & 0x01)
+        return(0);
+      permission16 >>= 1;
+    }
   }
 
   return(1);
