@@ -1,14 +1,9 @@
 ////////////////////////////////////////////////////////////////////////
-// $Id: iret.cc,v 1.15 2006/06/12 16:58:27 sshwarts Exp $
+// $Id: iret.cc,v 1.25 2007/12/23 17:21:27 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//   Copyright (c) 2005 Stanislav Shwartsman
+//          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,7 +19,7 @@
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 //
-
+/////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
@@ -36,7 +31,6 @@
 #define RIP EIP
 #endif
 
-
   void BX_CPP_AttrRegparmN(1)
 BX_CPU_C::iret_protected(bxInstruction_c *i)
 {
@@ -45,17 +39,14 @@ BX_CPU_C::iret_protected(bxInstruction_c *i)
   Bit32u dword1, dword2;
   bx_descriptor_t cs_descriptor, ss_descriptor;
 
-  BX_DEBUG(("IRET PROTECTED"));
-
 #if BX_SUPPORT_X86_64
-  if (BX_CPU_THIS_PTR msr.lma)
-  {
+  if (long_mode()) {
     long_iret(i);
     return;
   }
 #endif
 
-  if (BX_CPU_THIS_PTR get_NT ())   /* NT = 1: RETURN FROM NESTED TASK */
+  if (BX_CPU_THIS_PTR get_NT())   /* NT = 1: RETURN FROM NESTED TASK */
   {
     /* what's the deal with NT & VM ? */
     Bit16u raw_link_selector;
@@ -69,7 +60,7 @@ BX_CPU_C::iret_protected(bxInstruction_c *i)
 
     if (BX_CPU_THIS_PTR tr.cache.valid==0)
       BX_PANIC(("IRET: TR not valid"));
-    Bit32u base32 = BX_CPU_THIS_PTR tr.cache.u.tss.base;
+    Bit32u base32 = (Bit32u) BX_CPU_THIS_PTR tr.cache.u.system.base;
 
     // examine back link selector in TSS addressed by current TR:
     access_linear(base32, 2, 0, BX_READ, &raw_link_selector);
@@ -120,8 +111,8 @@ BX_CPU_C::iret_protected(bxInstruction_c *i)
 
   /* NT = 0: INTERRUPT RETURN ON STACK -or STACK_RETURN_TO_V86 */
   unsigned top_nbytes_same, top_nbytes_outer;
-  Bit32u new_eip, new_esp, temp_ESP, new_eflags;
-  Bit16u new_ip, new_flags;
+  Bit32u new_eip = 0, new_esp, temp_ESP, new_eflags = 0;
+  Bit16u new_ip = 0, new_flags = 0;
   Bit32u ss_offset;
 
   /* 16bit opsize  |   32bit opsize
@@ -157,12 +148,9 @@ BX_CPU_C::iret_protected(bxInstruction_c *i)
     temp_ESP = SP;
 
   if (i->os32L()) {
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 4,
-      2, CPL==3, BX_READ, &raw_cs_selector);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 0,
-      4, CPL==3, BX_READ, &new_eip);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 8,
-      4, CPL==3, BX_READ, &new_eflags);
+    raw_cs_selector = read_virtual_word (BX_SEG_REG_SS, temp_ESP + 4);
+    new_eip         = read_virtual_dword(BX_SEG_REG_SS, temp_ESP + 0);
+    new_eflags      = read_virtual_dword(BX_SEG_REG_SS, temp_ESP + 8);
 
     // if VM=1 in flags image on stack then STACK_RETURN_TO_V86
     if (new_eflags & EFlagsVMMask) {
@@ -174,18 +162,15 @@ BX_CPU_C::iret_protected(bxInstruction_c *i)
     }
   }
   else {
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 2,
-      2, CPL==3, BX_READ, &raw_cs_selector);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 0,
-      2, CPL==3, BX_READ, &new_ip);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 4,
-      2, CPL==3, BX_READ, &new_flags);
+    raw_cs_selector = read_virtual_word(BX_SEG_REG_SS, temp_ESP + 2);
+    new_ip          = read_virtual_word(BX_SEG_REG_SS, temp_ESP + 0);
+    new_flags       = read_virtual_word(BX_SEG_REG_SS, temp_ESP + 4);
   }
 
   parse_selector(raw_cs_selector, &cs_selector);
 
   // return CS selector must be non-null, else #GP(0)
-  if ( (raw_cs_selector & 0xfffc) == 0 ) {
+  if ((raw_cs_selector & 0xfffc) == 0) {
     BX_ERROR(("iret: return CS selector null"));
     exception(BX_GP_EXCEPTION, 0, 0);
   }
@@ -232,7 +217,7 @@ BX_CPU_C::iret_protected(bxInstruction_c *i)
       branch_far32(&cs_selector, &cs_descriptor, (Bit32u) new_ip, cs_selector.rpl);
 
       /* load flags with third word on stack */
-      write_flags(new_flags, CPL==0, CPL<=BX_CPU_THIS_PTR get_IOPL ());
+      write_flags(new_flags, CPL==0, CPL<=BX_CPU_THIS_PTR get_IOPL());
     }
 
     /* increment stack by 6/12 */
@@ -259,8 +244,7 @@ BX_CPU_C::iret_protected(bxInstruction_c *i)
     }
 
     /* examine return SS selector and associated descriptor */
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + ss_offset,
-      2, 0, BX_READ, &raw_ss_selector);
+    raw_ss_selector = read_virtual_word(BX_SEG_REG_SS, temp_ESP + ss_offset);
 
     /* selector must be non-null, else #GP(0) */
     if ( (raw_ss_selector & 0xfffc) == 0 ) {
@@ -289,7 +273,7 @@ BX_CPU_C::iret_protected(bxInstruction_c *i)
          IS_CODE_SEGMENT(ss_descriptor.type) ||
         !IS_DATA_SEGMENT_WRITEABLE(ss_descriptor.type))
     {
-      BX_ERROR(("iret: SS AR byte not writable code segment"));
+      BX_ERROR(("iret: SS AR byte not writable or code segment"));
       exception(BX_GP_EXCEPTION, raw_ss_selector & 0xfffc, 0);
     }
 
@@ -307,24 +291,14 @@ BX_CPU_C::iret_protected(bxInstruction_c *i)
     }
 
     if (i->os32L()) {
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 0,
-        4, 0, BX_READ, &new_eip);
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 8,
-        4, 0, BX_READ, &new_eflags);
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 12,
-        4, 0, BX_READ, &new_esp);
+      new_eip    = read_virtual_dword(BX_SEG_REG_SS, temp_ESP +  0);
+      new_eflags = read_virtual_dword(BX_SEG_REG_SS, temp_ESP +  8);
+      new_esp    = read_virtual_dword(BX_SEG_REG_SS, temp_ESP + 12);
     }
     else {
-      Bit16u new_sp = 0;
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 0,
-        2, 0, BX_READ, &new_ip);
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 4,
-        2, 0, BX_READ, &new_flags);
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_ESP + 6,
-        2, 0, BX_READ, &new_sp);
-      new_eip = new_ip;
-      new_esp = new_sp;
-      new_eflags = new_flags;
+      new_eip    = read_virtual_word(BX_SEG_REG_SS, temp_ESP + 0);
+      new_eflags = read_virtual_word(BX_SEG_REG_SS, temp_ESP + 4);
+      new_esp    = read_virtual_word(BX_SEG_REG_SS, temp_ESP + 6);
     }
 
     Bit8u prev_cpl = CPL; /* previous CPL */
@@ -403,14 +377,10 @@ BX_CPU_C::long_iret(bxInstruction_c *i)
   unsigned top_nbytes_same = 0; /* stop compiler warnings */
 
   if (i->os64L()) {
-    Bit64u new_rflags = 0;
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 8,
-      2, CPL==3, BX_READ, &raw_cs_selector);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 0,
-      8, CPL==3, BX_READ, &new_rip);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 16,
-      8, CPL==3, BX_READ, &new_rflags);
-    new_eflags = (Bit32u) new_rflags;
+    raw_cs_selector = read_virtual_word (BX_SEG_REG_SS, temp_RSP +  8);
+    new_rip         = read_virtual_qword(BX_SEG_REG_SS, temp_RSP +  0);
+    new_eflags      = (Bit32u) read_virtual_qword(BX_SEG_REG_SS, temp_RSP + 16);
+
     top_nbytes_outer = 40;
     ss_offset = 32;
   }
@@ -421,14 +391,10 @@ BX_CPU_C::long_iret(bxInstruction_c *i)
       exception(BX_SS_EXCEPTION, 0, 0);
     }
 
-    Bit32u return_EIP = 0;
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 4,
-      2, CPL==3, BX_READ, &raw_cs_selector);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 0,
-      4, CPL==3, BX_READ, &return_EIP);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 8,
-      4, CPL==3, BX_READ, &new_eflags);
-    new_rip = return_EIP;
+    raw_cs_selector = read_virtual_word (BX_SEG_REG_SS, temp_RSP + 4);
+    new_rip         = (Bit64u) read_virtual_dword(BX_SEG_REG_SS, temp_RSP + 0);
+    new_eflags      = read_virtual_dword(BX_SEG_REG_SS, temp_RSP + 8);
+
     top_nbytes_outer = 20;
     top_nbytes_same = 12;
     ss_offset = 16;
@@ -440,15 +406,10 @@ BX_CPU_C::long_iret(bxInstruction_c *i)
       exception(BX_SS_EXCEPTION, 0, 0);
     }
 
-    Bit16u return_IP = 0, new_flags = 0;
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 2,
-      2, CPL==3, BX_READ, &raw_cs_selector);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 0,
-      2, CPL==3, BX_READ, &return_IP);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 4,
-      2, CPL==3, BX_READ, &new_flags);
-    new_rip = return_IP;
-    new_eflags = (Bit32u) new_flags;
+    raw_cs_selector  = read_virtual_word(BX_SEG_REG_SS, temp_RSP + 2);
+    new_rip          = (Bit64u) read_virtual_word(BX_SEG_REG_SS, temp_RSP + 0);
+    new_eflags       = read_virtual_word(BX_SEG_REG_SS, temp_RSP + 4);
+
     top_nbytes_outer = 10;
     top_nbytes_same = 6;
     ss_offset = 8;
@@ -528,8 +489,7 @@ BX_CPU_C::long_iret(bxInstruction_c *i)
     }
 
     /* examine return SS selector and associated descriptor */
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + ss_offset,
-      2, 0, BX_READ, &raw_ss_selector);
+    raw_ss_selector = read_virtual_word(BX_SEG_REG_SS, temp_RSP + ss_offset);
 
     if ((raw_ss_selector & 0xfffc) == 0) {
       if (! IS_LONG64_SEGMENT(cs_descriptor) || (cs_selector.rpl == 3)) {
@@ -558,7 +518,7 @@ BX_CPU_C::long_iret(bxInstruction_c *i)
           IS_CODE_SEGMENT(ss_descriptor.type) ||
          !IS_DATA_SEGMENT_WRITEABLE(ss_descriptor.type))
       {
-        BX_ERROR(("iret64: SS AR byte not writable code segment"));
+        BX_ERROR(("iret64: SS AR byte not writable or code segment"));
         exception(BX_GP_EXCEPTION, raw_ss_selector & 0xfffc, 0);
       }
 
@@ -577,20 +537,13 @@ BX_CPU_C::long_iret(bxInstruction_c *i)
     }
 
     if (i->os64L()) {
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 24,
-        8, CPL==3, BX_READ, &new_rsp);
+      new_rsp = read_virtual_qword(BX_SEG_REG_SS, temp_RSP + 24);
     }
     else if (i->os32L()) {
-      Bit32u return_ESP;
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 12,
-        4, CPL==3, BX_READ, &return_ESP);
-      new_rsp = return_ESP;
+      new_rsp = (Bit64u) read_virtual_dword(BX_SEG_REG_SS, temp_RSP + 12);
     }
     else {
-      Bit16u return_SP;
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 6,
-        2, CPL==3, BX_READ, &return_SP);
-      new_rsp = return_SP;
+      new_rsp = (Bit64u) read_virtual_word(BX_SEG_REG_SS, temp_RSP + 6);
     }
 
     Bit8u prev_cpl = CPL; /* previous CPL */
@@ -621,7 +574,7 @@ BX_CPU_C::long_iret(bxInstruction_c *i)
     }
     else {
       // we are in 64-bit mode !
-      loadSRegLMNominal(BX_SEG_REG_SS, raw_ss_selector, 0, cs_selector.rpl);
+      loadSRegLMNominal(BX_SEG_REG_SS, raw_ss_selector, cs_selector.rpl);
     }
 
     if (StackAddrSize64()) RSP = new_rsp;
