@@ -1,14 +1,9 @@
 ////////////////////////////////////////////////////////////////////////
-// $Id: ret_far.cc,v 1.7 2006/06/12 16:58:27 sshwarts Exp $
+// $Id: ret_far.cc,v 1.12 2007/12/20 20:58:37 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//   Copyright (c) 2005 Stanislav Shwartsman
+//          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -68,18 +63,11 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
 
 #if BX_SUPPORT_X86_64
   if (i->os64L()) {
-    /* operand size=64: 2nd qword on stack must be within stack limits,
-     *   else #SS(0); */
-    if (!can_pop(16))
-    {
-      BX_ERROR(("return_protected: 2rd qword not in stack limits"));
-      exception(BX_SS_EXCEPTION, 0, 0);
-    }
+    /* operand size=64: in long mode 1st and 2nd quadword on the stack 
+       must be in canonical address space */
 
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 8,
-        2, CPL==3, BX_READ, &raw_cs_selector);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 0,
-        8, CPL==3, BX_READ, &return_RIP);
+    raw_cs_selector = read_virtual_word (BX_SEG_REG_SS, temp_RSP + 8);
+    return_RIP      = read_virtual_qword(BX_SEG_REG_SS, temp_RSP);
 
     stack_param_offset = 16;
   } 
@@ -94,12 +82,8 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
       exception(BX_SS_EXCEPTION, 0, 0);
     }
 
-    Bit32u return_EIP = 0;
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 4,
-        2, CPL==3, BX_READ, &raw_cs_selector);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 0,
-        4, CPL==3, BX_READ, &return_EIP);
-    return_RIP = return_EIP;
+    raw_cs_selector = read_virtual_word (BX_SEG_REG_SS, temp_RSP + 4);
+    return_RIP      = read_virtual_dword(BX_SEG_REG_SS, temp_RSP);
 
     stack_param_offset = 8;
   }
@@ -112,12 +96,8 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
       exception(BX_SS_EXCEPTION, 0, 0);
     }
 
-    Bit16u return_IP = 0;
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 2,
-        2, CPL==3, BX_READ, &raw_cs_selector);
-    access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 0,
-        2, CPL==3, BX_READ, &return_IP);
-    return_RIP = return_IP;
+    raw_cs_selector = read_virtual_word(BX_SEG_REG_SS, temp_RSP + 2);
+    return_RIP      = read_virtual_word(BX_SEG_REG_SS, temp_RSP);
 
     stack_param_offset = 4;
   }
@@ -192,10 +172,8 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
         exception(BX_SS_EXCEPTION, 0, 0);
       }
 
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 24 + pop_bytes,
-        2, 0, BX_READ, &raw_ss_selector);
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 16 + pop_bytes,
-        8, 0, BX_READ, &return_RSP);
+      raw_ss_selector = read_virtual_word (BX_SEG_REG_SS, temp_RSP + 24 + pop_bytes);
+      return_RSP      = read_virtual_qword(BX_SEG_REG_SS, temp_RSP + 16 + pop_bytes);
     }
     else
 #endif
@@ -206,12 +184,8 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
         exception(BX_SS_EXCEPTION, 0, 0);
       }
 
-      Bit32u return_ESP = 0;
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 12 + pop_bytes,
-        2, 0, BX_READ, &raw_ss_selector);
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP +  8 + pop_bytes,
-        4, 0, BX_READ, &return_ESP);
-      return_RSP = return_ESP;
+      raw_ss_selector = read_virtual_word (BX_SEG_REG_SS, temp_RSP + 12 + pop_bytes);
+      return_RSP      = read_virtual_dword(BX_SEG_REG_SS, temp_RSP +  8 + pop_bytes);
     }
     else {
       /* top 8+immediate bytes on stack must be within stack limits, else #SS(0) */
@@ -220,12 +194,8 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
         exception(BX_SS_EXCEPTION, 0, 0);
       }
 
-      Bit16u return_SP = 0;
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 6 + pop_bytes,
-        2, 0, BX_READ, &raw_ss_selector);
-      access_linear(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS) + temp_RSP + 4 + pop_bytes,
-        2, 0, BX_READ, &return_SP);
-      return_RSP = return_SP;
+      raw_ss_selector = read_virtual_word(BX_SEG_REG_SS, temp_RSP + 6 + pop_bytes);
+      return_RSP      = read_virtual_word(BX_SEG_REG_SS, temp_RSP + 4 + pop_bytes);
     }
 
     /* selector index must be within its descriptor table limits,
@@ -233,7 +203,7 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
     parse_selector(raw_ss_selector, &ss_selector);
 
     if ((raw_ss_selector & 0xfffc) == 0) {
-      if (IsLongMode()) {
+      if (long_mode()) {
         if (! IS_LONG64_SEGMENT(cs_descriptor) || (cs_selector.rpl == 3)) {
           BX_ERROR(("return_protected: SS selector null"));
           exception(BX_GP_EXCEPTION, 0, 0);
