@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift16.cc,v 1.32 2006/03/26 18:58:01 sshwarts Exp $
+// $Id: shift16.cc,v 1.39 2007/12/20 20:58:37 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -23,6 +23,7 @@
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+/////////////////////////////////////////////////////////////////////////
 
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -36,6 +37,7 @@ void BX_CPU_C::SHLD_EwGw(bxInstruction_c *i)
   Bit16u op1_16, op2_16, result_16;
   Bit32u temp_32, result_32;
   unsigned count;
+  unsigned of, cf;
 
   /* op1:op2 << count.  result stored in op1 */
   if (i->b1() == 0x1a4)
@@ -51,17 +53,18 @@ void BX_CPU_C::SHLD_EwGw(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
+    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
-  // count is 1..31
 
   op2_16 = BX_READ_16BIT_REG(i->nnn());
 
+  /* count < 32, since only lower 5 bits used */
   temp_32 = ((Bit32u)(op1_16) << 16) | (op2_16); // double formed by op1:op2
   result_32 = temp_32 << count;
-  // Hack to act like x86 SHLD when count > 16
+
+  // hack to act like x86 SHLD when count > 16
   if (count > 16) {
     // when count > 16 actually shifting op1:op2:op2 << count,
     // it is the same as shifting op2:op2 by count-16
@@ -78,10 +81,11 @@ void BX_CPU_C::SHLD_EwGw(bxInstruction_c *i)
     write_RMW_virtual_word(result_16);
   }
 
-  /* set eflags:
-   * SHLD count affects the following flags: O,S,Z,A,P,C
-   */
-  SET_FLAGS_OSZAPC_16(op1_16, count, result_16, BX_INSTR_SHL16);
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+
+  cf = (temp_32 >> (32 - count)) & 0x1;
+  of = cf ^ (result_16 >> 15); // of = cf ^ result15
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::SHRD_EwGw(bxInstruction_c *i)
@@ -89,6 +93,7 @@ void BX_CPU_C::SHRD_EwGw(bxInstruction_c *i)
   Bit16u op1_16, op2_16, result_16;
   Bit32u temp_32, result_32;
   unsigned count;
+  unsigned cf, of;
 
   if (i->b1() == 0x1ac)
     count = i->Ib();
@@ -103,17 +108,18 @@ void BX_CPU_C::SHRD_EwGw(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
+    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
-  // count is 1..31
 
   op2_16 = BX_READ_16BIT_REG(i->nnn());
 
-  // Hack to act like x86 SHLD when count > 16
+  /* count < 32, since only lower 5 bits used */
   temp_32 = (op2_16 << 16) | op1_16; // double formed by op2:op1
   result_32 = temp_32 >> count;
+
+  // hack to act like x86 SHRD when count > 16
   if (count > 16) {
     // when count > 16 actually shifting op2:op2:op1 >> count,
     // it is the same as shifting op2:op2 by count-16
@@ -130,20 +136,22 @@ void BX_CPU_C::SHRD_EwGw(bxInstruction_c *i)
     write_RMW_virtual_word(result_16);
   }
 
-  /* set eflags:
-   * SHRD count affects the following flags: O,S,Z,A,P,C
-   */
-  SET_FLAGS_OSZAPC_16(op1_16, count, result_16, BX_INSTR_SHRD16);
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+
+  cf = (op1_16 >> (count - 1)) & 0x1;
+  of = ((result_16 << 1) ^ result_16) >> 15; // of = result14 ^ result15
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::ROL_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
+  unsigned bit0, bit15;
 
-  if ( i->b1() == 0xc1 )
+  if (i->b1() == 0xc1)
     count = i->Ib();
-  else if ( i->b1() == 0xd1 )
+  else if (i->b1() == 0xd1)
     count = 1;
   else // 0xd3
     count = CL;
@@ -154,17 +162,19 @@ void BX_CPU_C::ROL_Ew(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
+    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
 
-  if ( (count & 0x0f) == 0 ) {
-    if ( count & 0x10 ) {
-      unsigned bit0 = op1_16 & 1;
-      set_CF(bit0);
-      set_OF(bit0 ^ (op1_16 >> 15));
+  if ((count & 0x0f) == 0) {
+    if (count & 0x10) {
+      bit0  = (op1_16 & 0x1);
+      bit15 = (op1_16 >> 15);
+      // of = cf ^ result15
+      SET_FLAGS_OxxxxC(bit0 ^ bit15, bit0);
     }
     return;
   }
+
   count &= 0x0f; // only use bottom 4 bits
 
   result_16 = (op1_16 << count) | (op1_16 >> (16 - count));
@@ -177,23 +187,21 @@ void BX_CPU_C::ROL_Ew(bxInstruction_c *i)
     write_RMW_virtual_word(result_16);
   }
 
-  /* set eflags:
-   * ROL count affects the following flags: C, O
-   */
-  bx_bool temp_CF = (result_16 & 0x01);
-
-  set_CF(temp_CF);
-  set_OF(temp_CF ^ (result_16 >> 15));
+  bit0  = (result_16 & 0x1);
+  bit15 = (result_16 >> 15);
+  // of = cf ^ result15
+  SET_FLAGS_OxxxxC(bit0 ^ bit15, bit0);
 }
 
 void BX_CPU_C::ROR_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
+  unsigned bit14, bit15;
 
-  if ( i->b1() == 0xc1 )
+  if (i->b1() == 0xc1)
     count = i->Ib();
-  else if ( i->b1() == 0xd1 )
+  else if (i->b1() == 0xd1)
     count = 1;
   else // 0xd3
     count = CL;
@@ -204,18 +212,19 @@ void BX_CPU_C::ROR_Ew(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
+    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
 
-  if ( (count & 0x0f) == 0 ) {
-    if ( count & 0x10 ) {
-      unsigned bit14 = (op1_16 >> 14) & 1;
-      unsigned bit15 = (op1_16 >> 15);
-      set_CF(bit15);
-      set_OF(bit15 ^ bit14);
+  if ((count & 0x0f) == 0) {
+    if (count & 0x10) {
+      bit14 = (op1_16 >> 14) & 1;
+      bit15 = (op1_16 >> 15) & 1;
+      // of = result14 ^ result15 
+      SET_FLAGS_OxxxxC(bit14 ^ bit15, bit15);
     }
     return;
   }
+
   count &= 0x0f;  // use only 4 LSB's
 
   result_16 = (op1_16 >> count) | (op1_16 << (16 - count));
@@ -228,24 +237,21 @@ void BX_CPU_C::ROR_Ew(bxInstruction_c *i)
     write_RMW_virtual_word(result_16);
   }
   
-  /* set eflags:
-   * ROR count affects the following flags: C, O
-   */
-  bx_bool result_b15 = (result_16 & 0x8000) != 0;
-  bx_bool result_b14 = (result_16 & 0x4000) != 0;
-
-  set_CF(result_b15);
-  set_OF(result_b15 ^ result_b14);
+  bit14 = (result_16 >> 14) & 1;
+  bit15 = (result_16 >> 15) & 1;
+  // of = result14 ^ result15 
+  SET_FLAGS_OxxxxC(bit14 ^ bit15, bit15);
 }
 
 void BX_CPU_C::RCL_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
+  unsigned of, cf;
 
-  if ( i->b1() == 0xc1 )
+  if (i->b1() == 0xc1)
     count = i->Ib();
-  else if ( i->b1() == 0xd1 )
+  else if (i->b1() == 0xd1)
     count = 1;
   else // 0xd3
     count = CL;
@@ -258,7 +264,7 @@ void BX_CPU_C::RCL_Ew(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
+    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
@@ -282,23 +288,20 @@ void BX_CPU_C::RCL_Ew(bxInstruction_c *i)
     write_RMW_virtual_word(result_16);
   }
 
-  /* set eflags:
-   * RCL count affects the following flags: C, O
-   */
-  bx_bool temp_CF = (op1_16 >> (16 - count)) & 0x01;
-
-  set_CF(temp_CF);
-  set_OF(temp_CF ^ (result_16 >> 15));
+  cf = (op1_16 >> (16 - count)) & 0x1;
+  of = cf ^ (result_16 >> 15); // of = cf ^ result15
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::RCR_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
+  unsigned of, cf;
 
-  if ( i->b1() == 0xc1 )
+  if (i->b1() == 0xc1)
     count = i->Ib();
-  else if ( i->b1() == 0xd1 )
+  else if (i->b1() == 0xd1)
     count = 1;
   else // 0xd3
     count = CL;
@@ -311,14 +314,13 @@ void BX_CPU_C::RCR_Ew(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
+    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
 
   if (! count) return;
 
-  result_16 = (op1_16 >> count) |
-          (getB_CF() << (16 - count)) |
-          (op1_16 << (17 - count));
+  result_16 = (op1_16 >> count) | (getB_CF() << (16 - count)) |
+              (op1_16 << (17 - count));
 
   /* now write result back to destination */
   if (i->modC0()) {
@@ -328,22 +330,20 @@ void BX_CPU_C::RCR_Ew(bxInstruction_c *i)
     write_RMW_virtual_word(result_16);
   }
 
-  /* set eflags:
-   * RCR count affects the following flags: C, O
-   */
-
-  set_CF((op1_16 >> (count - 1)) & 0x01);
-  set_OF((((result_16 << 1) ^ result_16) & 0x8000) > 0);
+  cf = (op1_16 >> (count - 1)) & 0x1;
+  of = ((result_16 << 1) ^ result_16) >> 15; // of = result15 ^ result14
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::SHL_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
+  unsigned of = 0, cf = 0;
 
-  if ( i->b1() == 0xc1 )
+  if (i->b1() == 0xc1)
     count = i->Ib();
-  else if ( i->b1() == 0xd1 )
+  else if (i->b1() == 0xd1)
     count = 1;
   else // 0xd3
     count = CL;
@@ -356,12 +356,19 @@ void BX_CPU_C::SHL_Ew(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
+    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
 
-  result_16 = (op1_16 << count);
+  if (count <= 16) {
+    result_16 = (op1_16 << count);
+    cf = (op1_16 >> (16 - count)) & 0x1;
+    of = cf ^ (result_16 >> 15); // of = cf ^ result15
+  }
+  else {
+    result_16 = 0;
+  }
 
   /* now write result back to destination */
   if (i->modC0()) {
@@ -371,17 +378,19 @@ void BX_CPU_C::SHL_Ew(bxInstruction_c *i)
     write_RMW_virtual_word(result_16);
   }
 
-  SET_FLAGS_OSZAPC_16(op1_16, count, result_16, BX_INSTR_SHL16);
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::SHR_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
+  unsigned of, cf;
 
-  if ( i->b1() == 0xc1 )
+  if (i->b1() == 0xc1)
     count = i->Ib();
-  else if ( i->b1() == 0xd1 )
+  else if (i->b1() == 0xd1)
     count = 1;
   else // 0xd3
     count = CL;
@@ -394,19 +403,12 @@ void BX_CPU_C::SHR_Ew(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
+    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
 
-#if defined(BX_HostAsm_Shr16)
-  Bit32u flags32;
-  asmShr16(result_16, op1_16, count, flags32);
-  setEFlagsOSZAPC(flags32);
-#else
   result_16 = (op1_16 >> count);
-  SET_FLAGS_OSZAPC_16(op1_16, count, result_16, BX_INSTR_SHR16);
-#endif
 
   /* now write result back to destination */
   if (i->modC0()) {
@@ -415,16 +417,24 @@ void BX_CPU_C::SHR_Ew(bxInstruction_c *i)
   else {
     write_RMW_virtual_word(result_16);
   }
+
+  cf = (op1_16 >> (count - 1)) & 0x1;
+  // note, that of == result15 if count == 1 and
+  //            of == 0        if count >= 2
+  of = ((result_16 << 1) ^ result_16) >> 15;
+
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::SAR_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
-  unsigned count;
+  unsigned count, cf;
 
-  if ( i->b1() == 0xc1 )
+  if (i->b1() == 0xc1)
     count = i->Ib();
-  else if ( i->b1() == 0xd1 )
+  else if (i->b1() == 0xd1)
     count = 1;
   else // 0xd3
     count = CL;
@@ -437,7 +447,7 @@ void BX_CPU_C::SAR_Ew(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
+    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
@@ -449,6 +459,8 @@ void BX_CPU_C::SAR_Ew(bxInstruction_c *i)
     else {
       result_16 = (op1_16 >> count);
     }
+
+    cf = (op1_16 >> (count - 1)) & 0x1;
   }
   else {
     if (op1_16 & 0x8000) {
@@ -457,8 +469,14 @@ void BX_CPU_C::SAR_Ew(bxInstruction_c *i)
     else {
       result_16 = 0;
     }
+
+    cf = (result_16 & 0x1);
   }
 
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+  /* signed overflow cannot happen in SAR instruction */
+  SET_FLAGS_OxxxxC(0, cf); 
+
   /* now write result back to destination */
   if (i->modC0()) {
     BX_WRITE_16BIT_REG(i->rm(), result_16);
@@ -466,6 +484,4 @@ void BX_CPU_C::SAR_Ew(bxInstruction_c *i)
   else {
     write_RMW_virtual_word(result_16);
   }
-
-  SET_FLAGS_OSZAPC_16(op1_16, count, result_16, BX_INSTR_SAR16);
 }
