@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift64.cc,v 1.20 2006/03/26 18:58:01 sshwarts Exp $
+// $Id: shift64.cc,v 1.28 2007/12/20 20:58:37 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -23,6 +23,7 @@
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+/////////////////////////////////////////////////////////////////////////
 
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -37,13 +38,14 @@ void BX_CPU_C::SHLD_EqGq(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, result_64;
   unsigned count;
-
-  /* op1:op2 << count. result stored in op1 */
+  unsigned cf, of;
 
   if (i->b1() == 0x1a4)
-    count = i->Ib() & 0x3f;
+    count = i->Ib();
   else // 0x1a5
-    count = CL & 0x3f;
+    count = CL;
+
+  count &= 0x3f; // use only 6 LSB's
 
   /* op1 is a register or memory reference */
   if (i->modC0()) {
@@ -51,7 +53,7 @@ void BX_CPU_C::SHLD_EqGq(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
+    op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
@@ -68,21 +70,25 @@ void BX_CPU_C::SHLD_EqGq(bxInstruction_c *i)
     write_RMW_virtual_qword(result_64);
   }
 
-  /* set eflags:
-   * SHLD count affects the following flags: O,S,Z,A,P,C
-   */
-  SET_FLAGS_OSZAPC_64(op1_64, count, result_64, BX_INSTR_SHL64);
+  SET_FLAGS_OSZAPC_LOGIC_64(result_64); /* handle SF, ZF and AF flags */
+
+  cf = (op1_64 >> (64 - count)) & 0x1;
+  of = cf ^ (result_64 >> 63); // of = cf ^ result63
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::SHRD_EqGq(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, result_64;
   unsigned count;
+  unsigned cf, of;
 
   if (i->b1() == 0x1ac)
-    count = i->Ib() & 0x3f;
+    count = i->Ib();
   else // 0x1ad
-    count = CL & 0x3f;
+    count = CL;
+
+  count &= 0x3f; // use only 6 LSB's
 
   /* op1 is a register or memory reference */
   if (i->modC0()) {
@@ -90,7 +96,7 @@ void BX_CPU_C::SHRD_EqGq(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
+    op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
@@ -107,16 +113,18 @@ void BX_CPU_C::SHRD_EqGq(bxInstruction_c *i)
     write_RMW_virtual_qword(result_64);
   }
 
-  /* set eflags:
-   * SHRD count affects the following flags: O,S,Z,A,P,C
-   */
-  SET_FLAGS_OSZAPC_64(op1_64, count, result_64, BX_INSTR_SHRD64);
+  SET_FLAGS_OSZAPC_LOGIC_64(result_64); /* handle SF, ZF and AF flags */
+
+  cf = (op1_64 >> (count - 1)) & 0x1;
+  of = ((result_64 << 1) ^ result_64) >> 63; // of = result62 ^ result63
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::ROL_Eq(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
+  unsigned bit0, bit63;
 
   if (i->b1() == 0xc1)
     count = i->Ib() & 0x3f;
@@ -131,7 +139,7 @@ void BX_CPU_C::ROL_Eq(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
+    op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
   }
 
   if (! count) return;
@@ -146,19 +154,17 @@ void BX_CPU_C::ROL_Eq(bxInstruction_c *i)
     write_RMW_virtual_qword(result_64);
   }
 
-  /* set eflags:
-   * ROL count affects the following flags: C, O
-   */
-  bx_bool temp_CF = (result_64 & 0x01);
-
-  set_CF(temp_CF);
-  set_OF(temp_CF ^ (result_64 >> 63));
+  bit0  = (result_64 & 0x1);
+  bit63 = (result_64 >> 63);
+  // of = cf ^ result63
+  SET_FLAGS_OxxxxC(bit0 ^ bit63, bit0);
 }
 
 void BX_CPU_C::ROR_Eq(bxInstruction_c *i)
 {
-    Bit64u op1_64, result_64;
+  Bit64u op1_64, result_64;
   unsigned count;
+  unsigned bit62, bit63;
 
   if (i->b1() == 0xc1)
     count = i->Ib() & 0x3f;
@@ -173,7 +179,7 @@ void BX_CPU_C::ROR_Eq(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
+    op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
   }
 
   if (! count) return;
@@ -188,20 +194,17 @@ void BX_CPU_C::ROR_Eq(bxInstruction_c *i)
     write_RMW_virtual_qword(result_64);
   }
 
-  /* set eflags:
-   * ROR count affects the following flags: C, O
-   */
-  bx_bool result_b63 = (result_64 & BX_CONST64(0x8000000000000000)) != 0;
-  bx_bool result_b62 = (result_64 & BX_CONST64(0x4000000000000000)) != 0;
-
-  set_CF(result_b63);
-  set_OF(result_b63 ^ result_b62);
+  bit63 = (result_64 >> 63) & 1;
+  bit62 = (result_64 >> 62) & 1;
+  // of = result62 ^ result63
+  SET_FLAGS_OxxxxC(bit62 ^ bit63, bit63);
 }
 
 void BX_CPU_C::RCL_Eq(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
+  unsigned cf, of;
 
   if (i->b1() == 0xc1)
     count = i->Ib() & 0x3f;
@@ -216,7 +219,7 @@ void BX_CPU_C::RCL_Eq(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
+    op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
@@ -237,19 +240,16 @@ void BX_CPU_C::RCL_Eq(bxInstruction_c *i)
     write_RMW_virtual_qword(result_64);
   }
 
-  /* set eflags:
-   * RCL count affects the following flags: C, O
-   */
-  bx_bool temp_CF = (op1_64 >> (64 - count)) & 0x01;
-
-  set_CF(temp_CF);
-  set_OF(temp_CF ^ (result_64 >> 63));
+  cf = (op1_64 >> (64 - count)) & 0x1;
+  of = cf ^ (result_64 >> 63); // of = cf ^ result63
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::RCR_Eq(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
+  unsigned of, cf;
 
   if (i->b1() == 0xc1)
     count = i->Ib() & 0x3f;
@@ -264,7 +264,7 @@ void BX_CPU_C::RCR_Eq(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
+    op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
@@ -285,18 +285,16 @@ void BX_CPU_C::RCR_Eq(bxInstruction_c *i)
     write_RMW_virtual_qword(result_64);
   }
 
-  /* set eflags:
-   * RCR count affects the following flags: C, O
-   */
-
-  set_CF((op1_64 >> (count - 1)) & 0x01);
-  set_OF((((result_64 << 1) ^ result_64) & BX_CONST64(0x8000000000000000)) > 0);
+  cf = (op1_64 >> (count - 1)) & 0x1;
+  of = ((result_64 << 1) ^ result_64) >> 63;
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::SHL_Eq(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
+  unsigned cf, of;
 
   if (i->b1() == 0xc1)
     count = i->Ib() & 0x3f;
@@ -311,12 +309,15 @@ void BX_CPU_C::SHL_Eq(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
+    op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
 
+  /* count < 64, since only lower 6 bits used */
   result_64 = (op1_64 << count);
+  cf = (op1_64 >> (64 - count)) & 0x1;
+  of = cf ^ (result_64 >> 63);
 
   /* now write result back to destination */
   if (i->modC0()) {
@@ -326,13 +327,15 @@ void BX_CPU_C::SHL_Eq(bxInstruction_c *i)
     write_RMW_virtual_qword(result_64);
   }
 
-  SET_FLAGS_OSZAPC_64(op1_64, count, result_64, BX_INSTR_SHL64);
+  SET_FLAGS_OSZAPC_LOGIC_64(result_64); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::SHR_Eq(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
+  unsigned cf, of;
 
   if (i->b1() == 0xc1)
     count = i->Ib() & 0x3f;
@@ -347,7 +350,7 @@ void BX_CPU_C::SHR_Eq(bxInstruction_c *i)
   }
   else {
     /* pointer, segment address pair */
-    read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
+    op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
@@ -362,7 +365,13 @@ void BX_CPU_C::SHR_Eq(bxInstruction_c *i)
     write_RMW_virtual_qword(result_64);
   }
 
-  SET_FLAGS_OSZAPC_64(op1_64, count, result_64, BX_INSTR_SHR64);
+  cf = (op1_64 >> (count - 1)) & 0x1;
+  // note, that of == result63 if count == 1 and
+  //            of == 0        if count >= 2
+  of = ((result_64 << 1) ^ result_64) >> 63;
+
+  SET_FLAGS_OSZAPC_LOGIC_64(result_64); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OxxxxC(of, cf);
 }
 
 void BX_CPU_C::SAR_Eq(bxInstruction_c *i)
@@ -379,16 +388,16 @@ void BX_CPU_C::SAR_Eq(bxInstruction_c *i)
 
   /* op1 is a register or memory reference */
   if (i->modC0()) {
-      op1_64 = BX_READ_64BIT_REG(i->rm());
+    op1_64 = BX_READ_64BIT_REG(i->rm());
   }
   else {
-      /* pointer, segment address pair */
-      read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
+    /* pointer, segment address pair */
+    op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
   }
 
   if (!count) return;
 
-  /* count < 64, since only lower 5 bits used */
+  /* count < 64, since only lower 6 bits used */
   if (op1_64 & BX_CONST64(0x8000000000000000)) {
     result_64 = (op1_64 >> count) | (BX_CONST64(0xffffffffffffffff) << (64 - count));
   }               
@@ -404,7 +413,9 @@ void BX_CPU_C::SAR_Eq(bxInstruction_c *i)
     write_RMW_virtual_qword(result_64);
   }
 
-  SET_FLAGS_OSZAPC_64(op1_64, count, result_64, BX_INSTR_SAR64);
+  SET_FLAGS_OSZAPC_LOGIC_64(result_64); /* handle SF, ZF and AF flags */
+  set_CF((op1_64 >> (count - 1)) & 1);
+  clear_OF();  /* signed overflow cannot happen in SAR instruction */
 }
 
 #endif /* if BX_SUPPORT_X86_64 */
