@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: dis_tables.h,v 1.29 2006/04/27 15:11:45 sshwarts Exp $
+// $Id: dis_tables.h,v 1.32 2007/09/19 19:38:10 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 
 #ifndef _BX_DISASM_TABLES_
@@ -14,6 +14,7 @@
 #define _GRPRM         6
 #define _GRP3BOP       7
 #define _GRP3BTAB      8
+#define _GRP64B        9
 
 /* ************************************************************************ */
 #define GRPSSE(n)       _GRPSSE,   BxDisasmGroupSSE_##n
@@ -24,6 +25,7 @@
 #define GRP3DNOW        _GRP3DNOW, BxDisasm3DNowGroup
 #define GR3BOP(n)       _GRP3BOP,  BxDisasm3ByteOp##n
 #define GR3BTAB(n)      _GRP3BTAB, BxDisasm3ByteTable##n
+#define GR64BIT(n)      _GRP64B,   BxDisasmGrpOs64B_##n
 /* ************************************************************************ */
 
 #define Apw &disassembler::Apw
@@ -69,12 +71,18 @@
 #define  Gd &disassembler::Gd
 #define  Gq &disassembler::Gq
 
+#define  Hbd &disassembler::Hbd
+#define  Hwd &disassembler::Hwd
+#define   Hd &disassembler::Hd
+#define   Hq &disassembler::Hq
+
 #define  I1 &disassembler::I1
 #define  Ib &disassembler::Ib
 #define  Iw &disassembler::Iw
 #define  Id &disassembler::Id
 #define  Iq &disassembler::Iq
 
+#define IbIb &disassembler::IbIb
 #define IwIb &disassembler::IwIb
 
 #define sIbw &disassembler::sIbw
@@ -102,6 +110,8 @@
 #define Vpd &disassembler::Vpd
 #define Udq &disassembler::Udq
 
+#define  Ww &disassembler::Ww
+#define  Wd &disassembler::Wd
 #define  Wq &disassembler::Wq
 #define Wdq &disassembler::Wdq
 #define Wss &disassembler::Wss
@@ -126,6 +136,8 @@
 #define Mdq &disassembler::Mdq
 #define Mps &disassembler::Mps
 #define Mpd &disassembler::Mpd
+#define Mss &disassembler::Mss
+#define Msd &disassembler::Msd
 
 #define  Xb &disassembler::Xb
 #define  Xw &disassembler::Xw
