@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: syntax.cc,v 1.10 2006/04/27 15:11:45 sshwarts Exp $
+// $Id: syntax.cc,v 1.12 2007/11/18 21:29:17 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 
 #include <stdio.h>
@@ -144,32 +144,56 @@ void disassembler::initialize_modrm_segregs()
   sreg_mod01or10_rm16[6] = segment_name[SS_REG];
   sreg_mod01or10_rm16[7] = segment_name[DS_REG];
 
-  sreg_mod01or10_rm32[0] = segment_name[DS_REG];
-  sreg_mod01or10_rm32[1] = segment_name[DS_REG];
-  sreg_mod01or10_rm32[2] = segment_name[DS_REG];
-  sreg_mod01or10_rm32[3] = segment_name[DS_REG];
-  sreg_mod01or10_rm32[4] = segment_name[NULL_SEGMENT_REGISTER];
-  sreg_mod01or10_rm32[5] = segment_name[SS_REG];
-  sreg_mod01or10_rm32[6] = segment_name[DS_REG];
-  sreg_mod01or10_rm32[7] = segment_name[DS_REG];
+  sreg_mod01or10_rm32[0]  = segment_name[DS_REG];
+  sreg_mod01or10_rm32[1]  = segment_name[DS_REG];
+  sreg_mod01or10_rm32[2]  = segment_name[DS_REG];
+  sreg_mod01or10_rm32[3]  = segment_name[DS_REG];
+  sreg_mod01or10_rm32[4]  = segment_name[NULL_SEGMENT_REGISTER];
+  sreg_mod01or10_rm32[5]  = segment_name[SS_REG];
+  sreg_mod01or10_rm32[6]  = segment_name[DS_REG];
+  sreg_mod01or10_rm32[7]  = segment_name[DS_REG];
+  sreg_mod01or10_rm32[8]  = segment_name[DS_REG];
+  sreg_mod01or10_rm32[9]  = segment_name[DS_REG];
+  sreg_mod01or10_rm32[10] = segment_name[DS_REG];
+  sreg_mod01or10_rm32[11] = segment_name[DS_REG];
+  sreg_mod01or10_rm32[12] = segment_name[NULL_SEGMENT_REGISTER];
+  sreg_mod01or10_rm32[13] = segment_name[DS_REG];
+  sreg_mod01or10_rm32[14] = segment_name[DS_REG];
+  sreg_mod01or10_rm32[15] = segment_name[DS_REG];
 
-  sreg_mod00_base32[0] = segment_name[DS_REG];
-  sreg_mod00_base32[1] = segment_name[DS_REG];
-  sreg_mod00_base32[2] = segment_name[DS_REG];
-  sreg_mod00_base32[3] = segment_name[DS_REG];
-  sreg_mod00_base32[4] = segment_name[SS_REG];
-  sreg_mod00_base32[5] = segment_name[DS_REG];
-  sreg_mod00_base32[6] = segment_name[DS_REG];
-  sreg_mod00_base32[7] = segment_name[DS_REG];
+  sreg_mod00_base32[0]  = segment_name[DS_REG];
+  sreg_mod00_base32[1]  = segment_name[DS_REG];
+  sreg_mod00_base32[2]  = segment_name[DS_REG];
+  sreg_mod00_base32[3]  = segment_name[DS_REG];
+  sreg_mod00_base32[4]  = segment_name[SS_REG];
+  sreg_mod00_base32[5]  = segment_name[DS_REG];
+  sreg_mod00_base32[6]  = segment_name[DS_REG];
+  sreg_mod00_base32[7]  = segment_name[DS_REG];
+  sreg_mod00_base32[8]  = segment_name[DS_REG];
+  sreg_mod00_base32[9]  = segment_name[DS_REG];
+  sreg_mod00_base32[10] = segment_name[DS_REG];
+  sreg_mod00_base32[11] = segment_name[DS_REG];
+  sreg_mod00_base32[12] = segment_name[DS_REG];
+  sreg_mod00_base32[13] = segment_name[DS_REG];
+  sreg_mod00_base32[14] = segment_name[DS_REG];
+  sreg_mod00_base32[15] = segment_name[DS_REG];
 
-  sreg_mod01or10_base32[0] = segment_name[DS_REG];
-  sreg_mod01or10_base32[1] = segment_name[DS_REG];
-  sreg_mod01or10_base32[2] = segment_name[DS_REG];
-  sreg_mod01or10_base32[3] = segment_name[DS_REG];
-  sreg_mod01or10_base32[4] = segment_name[SS_REG];
-  sreg_mod01or10_base32[5] = segment_name[SS_REG];
-  sreg_mod01or10_base32[6] = segment_name[DS_REG];
-  sreg_mod01or10_base32[7] = segment_name[DS_REG];
+  sreg_mod01or10_base32[0]  = segment_name[DS_REG];
+  sreg_mod01or10_base32[1]  = segment_name[DS_REG];
+  sreg_mod01or10_base32[2]  = segment_name[DS_REG];
+  sreg_mod01or10_base32[3]  = segment_name[DS_REG];
+  sreg_mod01or10_base32[4]  = segment_name[SS_REG];
+  sreg_mod01or10_base32[5]  = segment_name[SS_REG];
+  sreg_mod01or10_base32[6]  = segment_name[DS_REG];
+  sreg_mod01or10_base32[7]  = segment_name[DS_REG];
+  sreg_mod01or10_base32[8]  = segment_name[DS_REG];
+  sreg_mod01or10_base32[9]  = segment_name[DS_REG];
+  sreg_mod01or10_base32[10] = segment_name[DS_REG];
+  sreg_mod01or10_base32[11] = segment_name[DS_REG];
+  sreg_mod01or10_base32[12] = segment_name[DS_REG];
+  sreg_mod01or10_base32[13] = segment_name[DS_REG];
+  sreg_mod01or10_base32[14] = segment_name[DS_REG];
+  sreg_mod01or10_base32[15] = segment_name[DS_REG];
 }
 
 //////////////////
@@ -208,6 +232,10 @@ void disassembler::print_disassembly_intel(const x86_insn *insn, const BxDisasmO
     dis_sprintf(", ");
     (this->*entry->Operand3)(insn);
   }
+  if (entry->Operand4) {
+    dis_sprintf(", ");
+    (this->*entry->Operand4)(insn);
+  }
 }
 
 //////////////////
@@ -241,6 +269,10 @@ void disassembler::print_disassembly_att(const x86_insn *insn, const BxDisasmOpc
   // print opcode
   dis_sprintf("%s ", entry->AttOpcode);
 
+  if (entry->Operand4) {                                         
+    (this->*entry->Operand4)(insn);
+    dis_sprintf(", ");
+  }
   if (entry->Operand3) {                                         
     (this->*entry->Operand3)(insn);
     dis_sprintf(", ");
