@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_arith.cc,v 1.9 2007/03/23 21:27:13 sshwarts Exp $
+// $Id: fpu_arith.cc,v 1.10 2007/12/20 20:58:38 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -139,8 +139,7 @@ void BX_CPU_C::FADD_SINGLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float32 load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), &load_reg);
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -170,8 +169,7 @@ void BX_CPU_C::FADD_DOUBLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float64 load_reg;
-  read_virtual_qword(i->seg(), RMAddr(i), &load_reg);
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -201,8 +199,7 @@ void BX_CPU_C::FIADD_WORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit16s load_reg;
-  read_virtual_word(i->seg(), RMAddr(i), (Bit16u*)(&load_reg));
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
@@ -234,8 +231,7 @@ void BX_CPU_C::FIADD_DWORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit32s load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), (Bit32u*)(&load_reg));
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80(load_reg);
@@ -331,8 +327,7 @@ void BX_CPU_C::FMUL_SINGLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float32 load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), &load_reg);
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -362,8 +357,7 @@ void BX_CPU_C::FMUL_DOUBLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float64 load_reg;
-  read_virtual_qword(i->seg(), RMAddr(i), &load_reg);
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -393,8 +387,7 @@ void BX_CPU_C::FIMUL_WORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit16s load_reg;
-  read_virtual_word(i->seg(), RMAddr(i), (Bit16u*)(&load_reg));
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
@@ -426,8 +419,7 @@ void BX_CPU_C::FIMUL_DWORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit32s load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), (Bit32u*)(&load_reg));
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80(load_reg);
@@ -589,8 +581,7 @@ void BX_CPU_C::FSUB_SINGLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float32 load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), &load_reg);
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -620,8 +611,7 @@ void BX_CPU_C::FSUBR_SINGLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float32 load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), &load_reg);
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -651,8 +641,7 @@ void BX_CPU_C::FSUB_DOUBLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float64 load_reg;
-  read_virtual_qword(i->seg(), RMAddr(i), &load_reg);
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -682,8 +671,7 @@ void BX_CPU_C::FSUBR_DOUBLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float64 load_reg;
-  read_virtual_qword(i->seg(), RMAddr(i), &load_reg);
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -713,8 +701,7 @@ void BX_CPU_C::FISUB_WORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit16s load_reg;
-  read_virtual_word(i->seg(), RMAddr(i), (Bit16u*)(&load_reg));
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
@@ -746,8 +733,7 @@ void BX_CPU_C::FISUBR_WORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit16s load_reg;
-  read_virtual_word(i->seg(), RMAddr(i), (Bit16u*)(&load_reg));
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   floatx80 a = int32_to_floatx80((Bit32s)(load_reg));
   floatx80 b = BX_READ_FPU_REG(0);
@@ -779,8 +765,7 @@ void BX_CPU_C::FISUB_DWORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit32s load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), (Bit32u*)(&load_reg));
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80(load_reg);
@@ -813,8 +798,7 @@ void BX_CPU_C::FISUBR_DWORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit32s load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), (Bit32u*)(&load_reg));
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   floatx80 a = int32_to_floatx80(load_reg);
   floatx80 b = BX_READ_FPU_REG(0);
@@ -974,8 +958,7 @@ void BX_CPU_C::FDIV_SINGLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float32 load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), &load_reg);
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -1005,8 +988,7 @@ void BX_CPU_C::FDIVR_SINGLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float32 load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), &load_reg);
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -1036,8 +1018,7 @@ void BX_CPU_C::FDIV_DOUBLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float64 load_reg;
-  read_virtual_qword(i->seg(), RMAddr(i), &load_reg);
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -1067,8 +1048,7 @@ void BX_CPU_C::FDIVR_DOUBLE_REAL(bxInstruction_c *i)
      return;
   }
 
-  float64 load_reg;
-  read_virtual_qword(i->seg(), RMAddr(i), &load_reg);
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   float_status_t status = 
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -1098,8 +1078,7 @@ void BX_CPU_C::FIDIV_WORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit16s load_reg;
-  read_virtual_word(i->seg(), RMAddr(i), (Bit16u*)(&load_reg));
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
@@ -1131,8 +1110,7 @@ void BX_CPU_C::FIDIVR_WORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit16s load_reg;
-  read_virtual_word(i->seg(), RMAddr(i), (Bit16u*)(&load_reg));
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   floatx80 a = int32_to_floatx80((Bit32s)(load_reg));
   floatx80 b = BX_READ_FPU_REG(0);
@@ -1164,8 +1142,7 @@ void BX_CPU_C::FIDIV_DWORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit32s load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), (Bit32u*)(&load_reg));
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80(load_reg);
@@ -1197,8 +1174,7 @@ void BX_CPU_C::FIDIVR_DWORD_INTEGER(bxInstruction_c *i)
      return;
   }
 
-  Bit32s load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), (Bit32u*)(&load_reg));
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   floatx80 a = int32_to_floatx80(load_reg);
   floatx80 b = BX_READ_FPU_REG(0);
