@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_load_store.cc,v 1.12 2007/03/23 21:27:13 sshwarts Exp $
+// $Id: fpu_load_store.cc,v 1.14 2007/12/20 20:58:38 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -72,8 +72,7 @@ void BX_CPU_C::FLD_SINGLE_REAL(bxInstruction_c *i)
       return; 
   }
 
-  float32 load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), &load_reg);
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   float_status_t status = 
      FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -104,8 +103,7 @@ void BX_CPU_C::FLD_DOUBLE_REAL(bxInstruction_c *i)
       return; 
   }
 
-  float64 load_reg;
-  read_virtual_qword(i->seg(), RMAddr(i), &load_reg);
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   float_status_t status = 
      FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
@@ -160,8 +158,7 @@ void BX_CPU_C::FILD_WORD_INTEGER(bxInstruction_c *i)
       return; 
   }
 
-  Bit16s load_reg;
-  read_virtual_word(i->seg(), RMAddr(i), (Bit16u*)(&load_reg));
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
   floatx80 result = int32_to_floatx80((Bit32s) load_reg);
 
   BX_CPU_THIS_PTR the_i387.FPU_push();
@@ -185,8 +182,7 @@ void BX_CPU_C::FILD_DWORD_INTEGER(bxInstruction_c *i)
       return; 
   }
 
-  Bit32s load_reg;
-  read_virtual_dword(i->seg(), RMAddr(i), (Bit32u*)(&load_reg));
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
   floatx80 result = int32_to_floatx80(load_reg);
 
   BX_CPU_THIS_PTR the_i387.FPU_push();
@@ -210,8 +206,7 @@ void BX_CPU_C::FILD_QWORD_INTEGER(bxInstruction_c *i)
       return; 
   }
 
-  Bit64s load_reg;
-  read_virtual_qword(i->seg(), RMAddr(i), (Bit64u*)(&load_reg));
+  Bit64s load_reg = (Bit64s) read_virtual_qword(i->seg(), RMAddr(i));
   floatx80 result = int64_to_floatx80(load_reg);
 
   BX_CPU_THIS_PTR the_i387.FPU_push();
@@ -235,12 +230,9 @@ void BX_CPU_C::FBLD_PACKED_BCD(bxInstruction_c *i)
       return; 
   }
 
-  Bit16u hi2;
-  Bit64u lo8;
-
   // read packed bcd from memory
-  read_virtual_qword(i->seg(), RMAddr(i),     &lo8);
-  read_virtual_word (i->seg(), RMAddr(i) + 8, &hi2);
+  Bit16u lo8 = read_virtual_qword(i->seg(), RMAddr(i));
+  Bit16u hi2 = read_virtual_word (i->seg(), RMAddr(i) + 8);
 
   Bit64s scale = 1; 
   Bit64s val64 = 0;
@@ -317,7 +309,7 @@ void BX_CPU_C::FST_SINGLE_REAL(bxInstruction_c *i)
         return;
   }
 
-  write_virtual_dword(i->seg(), RMAddr(i), &save_reg);
+  write_virtual_dword(i->seg(), RMAddr(i), save_reg);
 
   if (pop_stack)
      BX_CPU_THIS_PTR the_i387.FPU_pop();
@@ -353,7 +345,7 @@ void BX_CPU_C::FST_DOUBLE_REAL(bxInstruction_c *i)
         return;
   }
 
-  write_virtual_qword(i->seg(), RMAddr(i), &save_reg);
+  write_virtual_qword(i->seg(), RMAddr(i), save_reg);
 
   if (pop_stack)
      BX_CPU_THIS_PTR the_i387.FPU_pop();
@@ -419,7 +411,7 @@ void BX_CPU_C::FIST_WORD_INTEGER(bxInstruction_c *i)
         return;
   }
 
-  write_virtual_word(i->seg(), RMAddr(i), (Bit16u*)(&save_reg));
+  write_virtual_word(i->seg(), RMAddr(i), (Bit16u)(save_reg));
 
   if (pop_stack)
      BX_CPU_THIS_PTR the_i387.FPU_pop();
@@ -457,7 +449,7 @@ void BX_CPU_C::FIST_DWORD_INTEGER(bxInstruction_c *i)
          return;
   }
 
-  write_virtual_dword(i->seg(), RMAddr(i), (Bit32u*)(&save_reg));
+  write_virtual_dword(i->seg(), RMAddr(i), (Bit32u)(save_reg));
 
   if (pop_stack)
      BX_CPU_THIS_PTR the_i387.FPU_pop();
@@ -493,7 +485,7 @@ void BX_CPU_C::FISTP_QWORD_INTEGER(bxInstruction_c *i)
          return;
   }
 
-  write_virtual_qword(i->seg(), RMAddr(i), (Bit64u*)(&save_reg));
+  write_virtual_qword(i->seg(), RMAddr(i), (Bit64u)(save_reg));
   BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
   BX_INFO(("FISTP_QWORD_INTEGER: required FPU, configure --enable-fpu"));
@@ -560,8 +552,8 @@ void BX_CPU_C::FBSTP_PACKED_BCD(bxInstruction_c *i)
   }
 
   // write packed bcd to memory
-  write_virtual_qword(i->seg(), RMAddr(i),     &save_reg_lo);
-  write_virtual_word (i->seg(), RMAddr(i) + 8, &save_reg_hi);
+  write_virtual_qword(i->seg(), RMAddr(i),     save_reg_lo);
+  write_virtual_word (i->seg(), RMAddr(i) + 8, save_reg_hi);
 
   BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
@@ -597,7 +589,7 @@ void BX_CPU_C::FISTTP16(bxInstruction_c *i)
         return;
   }
 
-  write_virtual_word(i->seg(), RMAddr(i), (Bit16u*)(&save_reg));
+  write_virtual_word(i->seg(), RMAddr(i), (Bit16u)(save_reg));
   BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
   BX_INFO(("FISTTP16: required SSE3, use --enable-sse option"));
@@ -633,7 +625,7 @@ void BX_CPU_C::FISTTP32(bxInstruction_c *i)
          return;
   }
 
-  write_virtual_dword(i->seg(), RMAddr(i), (Bit32u*)(&save_reg));
+  write_virtual_dword(i->seg(), RMAddr(i), (Bit32u)(save_reg));
   BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
   BX_INFO(("FISTTP32: required SSE3, use --enable-sse option"));
@@ -669,7 +661,7 @@ void BX_CPU_C::FISTTP64(bxInstruction_c *i)
          return;
   }
 
-  write_virtual_qword(i->seg(), RMAddr(i), (Bit64u*)(&save_reg));
+  write_virtual_qword(i->seg(), RMAddr(i), (Bit64u)(save_reg));
   BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
   BX_INFO(("FISTTP64: required SSE3, use --enable-sse option"));
