@@ -1,9 +1,9 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: status_w.h,v 1.9 2005/05/12 18:07:48 sshwarts Exp $
+// $Id: status_w.h,v 1.11 2007/09/14 19:39:11 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
-//          Written by Stanislav Shwartsman <stl at fidonet.org.il>
+//          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -60,14 +60,8 @@
 #define FPU_EX_Precision_Lost_Up 	(EX_Precision | SW_C1)
 #define FPU_EX_Precision_Lost_Dn        (EX_Precision)
 
-/*
- * bbd: use do {...} while (0) structure instead of using curly brackets
- * inside parens, which most compilers do not like.
- */
-#define setcc(cc) do { 				\
-  FPU_PARTIAL_STATUS &= ~(FPU_SW_CC); 		\
-  FPU_PARTIAL_STATUS |= (cc) & FPU_SW_CC; 	\
-} while(0);
+#define setcc(cc)                               \
+  FPU_PARTIAL_STATUS = (FPU_PARTIAL_STATUS & ~(FPU_SW_CC)) | ((cc) & FPU_SW_CC)
 
 #define clear_C1() { FPU_PARTIAL_STATUS &= ~FPU_SW_C1; }
 #define clear_C2() { FPU_PARTIAL_STATUS &= ~FPU_SW_C2; }
