@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: gui.cc,v 1.100 2007/08/18 08:05:30 vruppert Exp $
+// $Id: gui.cc,v 1.102 2007/10/24 23:07:30 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -39,9 +39,7 @@
 #include "gui/bitmaps/configbutton.h"
 #include "gui/bitmaps/cdromd.h"
 #include "gui/bitmaps/userbutton.h"
-#if BX_SUPPORT_SAVE_RESTORE
 #include "gui/bitmaps/saverestore.h"
-#endif
 
 #if BX_WITH_MACOS
 #  include <Disks.h>
@@ -56,7 +54,7 @@ bx_gui_c *bx_gui = NULL;
 #define N_USER_KEYS 36
 
 typedef struct {
-  char *key;
+  const char *key;
   Bit32u symbol;
 } user_key_t;
 
@@ -150,11 +148,8 @@ void bx_gui_c::init(int argc, char **argv, unsigned tilewidth, unsigned tileheig
   BX_GUI_THIS paste_bmap_id = create_bitmap(bx_paste_bmap, BX_PASTE_BMAP_X, BX_PASTE_BMAP_Y);
   BX_GUI_THIS config_bmap_id = create_bitmap(bx_config_bmap, BX_CONFIG_BMAP_X, BX_CONFIG_BMAP_Y);
   BX_GUI_THIS user_bmap_id = create_bitmap(bx_user_bmap, BX_USER_BMAP_X, BX_USER_BMAP_Y);
-
-#if BX_SUPPORT_SAVE_RESTORE
   BX_GUI_THIS save_restore_bmap_id = create_bitmap(bx_save_restore_bmap,
                           BX_SAVE_RESTORE_BMAP_X, BX_SAVE_RESTORE_BMAP_Y);
-#endif
 
   // Add the initial bitmaps to the headerbar, and enable callback routine, for use
   // when that bitmap is clicked on
@@ -203,11 +198,9 @@ void bx_gui_c::init(int argc, char **argv, unsigned tilewidth, unsigned tileheig
                           BX_GRAVITY_RIGHT, power_handler);
   BX_GUI_THIS set_tooltip(BX_GUI_THIS power_hbar_id, "Turn power off");
   // Save/Restore Button
-#if BX_SUPPORT_SAVE_RESTORE
   BX_GUI_THIS save_restore_hbar_id = headerbar_bitmap(BX_GUI_THIS save_restore_bmap_id,
                           BX_GRAVITY_RIGHT, save_restore_handler);
   BX_GUI_THIS set_tooltip(BX_GUI_THIS save_restore_hbar_id, "Save simulation state");
-#endif
   // Reset button
   BX_GUI_THIS reset_hbar_id = headerbar_bitmap(BX_GUI_THIS reset_bmap_id,
                           BX_GRAVITY_RIGHT, reset_handler);
@@ -604,7 +597,6 @@ void bx_gui_c::userbutton_handler(void)
   }
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_gui_c::save_restore_handler(void)
 {
   int ret;
@@ -628,9 +620,8 @@ void bx_gui_c::save_restore_handler(void)
     }
   }
 }
-#endif
 
-void bx_gui_c::mouse_enabled_changed (bx_bool val)
+void bx_gui_c::mouse_enabled_changed(bx_bool val)
 {
   // This is only called when SIM->get_init_done is 1.  Note that VAL
   // is the new value of mouse_enabled, which may not match the old
