@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sdl.cc,v 1.66 2006/01/26 22:13:20 vruppert Exp $
+// $Id: sdl.cc,v 1.69 2006/10/15 16:23:09 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -319,7 +319,7 @@ void bx_sdl_gui_c::specific_init(
   SDL_WarpMouse(half_res_x, half_res_y);
 
   // load keymap for sdl
-  if(bx_options.keyboard.OuseMapping->get()) {
+  if (SIM->get_param_bool(BXPN_KBD_USEMAPPING)->get()) {
     bx_keymap.loadKeymap(convertStringToSDLKey);
   }
 
@@ -425,17 +425,17 @@ void bx_sdl_gui_c::text_update(
     unsigned nrows)
 {
   Bit8u *pfont_row, *old_line, *new_line, *text_base;
-  unsigned int cs_y, x, y;
+  unsigned int cs_y, i, x, y;
   unsigned int curs, hchars, offset;
   Bit8u fontline, fontpixels, fontrows;
-  int rows, fgcolor_ndx, bgcolor_ndx;
-  Uint32 fgcolor;
-  Uint32 bgcolor;
+  int rows;
+  Uint32 fgcolor, bgcolor;
   Uint32 *buf, *buf_row, *buf_char;
   Uint32 disp;
   Bit16u font_row, mask;
   Bit8u cfstart, cfwidth, cfheight, split_fontrows, split_textrow;
   bx_bool cursor_visible, gfxcharw9, invert, forceUpdate, split_screen;
+  Uint32 text_palette[16];
 
   UNUSED(nrows);
   forceUpdate = 0;
@@ -444,6 +444,10 @@ void bx_sdl_gui_c::text_update(
     forceUpdate = 1;
     charmap_updated = 0;
   }
+  for (i=0; i<16; i++)
+  {
+    text_palette[i] = palette[DEV_vga_get_actl_pal_idx(i)];
+  }
   if((tm_info.h_panning != h_panning) || (tm_info.v_panning != v_panning))
   {
     forceUpdate = 1;
@@ -545,12 +549,10 @@ void bx_sdl_gui_c::text_update(
       {
 
 	// Get Foreground/Background pixel colors
-	fgcolor_ndx = DEV_vga_get_actl_pal_idx(new_text[1] & 0x0F);
-	bgcolor_ndx = DEV_vga_get_actl_pal_idx((new_text[1] >> 4) & 0x0F);
-	fgcolor = palette[fgcolor_ndx];
-	bgcolor = palette[bgcolor_ndx];
-	invert = ( (offset == curs) && (cursor_visible) );
-	gfxcharw9 = ( (tm_info.line_graphics) && ((new_text[0] & 0xE0) == 0xC0) );
+	fgcolor = text_palette[new_text[1] & 0x0F];
+	bgcolor = text_palette[(new_text[1] >> 4) & 0x0F];
+	invert = ((offset == curs) && (cursor_visible));
+	gfxcharw9 = ((tm_info.line_graphics) && ((new_text[0] & 0xE0) == 0xC0));
 
 	// Display this one char
 	fontrows = cfheight;
@@ -914,7 +916,7 @@ static Bit32u sdl_sym_to_bx_key (SDLKey sym)
     case SDLK_RALT:                 return BX_KEY_ALT_R;
     case SDLK_LALT:                 return BX_KEY_ALT_L;
     case SDLK_RMETA:                return BX_KEY_ALT_R;
-//  case SDLK_LMETA:                return BX_KEY_LMETA;
+    case SDLK_LMETA:                return BX_KEY_WIN_L;
     case SDLK_LSUPER:               return BX_KEY_WIN_L;
     case SDLK_RSUPER:               return BX_KEY_WIN_R;
 //  case SDLK_MODE:                 return BX_KEY_MODE;
@@ -1061,7 +1063,7 @@ void bx_sdl_gui_c::handle_events(void)
 
 	// convert sym->bochs code
 	if( sdl_event.key.keysym.sym > SDLK_LAST ) break;
-        if (!bx_options.keyboard.OuseMapping->get()) {
+        if (!SIM->get_param_bool(BXPN_KBD_USEMAPPING)->get()) {
 	  key_event = sdl_sym_to_bx_key (sdl_event.key.keysym.sym);
 	  BX_DEBUG (("keypress scancode=%d, sym=%d, bx_key = %d", sdl_event.key.keysym.scancode, sdl_event.key.keysym.sym, key_event));
 	} else {
@@ -1089,7 +1091,7 @@ void bx_sdl_gui_c::handle_events(void)
 	    && (sdl_event.key.keysym.sym < SDLK_LAST ))
 	{
 	  // convert sym->bochs code
-          if (!bx_options.keyboard.OuseMapping->get()) {
+          if (!SIM->get_param_bool(BXPN_KBD_USEMAPPING)->get()) {
             key_event = sdl_sym_to_bx_key (sdl_event.key.keysym.sym);
           } else {
             /* use mapping */
