@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: siminterface.h,v 1.199 2006/06/21 20:42:26 sshwarts Exp $
+// $Id: siminterface.h,v 1.214 2007/10/24 23:09:13 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 // Intro to siminterface by Bryce Denney:
@@ -120,6 +120,12 @@ typedef enum {
 // define parameter path names.  These names give the location in the
 // parameter tree where each can be found.  The names correspond to
 // the old BXP_* enum values, which have been eliminated.
+#define BXPN_SEL_CONFIG_INTERFACE        "general.config_interface"
+#define BXPN_BOCHS_START                 "general.start_mode"
+#define BXPN_BOCHS_BENCHMARK             "general.benchmark"
+#define BXPN_RESTORE_FLAG                "general.restore"
+#define BXPN_RESTORE_PATH                "general.restore_path"
+#define BXPN_DEBUG_RUNNING               "general.debug_running"
 #define BXPN_CPU_NPROCESSORS             "cpu.n_processors"
 #define BXPN_CPU_NCORES                  "cpu.n_cores"
 #define BXPN_CPU_NTHREADS                "cpu.n_threads"
@@ -154,7 +160,6 @@ typedef enum {
 #define BXPN_I440FX_SUPPORT              "pci.i440fx_support"
 #define BXPN_PCIDEV_VENDOR               "pci.pcidev.vendor"
 #define BXPN_PCIDEV_DEVICE               "pci.pcidev.device"
-#define BXPN_SEL_CONFIG_INTERFACE        "display.config_interface"
 #define BXPN_SEL_DISPLAY_LIBRARY         "display.display_library"
 #define BXPN_DISPLAYLIB_OPTIONS          "display.displaylib_options"
 #define BXPN_PRIVATE_COLORMAP            "display.private_colormap"
@@ -204,9 +209,7 @@ typedef enum {
 #define BXPN_ATA3_SLAVE                  "ata.3.slave"
 #define BXPN_USB1_ENABLED                "ports.usb.1.enabled"
 #define BXPN_USB1_PORT1                  "ports.usb.1.port1"
-#define BXPN_USB1_OPTION1                "ports.usb.1.option1"
 #define BXPN_USB1_PORT2                  "ports.usb.1.port2"
-#define BXPN_USB1_OPTION2                "ports.usb.1.option2"
 #define BXPN_NE2K                        "network.ne2k"
 #define BXPN_NE2K_ENABLED                "network.ne2k.enabled"
 #define BXPN_PNIC                        "network.pnic"
@@ -222,10 +225,6 @@ typedef enum {
 #define BXPN_LOG_FILENAME                "log.filename"
 #define BXPN_LOG_PREFIX                  "log.prefix"
 #define BXPN_DEBUGGER_LOG_FILENAME       "log.debugger_filename"
-#define BXPN_BOCHS_START                 "general.start_mode"
-#define BXPN_DEBUG_RUNNING               "general.debug_running"
-#define BXPN_RESTORE_FLAG                "general.restore"
-#define BXPN_RESTORE_PATH                "general.restore_path"
 #define BXPN_MENU_DISK                   "menu.disk"
 #define BXPN_MENU_MEMORY                 "menu.memory"
 #define BXPN_MENU_RUNTIME                "menu.runtime"
@@ -244,9 +243,7 @@ typedef enum {
   BX_TOOLBAR_CDROMD,
   BX_TOOLBAR_RESET,
   BX_TOOLBAR_POWER,
-#if BX_SUPPORT_SAVE_RESTORE
   BX_TOOLBAR_SAVE_RESTORE,
-#endif
   BX_TOOLBAR_COPY,
   BX_TOOLBAR_PASTE,
   BX_TOOLBAR_SNAPSHOT,
@@ -278,6 +275,7 @@ typedef enum {
 #define BX_BOOT_FLOPPYA 1
 #define BX_BOOT_DISKC   2
 #define BX_BOOT_CDROM   3
+#define BX_BOOT_NETWORK 4
 
 // loader hack
 #define Load32bitOSNone        0
@@ -641,9 +639,7 @@ public:
 };
 
 typedef Bit64s (*param_event_handler)(class bx_param_c *, int set, Bit64s val);
-#if BX_SUPPORT_SAVE_RESTORE
 typedef Bit64s (*param_sr_handler)(void *devptr, class bx_param_c *, Bit64s val);
-#endif
 typedef int (*param_enable_handler)(class bx_param_c *, int en);
 
 class BOCHSAPI bx_param_num_c : public bx_param_c {
@@ -665,11 +661,9 @@ protected:
     bx_bool *pbool;  // used by bx_shadow_bool_c
   } val;
   param_event_handler handler;
-#if BX_SUPPORT_SAVE_RESTORE
   void *sr_devptr;
   param_sr_handler save_handler;
   param_sr_handler restore_handler;
-#endif
   param_enable_handler enable_handler;
   int base;
   Bit32u options;
@@ -686,12 +680,10 @@ public:
       const char *description,
       Bit64s min, Bit64s max, Bit64s initial_val,
       bx_bool is_shadow = 0);
-  virtual void reset();
+  virtual void reset() { val.number = initial_val; }
   void set_handler(param_event_handler handler);
-#if BX_SUPPORT_SAVE_RESTORE
   void set_sr_handlers(void *devptr, param_sr_handler save, param_sr_handler restore);
-#endif
-  void set_enable_handler(param_enable_handler handler);
+  void set_enable_handler(param_enable_handler handler) { enable_handler = handler; }
   virtual bx_list_c *get_dependent_list() { return dependent_list; }
   void set_dependent_list(bx_list_c *l);
   virtual void set_enabled(int enabled);
@@ -815,17 +807,17 @@ public:
 
 
 class BOCHSAPI bx_param_enum_c : public bx_param_num_c {
-  char **choices;
+  const char **choices;
 public:
   bx_param_enum_c(bx_param_c *parent,
       const char *name,
       const char *label,
       const char *description,
-      char **choices,
+      const char **choices,
       Bit64s initial_val,
       Bit64s value_base = 0);
-  char *get_choice(int n) { return choices[n]; }
-  char *get_selected() { return choices[val.number - min]; }
+  const char *get_choice(int n) { return choices[n]; }
+  const char *get_selected() { return choices[val.number - min]; }
   int find_by_name(const char *string);
   bx_bool set_by_name(const char *string);
 #if BX_USE_TEXTCONFIG
@@ -849,7 +841,8 @@ public:
     IS_FILENAME = 2,       // 1=yes it's a filename, 0=not a filename.
                            // Some guis have a file browser. This
                            // bit suggests that they use it.
-    SAVE_FILE_DIALOG = 4   // Use save dialog opposed to open file dialog
+    SAVE_FILE_DIALOG = 4,  // Use save dialog opposed to open file dialog
+    SELECT_FOLDER_DLG = 8  // Use folder selection dialog
   } bx_string_opt_bits;
   bx_param_string_c(bx_param_c *parent,
       const char *name,
@@ -890,7 +883,6 @@ public:
       int maxsize=-1);
 };
 
-#if BX_SUPPORT_SAVE_RESTORE
 class BOCHSAPI bx_shadow_data_c : public bx_param_c {
   Bit32u data_size;
   Bit8u *data_ptr;
@@ -902,7 +894,6 @@ public:
   Bit8u *getptr() {return data_ptr;}
   Bit32u get_size() const {return data_size;}
 };
-#endif
 
 #define BX_DEFAULT_LIST_SIZE 6
 
@@ -947,10 +938,10 @@ public:
     // item (used in the runtime menu).
     SHOW_GROUP_NAME = (1<<4)
   } bx_listopt_bits;
-  bx_list_c(bx_param_c *parent, int maxsize = BX_DEFAULT_LIST_SIZE);
-  bx_list_c(bx_param_c *parent, const char *name, int maxsize = BX_DEFAULT_LIST_SIZE);
-  bx_list_c(bx_param_c *parent, const char *name, char *title, int maxsize = BX_DEFAULT_LIST_SIZE);
-  bx_list_c(bx_param_c *parent, const char *name, char *title, bx_param_c **init_list);
+  bx_list_c(bx_param_c *parent, int maxsize);
+  bx_list_c(bx_param_c *parent, const char *name, int maxsize);
+  bx_list_c(bx_param_c *parent, const char *name, const char *title, int maxsize = BX_DEFAULT_LIST_SIZE);
+  bx_list_c(bx_param_c *parent, const char *name, const char *title, bx_param_c **init_list);
   virtual ~bx_list_c();
   bx_list_c *clone();
   void add(bx_param_c *param);
@@ -963,6 +954,7 @@ public:
   void set_parent(bx_param_c *newparent);
   bx_param_c *get_parent() { return parent; }
   virtual void reset();
+  virtual void clear();
 #if BX_USE_TEXTCONFIG
   virtual void text_print(FILE *);
   virtual int text_ask(FILE *fpin, FILE *fpout);
@@ -995,12 +987,10 @@ enum {
   BX_MOUSE_TYPE_IMPS2,
 #if BX_SUPPORT_BUSMOUSE
   BX_MOUSE_TYPE_BUS,
-#endif
-#if BX_SUPPORT_PCIUSB
-  BX_MOUSE_TYPE_USB,
 #endif
   BX_MOUSE_TYPE_SERIAL,
-  BX_MOUSE_TYPE_SERIAL_WHEEL
+  BX_MOUSE_TYPE_SERIAL_WHEEL,
+  BX_MOUSE_TYPE_SERIAL_MSYS
 };
 
 #define BX_FLOPPY_NONE   10 // floppy not present
@@ -1038,12 +1028,13 @@ enum {
 #define BX_ATA_MODE_DLL_HD      3
 #define BX_ATA_MODE_SPARSE      4
 #define BX_ATA_MODE_VMWARE3     5
-#define BX_ATA_MODE_UNDOABLE    6
-#define BX_ATA_MODE_GROWING     7
-#define BX_ATA_MODE_VOLATILE    8
-#define BX_ATA_MODE_Z_UNDOABLE  9
-#define BX_ATA_MODE_Z_VOLATILE  10
-#define BX_ATA_MODE_LAST        10
+#define BX_ATA_MODE_VMWARE4     6
+#define BX_ATA_MODE_UNDOABLE    7
+#define BX_ATA_MODE_GROWING     8
+#define BX_ATA_MODE_VOLATILE    9
+#define BX_ATA_MODE_Z_UNDOABLE  10
+#define BX_ATA_MODE_Z_VOLATILE  11
+#define BX_ATA_MODE_LAST        11
 
 #define BX_CLOCK_SYNC_NONE     0
 #define BX_CLOCK_SYNC_REALTIME 1
@@ -1054,31 +1045,19 @@ enum {
 #define BX_CLOCK_TIME0_LOCAL     1
 #define BX_CLOCK_TIME0_UTC       2
 
-BOCHSAPI extern char *bochs_start_names[];
-BOCHSAPI extern int n_bochs_start_names;
-BOCHSAPI extern char *floppy_type_names[];
+BOCHSAPI extern const char *bochs_start_names[];
+BOCHSAPI extern const char *floppy_type_names[];
 BOCHSAPI extern int floppy_type_n_sectors[];
-BOCHSAPI extern int n_floppy_type_names;
-BOCHSAPI extern char *floppy_status_names[];
-BOCHSAPI extern int n_floppy_status_names;
-BOCHSAPI extern char *bochs_bootdisk_names[];
-BOCHSAPI extern int n_bochs_bootdisk_names;
-BOCHSAPI extern char *loader_os_names[];
-BOCHSAPI extern int n_loader_os_names;
-BOCHSAPI extern char *keyboard_type_names[];
-BOCHSAPI extern int n_keyboard_type_names;
-BOCHSAPI extern char *atadevice_type_names[];
-BOCHSAPI extern int n_atadevice_type_names;
-BOCHSAPI extern char *atadevice_mode_names[];
-BOCHSAPI extern int n_atadevice_mode_names;
-BOCHSAPI extern char *atadevice_status_names[];
-BOCHSAPI extern int n_atadevice_status_names;
-BOCHSAPI extern char *atadevice_biosdetect_names[];
-BOCHSAPI extern int n_atadevice_biosdetect_names;
-BOCHSAPI extern char *atadevice_translation_names[];
-BOCHSAPI extern int n_atadevice_translation_names;
-BOCHSAPI extern char *clock_sync_names[];
-BOCHSAPI extern int clock_sync_n_names;
+BOCHSAPI extern const char *floppy_status_names[];
+BOCHSAPI extern const char *bochs_bootdisk_names[];
+BOCHSAPI extern const char *loader_os_names[];
+BOCHSAPI extern const char *keyboard_type_names[];
+BOCHSAPI extern const char *atadevice_type_names[];
+BOCHSAPI extern const char *atadevice_mode_names[];
+BOCHSAPI extern const char *atadevice_status_names[];
+BOCHSAPI extern const char *atadevice_biosdetect_names[];
+BOCHSAPI extern const char *atadevice_translation_names[];
+BOCHSAPI extern const char *clock_sync_names[];
 
 ////////////////////////////////////////////////////////////////////
 // base class simulator interface, contains just virtual functions.
@@ -1087,7 +1066,7 @@ BOCHSAPI extern int clock_sync_n_names;
 
 #include <setjmp.h>
 
-enum ci_command_t { CI_START, CI_RUNTIME_CONFIG, CI_SAVE_RESTORE, CI_SHUTDOWN };
+enum ci_command_t { CI_START, CI_RUNTIME_CONFIG, CI_SHUTDOWN };
 enum ci_return_t { 
   CI_OK,                  // normal return value 
   CI_ERR_NO_TEXT_CONSOLE  // err: can't work because there's no text console
@@ -1110,6 +1089,7 @@ enum disp_mode_t { DISP_MODE_CONFIG=100, DISP_MODE_SIM };
 class BOCHSAPI bx_simulator_interface_c {
 public:
   bx_simulator_interface_c() {}
+  virtual ~bx_simulator_interface_c() {}
   virtual void set_quit_context(jmp_buf *context) {}
   virtual int get_init_done() { return -1; }
   virtual int set_init_done(int n) {return -1;}
@@ -1177,9 +1157,9 @@ public:
   virtual int ask_param(const char *pname) {return -1;}
 
   // ask the user for a pathname
-  virtual int ask_filename(char *filename, int maxlen, char *prompt, char *the_default, int flags) {return -1;}
+  virtual int ask_filename(const char *filename, int maxlen, const char *prompt, const char *the_default, int flags) {return -1;}
   // yes/no dialog
-  virtual int ask_yes_no(char *title, char *prompt, bx_bool the_default) {return -1;}
+  virtual int ask_yes_no(const char *title, const char *prompt, bx_bool the_default) {return -1;}
   // called at a regular interval, currently by the keyboard handler.
   virtual void periodic() {}
   virtual int create_disk_image(const char *filename, int sectors, bx_bool overwrite) {return -3;}
@@ -1218,7 +1198,8 @@ public:
     is_sim_thread_func = func;
   }
   virtual bx_bool is_sim_thread() {return 1;}
-  virtual bx_bool is_wx_selected() {return 0;}
+  virtual void set_debug_gui(bx_bool val) {}
+  virtual bx_bool has_debug_gui() {return 0;}
   // provide interface to bx_gui->set_display_mode() method for config
   // interfaces to use.
   virtual void set_display_mode(disp_mode_t newmode) {}
@@ -1228,14 +1209,14 @@ public:
   virtual bx_bool register_user_option(const char *keyword, user_option_parser_t parser, user_option_save_t save_func) {return 0;}
   virtual Bit32s parse_user_option(int idx, const char *context, int num_params, char *params []) {return -1;}
   virtual Bit32s save_user_options(FILE *fp) {return -1;}
-#if BX_SUPPORT_SAVE_RESTORE
   // save/restore support
+  virtual void init_save_restore() {}
   virtual bx_bool save_state(const char *checkpoint_path) {return 0;}
   virtual bx_bool restore_config() {return 0;}
   virtual bx_bool restore_logopts() {return 0;}
   virtual bx_bool restore_hardware() {return 0;}
-  virtual bx_list_c *get_sr_root() {return NULL;}
-#endif 
+  virtual bx_list_c *get_bochs_root() {return NULL;}
+  virtual bx_bool restore_bochs_param(bx_list_c *root, const char *sr_path, const char *restore_name) { return 0; }
 };
 
 BOCHSAPI extern bx_simulator_interface_c *SIM;
