@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: win32dialog.h,v 1.9 2006/06/08 17:02:51 vruppert Exp $
+// $Id: win32dialog.h,v 1.11 2006/11/17 16:50:39 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 
 #include "config.h"
@@ -7,8 +7,8 @@
 #if BX_USE_TEXTCONFIG && defined(WIN32)
 
 int RuntimeOptionsDialog();
-#if BX_SUPPORT_SAVE_RESTORE
-bx_bool win32SaveState();
+#if BX_DEBUGGER
+void InitDebugDialog(HWND mainwnd);
 #endif
 void win32_init_notify_callback();
 
