@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////
-// $Id: wxdialog.cc,v 1.100 2006/05/30 17:41:43 vruppert Exp $
+// $Id: wxdialog.cc,v 1.103 2007/11/02 16:58:46 vruppert Exp $
 /////////////////////////////////////////////////////////////////
 
 // Define BX_PLUGGABLE in files that can be compiled into plugins.  For
@@ -118,38 +118,35 @@ void LogMsgAskDialog::Init()
     btnSizer->Add(btn, 1, wxALL, 5);
   }
   wxSize ms = message->GetSize();
-  // wxLogMessage(wxT("message size is %d,%d"), ms.GetWidth(), ms.GetHeight());
   SetAutoLayout(TRUE);
   SetSizer(vertSizer);
   vertSizer->Fit(this);
   wxSize size = vertSizer->GetMinSize();
-  // wxLogMessage(wxT("minsize is %d,%d"), size.GetWidth(), size.GetHeight());
   int margin = 10;
   SetSizeHints (size.GetWidth () + margin, size.GetHeight () + margin);
   Center ();
 }
 
 // Event handler for dialog buttons. Just translate the wx ids into
-// enum values and return them with EndModel() to make the dialog
+// enum values and return them with EndModal() to make the dialog
 // go away.
 void LogMsgAskDialog::OnEvent(wxCommandEvent& event)
 {
-  int id = event.GetId ();
+  int id = event.GetId();
   int ret = -1;
   switch (id) {
     case ID_Continue:   ret = BX_LOG_ASK_CHOICE_CONTINUE;  break;
     case ID_Die:        ret = BX_LOG_ASK_CHOICE_DIE;   break;
     case ID_DumpCore:   ret = BX_LOG_ASK_CHOICE_DUMP_CORE;  break;
     case ID_Debugger:   ret = BX_LOG_ASK_CHOICE_ENTER_DEBUG; break;
-    case wxID_HELP: ShowHelp (); return;
+    case wxID_HELP: ShowHelp(); return;
     default:
       return;  // without EndModal
   }
-  // wxLogMessage(wxT("you pressed button id=%d, return value=%d"), id, ret);
-  EndModal (ret);
+  EndModal(ret);
 }
 
-void LogMsgAskDialog::ShowHelp ()
+void LogMsgAskDialog::ShowHelp()
 {
   wxMessageBox(MSG_NO_HELP, MSG_NO_HELP_CAPTION, wxOK | wxICON_ERROR, this );
 }
@@ -247,20 +244,23 @@ void FloppyConfigDialog::AddRadio (
   n_rbtns++;
 }
 
-void FloppyConfigDialog::SetDriveName (wxString name)
+void FloppyConfigDialog::SetDriveName(wxString name)
 {
   SetTitle(wxString(FLOPPY_CONFIG_TITLE) + name);
   ChangeStaticText(vertSizer, instr, wxString(FLOPPY_CONFIG_INSTRS) + name +
     wxT("."));
 }
 
-void FloppyConfigDialog::SetCapacityChoices(int n, char *choices[])
+void FloppyConfigDialog::SetCapacityChoices(const char *choices[])
 {
-  for (int i=0; i<n; i++)
+  int i = 0;
+  while (choices[i] != NULL) {
     capacity->Append(wxString(choices[i], wxConvUTF8));
+    i++;
+  }
 }
 
-void FloppyConfigDialog::SetCapacity (int cap)
+void FloppyConfigDialog::SetCapacity(int cap)
 {
   capacity->SetSelection(cap);
   CreateBtn->Enable(floppy_type_n_sectors[cap] > 0);
@@ -279,7 +279,6 @@ void FloppyConfigDialog::Init()
   SetSizer(vertSizer);
   vertSizer->Fit(this);
   wxSize size = vertSizer->GetMinSize();
-  // wxLogMessage(wxT("minsize is %d,%d"), size.GetWidth(), size.GetHeight());
   int margin = 5;
   SetSizeHints (size.GetWidth() + margin, size.GetHeight() + margin);
   Center();
@@ -469,13 +468,17 @@ AdvancedLogOptionsDialog::AdvancedLogOptionsDialog(
   int devmax = SIM->get_n_log_modules(); 
   action = new wxChoice** [devmax];   // array of pointers
   for (int dev=0; dev<devmax; dev++) {
-    action[dev] = new wxChoice* [ADVLOG_OPTS_N_TYPES];
-    // name of device in first column
-    gridSizer->Add(new wxStaticText(scrollPanel, -1, wxString(SIM->get_prefix(dev), wxConvUTF8)));
-    // wxChoice in every other column
-    for (type=0; type < typemax; type++) {
-      action[dev][type] = makeLogOptionChoiceBox(scrollPanel, -1, type);
-      gridSizer->Add(action[dev][type], 1, wxALL|wxGROW|wxADJUST_MINSIZE, 2);
+    if (strcmp(SIM->get_prefix(dev), "[     ]")) {
+      action[dev] = new wxChoice* [ADVLOG_OPTS_N_TYPES];
+      // name of device in first column
+      gridSizer->Add(new wxStaticText(scrollPanel, -1, wxString(SIM->get_prefix(dev), wxConvUTF8)));
+      // wxChoice in every other column
+      for (type=0; type < typemax; type++) {
+        action[dev][type] = makeLogOptionChoiceBox(scrollPanel, -1, type);
+        gridSizer->Add(action[dev][type], 1, wxALL|wxGROW|wxADJUST_MINSIZE, 2);
+      }
+    } else {
+      action[dev] = NULL;
     }
   }
   headerSizer->Fit(this);
@@ -530,7 +533,6 @@ void AdvancedLogOptionsDialog::Init()
   SetSizer(vertSizer);
   vertSizer->Fit(this);
   wxSize size = vertSizer->GetMinSize();
-  // wxLogMessage(wxT("minsize is %d,%d"), size.GetWidth(), size.GetHeight());
   int margin = 5;
   SetSizeHints(size.GetWidth() + margin, size.GetHeight() + margin);
   Center();
@@ -565,6 +567,7 @@ void AdvancedLogOptionsDialog::CopyGuiToParam() {
 }
 
 void AdvancedLogOptionsDialog::SetAction(int dev, int evtype, int act) {
+  if (action[dev] == NULL) return;
   // find the choice whose client data matches "act".
   int *ptr;
   // wxLogDebug(wxT("SetAction dev=%d type=%d act=%d"), dev, evtype, act);
@@ -584,6 +587,7 @@ void AdvancedLogOptionsDialog::SetAction(int dev, int evtype, int act) {
 }
 
 int AdvancedLogOptionsDialog::GetAction(int dev, int evtype) {
+  if (action[dev] == NULL) return LOG_OPTS_NO_CHANGE;
   int sel = action[dev][evtype]->GetSelection(); 
   int *ptrToChoice = (int*)action[dev][evtype]->GetClientData(sel);
   wxASSERT(ptrToChoice != NULL);
@@ -593,7 +597,6 @@ int AdvancedLogOptionsDialog::GetAction(int dev, int evtype) {
 void AdvancedLogOptionsDialog::OnEvent(wxCommandEvent& event)
 {
   int id = event.GetId();
-  // wxLogMessage(wxT("you pressed button id=%d"), id);
   switch (id) {
     case ID_Browse:
       BrowseTextCtrl(logfile);
@@ -701,7 +704,6 @@ void DebugLogDialog::Init()
   SetSizer(mainSizer);
   mainSizer->Fit(this);
   wxSize size = mainSizer->GetMinSize();
-  // wxLogMessage(wxT("minsize is %d,%d"), size.GetWidth(), size.GetHeight());
   int margin = 5;
   SetSizeHints(size.GetWidth() + margin, size.GetHeight() + margin);
   Center();
@@ -765,7 +767,6 @@ void DebugLogDialog::AppendText (wxString text) {
 void DebugLogDialog::OnEvent(wxCommandEvent& event)
 {
   int id = event.GetId();
-  //wxLogMessage(wxT("event was from id=%d, type=%d"), id, (int)event.GetEventType ());
   switch (id) {
     case wxID_OK:
       Show(FALSE);
@@ -857,7 +858,6 @@ void ParamDialog::Init()
   SetSizer(mainSizer);
   mainSizer->Fit(this);
   wxSize size = mainSizer->GetMinSize();
-  // wxLogMessage(wxT("minsize is %d,%d"), size.GetWidth(), size.GetHeight());
   int margin = 5;
   SetSizeHints(size.GetWidth() + margin, size.GetHeight() + margin);
   Center();
@@ -989,7 +989,7 @@ void ParamDialog::AddParam (
 	if (!plain) sizer->Add (1, 1);  // spacer
 	// fill in the choices
 	int i=0;
-	char *ptr;
+	const char *ptr;
 	while (NULL != (ptr = param->get_choice(i++)))
 	  choice->Append(wxString(ptr, wxConvUTF8));
 	choice->SetSelection (param->get() - param->get_min());
@@ -1448,10 +1448,9 @@ void ParamDialog::CopyParamToGui ()
 
 void ParamDialog::OnEvent(wxCommandEvent& event)
 {
-  int id = event.GetId ();
-  //wxLogMessage ("event was from id=%d", id);
-  if (isGeneratedId (id)) {
-    ParamStruct *pstr = (ParamStruct*) idHash->Get (id);
+  int id = event.GetId();
+  if (isGeneratedId(id)) {
+    ParamStruct *pstr = (ParamStruct*) idHash->Get(id);
     if (pstr == NULL) {
       wxLogDebug(wxT("ParamStruct not found for id=%d"), id);
       return;
