@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.cc,v 1.15 2005/11/14 18:25:41 sshwarts Exp $
+// $Id: instrument.cc,v 1.20 2007/12/13 21:53:55 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,13 +25,15 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 
-
 #include "bochs.h"
 
+#if BX_INSTRUMENTATION
+
 void bx_instr_init(unsigned cpu) {}
 void bx_instr_shutdown(unsigned cpu) {}
 void bx_instr_reset(unsigned cpu) {}
 void bx_instr_hlt(unsigned cpu) {}
+void bx_instr_mwait(unsigned cpu, bx_phy_address addr, unsigned len, Bit32u flags) {}
 void bx_instr_new_instruction(unsigned cpu) {}
 
 void bx_instr_debug_promt() {}
@@ -53,7 +55,7 @@ void bx_instr_interrupt(unsigned cpu, unsigned vector) {}
 void bx_instr_exception(unsigned cpu, unsigned vector) {}
 void bx_instr_hwinterrupt(unsigned cpu, unsigned vector, Bit16u cs, bx_address eip) {}
 
-void bx_instr_tlb_cntrl(unsigned cpu, unsigned what, Bit32u newval) {}
+void bx_instr_tlb_cntrl(unsigned cpu, unsigned what, bx_phy_address new_cr3) {}
 void bx_instr_cache_cntrl(unsigned cpu, unsigned what) {}
 void bx_instr_prefetch_hint(unsigned cpu, unsigned what, unsigned seg, bx_address offset) {}
 
@@ -66,13 +68,14 @@ void bx_instr_outp(Bit16u addr, unsigned len) {}
 void bx_instr_inp2(Bit16u addr, unsigned len, unsigned val) {}
 void bx_instr_outp2(Bit16u addr, unsigned len, unsigned val) {}
 
-void bx_instr_mem_code(unsigned cpu, bx_address linear, unsigned size) {}
-void bx_instr_mem_data(unsigned cpu, bx_address linear, unsigned size, unsigned rw) {}
+void bx_instr_mem_code(unsigned cpu, bx_address linear, unsigned len) {}
+void bx_instr_mem_data(unsigned cpu, bx_address linear, unsigned len, unsigned rw) {}
 
-void bx_instr_lin_read(unsigned cpu, bx_address lin, bx_address phy, unsigned len) {}
-void bx_instr_lin_write(unsigned cpu, bx_address lin, bx_address phy, unsigned len) {}
+void bx_instr_lin_access(unsigned cpu, bx_address lin, bx_address phy, unsigned len, unsigned rw) {}
 
 void bx_instr_phy_write(unsigned cpu, bx_address addr, unsigned len) {}
 void bx_instr_phy_read(unsigned cpu, bx_address addr, unsigned len) {}
 
 void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value) {}
+
+#endif
