@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: biosdev.cc,v 1.10 2005/10/23 13:23:49 vruppert Exp $
+// $Id: biosdev.cc,v 1.11 2006/09/10 17:18:44 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -45,57 +45,49 @@
 
 #include "iodev.h"
 
-bx_biosdev_c *theBiosDevice;
+bx_biosdev_c *theBiosDevice = NULL;
 
-  int
-libbiosdev_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
+logfunctions  *bioslog;
+logfunctions  *vgabioslog;
+
+int libbiosdev_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
-  theBiosDevice = new bx_biosdev_c ();
+  theBiosDevice = new bx_biosdev_c();
   bx_devices.pluginBiosDevice = theBiosDevice;
   BX_REGISTER_DEVICE_DEVMODEL(plugin, type, theBiosDevice, BX_PLUGIN_BIOSDEV);
   return(0); // Success
 }
 
-  void
-libbiosdev_LTX_plugin_fini(void)
+void libbiosdev_LTX_plugin_fini(void)
 {
+  delete theBiosDevice;
 }
 
-logfunctions  *bioslog;
-logfunctions  *vgabioslog;
-
 bx_biosdev_c::bx_biosdev_c(void)
 {
   bioslog = new logfunctions();
   bioslog->put("BIOS");
   bioslog->settype(BIOSLOG);
-  s.bios_message_i = 0;
-  s.bios_panic_flag = 0;
 
   vgabioslog = new logfunctions();
   vgabioslog->put("VBIOS");
   vgabioslog->settype(BIOSLOG);
-  s.vgabios_message_i = 0;
-  s.vgabios_panic_flag = 0;
 }
 
 bx_biosdev_c::~bx_biosdev_c(void)
 {
-    if ( bioslog != NULL )
-    {
-        delete bioslog;
-        bioslog = NULL;
-    }
-
-    if ( vgabioslog != NULL )
-    {
-        delete vgabioslog;
-        vgabioslog = NULL;
-    }
+  bioslog->ldebug("Exit");
+  if (bioslog != NULL) {
+    delete bioslog;
+    bioslog = NULL;
+  }
+  if (vgabioslog != NULL) {
+    delete vgabioslog;
+    vgabioslog = NULL;
+  }
 }
 
-  void
-bx_biosdev_c::init(void)
+void bx_biosdev_c::init(void)
 {
   DEV_register_iowrite_handler(this, write_handler, 0x0400, "Bios Panic Port 1", 3);
   DEV_register_iowrite_handler(this, write_handler, 0x0401, "Bios Panic Port 2", 3);
@@ -106,18 +98,17 @@ bx_biosdev_c::init(void)
   DEV_register_iowrite_handler(this, write_handler, 0x0501, "VGABios Panic Port 1", 3);
   DEV_register_iowrite_handler(this, write_handler, 0x0502, "VGABios Panic Port 2", 3);
   DEV_register_iowrite_handler(this, write_handler, 0x0503, "VGABios Debug Port", 1);
-}
 
-  void
-bx_biosdev_c::reset(unsigned type)
-{
+  s.bios_message_i = 0;
+  s.bios_panic_flag = 0;
+  s.vgabios_message_i = 0;
+  s.vgabios_panic_flag = 0;
 }
 
   // static IO port write callback handler
   // redirects to non-static class handler to avoid virtual functions
 
-  void
-bx_biosdev_c::write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len)
+void bx_biosdev_c::write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len)
 {
 #if !BX_USE_BIOS_SMF
   bx_biosdev_c *class_ptr = (bx_biosdev_c *) this_ptr;
@@ -125,8 +116,7 @@ bx_biosdev_c::write_handler(void *this_ptr, Bit32u address, Bit32u value, unsign
   class_ptr->write(address, value, io_len);
 }
 
-  void
-bx_biosdev_c::write(Bit32u address, Bit32u value, unsigned io_len)
+void bx_biosdev_c::write(Bit32u address, Bit32u value, unsigned io_len)
 {
 #else
   UNUSED(this_ptr);
