@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cmos.cc,v 1.56 2006/05/29 22:33:38 sshwarts Exp $
+// $Id: cmos.cc,v 1.62 2007/09/28 19:51:59 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -101,14 +101,17 @@ Bit8u bin_to_bcd(Bit8u value, bx_bool is_binary)
 
 int libcmos_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
-  theCmosDevice = new bx_cmos_c ();
+  theCmosDevice = new bx_cmos_c();
   bx_devices.pluginCmosDevice = theCmosDevice;
   BX_REGISTER_DEVICE_DEVMODEL(plugin, type, theCmosDevice, BX_PLUGIN_CMOS);
   return(0); // Success
 }
 
 void libcmos_LTX_plugin_fini(void)
-{
+{ if (theCmosDevice != NULL)
+  { delete theCmosDevice;
+    theCmosDevice = NULL;
+  }
 }
 
 bx_cmos_c::bx_cmos_c(void)
@@ -123,11 +126,21 @@ bx_cmos_c::bx_cmos_c(void)
   s.uip_timer_index = BX_NULL_TIMER_HANDLE;
 }
 
-bx_cmos_c::~bx_cmos_c() {}
+bx_cmos_c::~bx_cmos_c(void)
+{
+  save_image();
+  char *tmptime;
+  if ((tmptime = strdup(ctime(&(BX_CMOS_THIS s.timeval)))) != NULL) {
+    tmptime[strlen(tmptime)-1]='\0';
+    BX_INFO(("Last time is %u (%s)", (unsigned) get_timeval(), tmptime));
+    free(tmptime);
+  }
+  BX_DEBUG(("Exit"));
+}
 
 void bx_cmos_c::init(void)
 {
-  BX_DEBUG(("Init $Id: cmos.cc,v 1.56 2006/05/29 22:33:38 sshwarts Exp $"));
+  BX_DEBUG(("Init $Id: cmos.cc,v 1.62 2007/09/28 19:51:59 sshwarts Exp $"));
   // CMOS RAM & RTC
 
   DEV_register_ioread_handler(this, read_handler, 0x0070, "CMOS RAM", 1);
@@ -283,10 +296,9 @@ void bx_cmos_c::save_image(void)
   }
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_cmos_c::register_state(void)
 {
-  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "cmos", "CMOS State");
+  bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "cmos", "CMOS State", 2);
   BXRS_HEX_PARAM_FIELD(list, mem_address, BX_CMOS_THIS s.cmos_mem_address);
   bx_list_c *ram = new bx_list_c(list, "ram", 128);
   for (unsigned i=0; i<128; i++) {
@@ -303,7 +315,6 @@ void bx_cmos_c::after_restore_state(void)
   BX_CMOS_THIS update_timeval();
   BX_CMOS_THIS CRA_change();
 }
-#endif
 
 void bx_cmos_c::CRA_change(void)
 {
@@ -357,7 +368,7 @@ Bit32u bx_cmos_c::read(Bit32u address, unsigned io_len)
   switch (address) {
     case 0x0070:
       // this register is write-only on most machines
-      BX_INFO(("read of index port 0x70. returning 0xff"));
+      BX_DEBUG(("read of index port 0x70. returning 0xff"));
       return(0xff);
     case 0x0071:
       ret8 = BX_CMOS_THIS s.reg[BX_CMOS_THIS s.cmos_mem_address];
