@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: devices.cc,v 1.101 2006/05/27 15:54:48 sshwarts Exp $
+// $Id: devices.cc,v 1.118 2007/09/28 19:51:59 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -50,10 +50,32 @@ bx_devices_c::bx_devices_c()
 
   read_port_to_handler = NULL;
   write_port_to_handler = NULL;
+  io_read_handlers.next = NULL;
+  io_read_handlers.handler_name = NULL;
+  io_write_handlers.next = NULL;
+  io_write_handlers.handler_name = NULL;
+  init_stubs();
+
+  for (unsigned i=0; i < BX_MAX_IRQS; i++) {
+    irq_handler_name[i] = NULL;
+  }
+}
+
+bx_devices_c::~bx_devices_c()
+{
+  // nothing needed for now
+  timer_handle = BX_NULL_TIMER_HANDLE;
+}
+
+void bx_devices_c::init_stubs()
+{
 #if BX_SUPPORT_PCI
   pluginPciBridge = &stubPci;
   pluginPci2IsaBridge = &stubPci2Isa;
   pluginPciIdeController = &stubPciIde;
+#if BX_SUPPORT_ACPI
+  pluginACPIController = &stubACPIController;
+#endif
 #if BX_SUPPORT_PCIVGA
     pluginPciVgaAdapter = NULL;
 #endif
@@ -95,40 +117,29 @@ bx_devices_c::bx_devices_c()
 #endif
 }
 
-bx_devices_c::~bx_devices_c()
-{
-  // nothing needed for now
-  timer_handle = BX_NULL_TIMER_HANDLE;
-}
-
 void bx_devices_c::init(BX_MEM_C *newmem)
 {
   unsigned i;
+  const char def_name[] = "Default";
 
-  BX_DEBUG(("Init $Id: devices.cc,v 1.101 2006/05/27 15:54:48 sshwarts Exp $"));
+  BX_DEBUG(("Init $Id: devices.cc,v 1.118 2007/09/28 19:51:59 sshwarts Exp $"));
   mem = newmem;
 
   /* set no-default handlers, will be overwritten by the real default handler */
+  register_default_io_read_handler(NULL, &default_read_handler, def_name, 7);
   io_read_handlers.next = &io_read_handlers;
   io_read_handlers.prev = &io_read_handlers;
-  io_read_handlers.handler_name  = "Default";
-  io_read_handlers.funct         = (void *)&default_read_handler;
-  io_read_handlers.this_ptr      = NULL;
   io_read_handlers.usage_count = 0; // not used with the default handler
-  io_read_handlers.mask          = 7;
-  
+
+  register_default_io_write_handler(NULL, &default_write_handler, def_name, 7);
   io_write_handlers.next = &io_write_handlers;
   io_write_handlers.prev = &io_write_handlers;
-  io_write_handlers.handler_name = "Default";
-  io_write_handlers.funct        = (void *)&default_write_handler;
-  io_write_handlers.this_ptr     = NULL;
   io_write_handlers.usage_count = 0; // not used with the default handler
-  io_write_handlers.mask         = 7;
 
   if (read_port_to_handler)
-	  delete [] read_port_to_handler;
+    delete [] read_port_to_handler;
   if (write_port_to_handler)
-	  delete [] write_port_to_handler;
+    delete [] write_port_to_handler;
   read_port_to_handler = new struct io_handler_struct *[PORTS];
   write_port_to_handler = new struct io_handler_struct *[PORTS];
 
@@ -139,6 +150,7 @@ void bx_devices_c::init(BX_MEM_C *newmem)
   }
 
   for (i=0; i < BX_MAX_IRQS; i++) {
+    delete [] irq_handler_name[i];
     irq_handler_name[i] = NULL;
   }
 
@@ -186,6 +198,9 @@ void bx_devices_c::init(BX_MEM_C *newmem)
     PLUG_load_plugin(pci, PLUGTYPE_CORE);
     PLUG_load_plugin(pci2isa, PLUGTYPE_CORE);
     PLUG_load_plugin(pci_ide, PLUGTYPE_OPTIONAL);
+#if BX_SUPPORT_ACPI
+    PLUG_load_plugin(acpi, PLUGTYPE_OPTIONAL);
+#endif
 #if BX_SUPPORT_PCIVGA
     if ((DEV_is_pci_device("pcivga")) &&
         (!strcmp(SIM->get_param_string(BXPN_VGA_EXTENSION)->getptr(), "vbe"))) {
@@ -341,7 +356,6 @@ void bx_devices_c::reset(unsigned type)
   bx_reset_plugins(type);
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_devices_c::register_state()
 {
   bx_virt_timer.register_state();
@@ -377,7 +391,48 @@ void bx_devices_c::after_restore_state()
   pluginVgaDevice->after_restore_state();
   bx_plugins_after_restore_state();
 }
+
+void bx_devices_c::exit()
+{
+  // delete i/o handlers before unloading plugins
+  struct io_handler_struct *io_read_handler = io_read_handlers.next;
+  struct io_handler_struct *curr = NULL;
+  while (io_read_handler != &io_read_handlers) {
+    io_read_handler->prev->next = io_read_handler->next;
+    io_read_handler->next->prev = io_read_handler->prev;
+    curr = io_read_handler;
+    io_read_handler = io_read_handler->next;
+    delete [] curr->handler_name;
+    delete curr;
+  }
+  struct io_handler_struct *io_write_handler = io_write_handlers.next;
+  while (io_write_handler != &io_write_handlers) {
+    io_write_handler->prev->next = io_write_handler->next;
+    io_write_handler->next->prev = io_write_handler->prev;
+    curr = io_write_handler;
+    io_write_handler = io_write_handler->next;
+    delete [] curr->handler_name;
+    delete curr;
+  }
+
+  pit->exit();
+  bx_virt_timer.setup();
+  bx_slowdown_timer.exit();
+
+  PLUG_unload_plugin(unmapped);
+  PLUG_unload_plugin(biosdev);
+  PLUG_unload_plugin(cmos);
+  PLUG_unload_plugin(dma);
+  PLUG_unload_plugin(pic);
+  PLUG_unload_plugin(vga);
+  PLUG_unload_plugin(floppy);
+#if BX_SUPPORT_PCI
+  PLUG_unload_plugin(pci);
+  PLUG_unload_plugin(pci2isa);
 #endif
+  bx_unload_plugins();
+  init_stubs();
+}
 
 Bit32u bx_devices_c::read_handler(void *this_ptr, Bit32u address, unsigned io_len)
 {
@@ -445,18 +500,10 @@ void bx_devices_c::timer_handler(void *this_ptr)
 
 void bx_devices_c::timer()
 {
-#if (BX_USE_NEW_PIT==0)
-  if (pit->periodic(BX_IODEV_HANDLER_PERIOD)) {
-    // This is a hack to make the IRQ0 work
-    DEV_pic_lower_irq(0);
-    DEV_pic_raise_irq(0);
-  }
-#endif
-
   // separate calls to bx_gui->handle_events from the keyboard code.
   {
     static int multiple=0;
-    if ( ++multiple==10)
+    if (++multiple==10)
     {
       multiple=0;
       SIM->periodic();
@@ -471,15 +518,16 @@ bx_bool bx_devices_c::register_irq(unsigned irq, const char *name)
   if (irq >= BX_MAX_IRQS) {
     BX_PANIC(("IO device %s registered with IRQ=%d above %u",
              name, irq, (unsigned) BX_MAX_IRQS-1));
-    return false;
+    return 0;
   }
   if (irq_handler_name[irq]) {
     BX_PANIC(("IRQ %u conflict, %s with %s", irq,
       irq_handler_name[irq], name));
-    return false;
+    return 0;
   }
-  irq_handler_name[irq] = name;
-  return true;
+  irq_handler_name[irq] = new char[strlen(name)+1];
+  strcpy(irq_handler_name[irq], name);
+  return 1;
 }
 
 bx_bool bx_devices_c::unregister_irq(unsigned irq, const char *name)
@@ -487,31 +535,31 @@ bx_bool bx_devices_c::unregister_irq(unsigned irq, const char *name)
   if (irq >= BX_MAX_IRQS) {
     BX_PANIC(("IO device %s tried to unregister IRQ %d above %u",
              name, irq, (unsigned) BX_MAX_IRQS-1));
-    return false;
+    return 0;
   }
-
   if (!irq_handler_name[irq]) {
     BX_INFO(("IO device %s tried to unregister IRQ %d, not registered",
 	      name, irq));
-    return false;
+    return 0;
   }
 
   if (strcmp(irq_handler_name[irq], name)) {
     BX_INFO(("IRQ %u not registered to %s but to %s", irq,
       name, irq_handler_name[irq]));
-    return false;
+    return 0;
   }
+  delete [] irq_handler_name[irq];
   irq_handler_name[irq] = NULL;
-  return true;
+  return 1;
 }
 
 bx_bool bx_devices_c::register_io_read_handler(void *this_ptr, bx_read_handler_t f,
-                                       Bit32u addr, const char *name, Bit8u mask)
+                                               Bit32u addr, const char *name, Bit8u mask)
 {
   addr &= 0x0000ffff;
 
   if (!f)
-    return false;
+    return 0;
 
   /* first check if the port already has a handlers != the default handler */
   if (read_port_to_handler[addr] &&
@@ -520,7 +568,7 @@ bx_bool bx_devices_c::register_io_read_handler(void *this_ptr, bx_read_handler_t
               (unsigned) addr));
     BX_ERROR(("  conflicting devices: %s & %s",
               read_port_to_handler[addr]->handler_name, name));
-    return false;
+    return 0;
   }
 
   /* first find existing handle for function or create new one */
@@ -541,7 +589,8 @@ bx_bool bx_devices_c::register_io_read_handler(void *this_ptr, bx_read_handler_t
     io_read_handler = new struct io_handler_struct;
     io_read_handler->funct = (void *)f;
     io_read_handler->this_ptr = this_ptr;
-    io_read_handler->handler_name = name;
+    io_read_handler->handler_name = new char[strlen(name)+1];
+    strcpy(io_read_handler->handler_name, name);
     io_read_handler->mask = mask;
     io_read_handler->usage_count = 0;
     // add the handler to the double linked list of handlers
@@ -553,16 +602,16 @@ bx_bool bx_devices_c::register_io_read_handler(void *this_ptr, bx_read_handler_t
 
   io_read_handler->usage_count++;
   read_port_to_handler[addr] = io_read_handler;
-  return true; // address mapped successfully
+  return 1; // address mapped successfully
 }
 
 bx_bool bx_devices_c::register_io_write_handler(void *this_ptr, bx_write_handler_t f,
-                                        Bit32u addr, const char *name, Bit8u mask)
+                                                Bit32u addr, const char *name, Bit8u mask)
 {
   addr &= 0x0000ffff;
 
   if (!f)
-    return false;
+    return 0;
 
   /* first check if the port already has a handlers != the default handler */
   if (write_port_to_handler[addr] &&
@@ -571,7 +620,7 @@ bx_bool bx_devices_c::register_io_write_handler(void *this_ptr, bx_write_handler
               (unsigned) addr));
     BX_ERROR(("  conflicting devices: %s & %s",
               write_port_to_handler[addr]->handler_name, name));
-    return false;
+    return 0;
   }
 
   /* first find existing handle for function or create new one */
@@ -592,7 +641,8 @@ bx_bool bx_devices_c::register_io_write_handler(void *this_ptr, bx_write_handler
     io_write_handler = new struct io_handler_struct;
     io_write_handler->funct = (void *)f;
     io_write_handler->this_ptr = this_ptr;
-    io_write_handler->handler_name = name;
+    io_write_handler->handler_name = new char[strlen(name)+1];
+    strcpy(io_write_handler->handler_name, name);
     io_write_handler->mask = mask;
     io_write_handler->usage_count = 0;
     // add the handler to the double linked list of handlers
@@ -604,7 +654,7 @@ bx_bool bx_devices_c::register_io_write_handler(void *this_ptr, bx_write_handler
 
   io_write_handler->usage_count++;
   write_port_to_handler[addr] = io_write_handler;
-  return true; // address mapped successfully
+  return 1; // address mapped successfully
 }
 
 bx_bool bx_devices_c::register_io_read_handler_range(void *this_ptr, bx_read_handler_t f,
@@ -617,12 +667,12 @@ bx_bool bx_devices_c::register_io_read_handler_range(void *this_ptr, bx_read_han
 
   if (end_addr < begin_addr) {
     BX_ERROR(("!!! end_addr < begin_addr !!!"));
-    return false;
+    return 0;
   }
   
   if (!f) {
     BX_ERROR(("!!! f == NULL !!!"));
-    return false;
+    return 0;
   }
 
   /* first check if the port already has a handlers != the default handler */
@@ -633,7 +683,7 @@ bx_bool bx_devices_c::register_io_read_handler_range(void *this_ptr, bx_read_han
                 (unsigned) addr));
       BX_ERROR(("  conflicting devices: %s & %s",
                 read_port_to_handler[addr]->handler_name, name));
-      return false;
+      return 0;
   }
 
   /* first find existing handle for function or create new one */
@@ -654,7 +704,8 @@ bx_bool bx_devices_c::register_io_read_handler_range(void *this_ptr, bx_read_han
     io_read_handler = new struct io_handler_struct;
     io_read_handler->funct = (void *)f;
     io_read_handler->this_ptr = this_ptr;
-    io_read_handler->handler_name = name;
+    io_read_handler->handler_name = new char[strlen(name)+1];
+    strcpy(io_read_handler->handler_name, name);
     io_read_handler->mask = mask;
     io_read_handler->usage_count = 0;
     // add the handler to the double linked list of handlers
@@ -667,7 +718,7 @@ bx_bool bx_devices_c::register_io_read_handler_range(void *this_ptr, bx_read_han
   io_read_handler->usage_count += end_addr - begin_addr + 1;
   for (addr = begin_addr; addr <= end_addr; addr++)
 	  read_port_to_handler[addr] = io_read_handler;
-  return true; // address mapped successfully
+  return 1; // address mapped successfully
 }
 
 bx_bool bx_devices_c::register_io_write_handler_range(void *this_ptr, bx_write_handler_t f,
@@ -680,12 +731,12 @@ bx_bool bx_devices_c::register_io_write_handler_range(void *this_ptr, bx_write_h
 
   if (end_addr < begin_addr) {
     BX_ERROR(("!!! end_addr < begin_addr !!!"));
-    return false;
+    return 0;
   }
   
   if (!f) {
     BX_ERROR(("!!! f == NULL !!!"));
-    return false;
+    return 0;
   }
 
   /* first check if the port already has a handlers != the default handler */
@@ -696,7 +747,7 @@ bx_bool bx_devices_c::register_io_write_handler_range(void *this_ptr, bx_write_h
                 (unsigned) addr));
       BX_ERROR(("  conflicting devices: %s & %s",
                 write_port_to_handler[addr]->handler_name, name));
-      return false;
+      return 0;
     }
 
   /* first find existing handle for function or create new one */
@@ -717,7 +768,8 @@ bx_bool bx_devices_c::register_io_write_handler_range(void *this_ptr, bx_write_h
     io_write_handler = new struct io_handler_struct;
     io_write_handler->funct = (void *)f;
     io_write_handler->this_ptr = this_ptr;
-    io_write_handler->handler_name = name;
+    io_write_handler->handler_name = new char[strlen(name)+1];
+    strcpy(io_write_handler->handler_name, name);
     io_write_handler->mask = mask;
     io_write_handler->usage_count = 0;
     // add the handler to the double linked list of handlers
@@ -730,7 +782,7 @@ bx_bool bx_devices_c::register_io_write_handler_range(void *this_ptr, bx_write_h
   io_write_handler->usage_count += end_addr - begin_addr + 1;
   for (addr = begin_addr; addr <= end_addr; addr++)
 	  write_port_to_handler[addr] = io_write_handler;
-  return true; // address mapped successfully
+  return 1; // address mapped successfully
 }
 
 
@@ -740,9 +792,13 @@ bx_bool bx_devices_c::register_default_io_read_handler(void *this_ptr, bx_read_h
 {
   io_read_handlers.funct = (void *)f;
   io_read_handlers.this_ptr = this_ptr;
-  io_read_handlers.handler_name = name;
+  if (io_read_handlers.handler_name) {
+    delete [] io_read_handlers.handler_name;
+  }
+  io_read_handlers.handler_name = new char[strlen(name)+1];
+  strcpy(io_read_handlers.handler_name, name);
   io_read_handlers.mask = mask;
-  return true; 
+  return 1; 
 }
 
 bx_bool bx_devices_c::register_default_io_write_handler(void *this_ptr, bx_write_handler_t f,
@@ -750,9 +806,13 @@ bx_bool bx_devices_c::register_default_io_write_handler(void *this_ptr, bx_write
 {
   io_write_handlers.funct = (void *)f;
   io_write_handlers.this_ptr = this_ptr;
-  io_write_handlers.handler_name = name;
+  if (io_write_handlers.handler_name) {
+    delete [] io_write_handlers.handler_name;
+  }
+  io_write_handlers.handler_name = new char[strlen(name)+1];
+  strcpy(io_write_handlers.handler_name, name);
   io_write_handlers.mask = mask;
-  return true; 
+  return 1; 
 }
 
 bx_bool bx_devices_c::unregister_io_read_handler(void *this_ptr, bx_read_handler_t f,
@@ -766,27 +826,27 @@ bx_bool bx_devices_c::unregister_io_read_handler(void *this_ptr, bx_read_handler
 
   if (!io_read_handler) {
     BX_ERROR((">>> NO IO_READ_HANDLER <<<"));
-    return false;
+    return 0;
   }
 
   if (io_read_handler == &io_read_handlers) {
     BX_ERROR((">>> CANNOT UNREGISTER THE DEFAULT IO_READ_HANDLER <<<"));
-    return false; // cannot unregister the default handler
+    return 0; // cannot unregister the default handler
   }
 
   if (io_read_handler->funct != f) {
     BX_ERROR((">>> NOT THE SAME IO_READ_HANDLER FUNC <<<"));
-    return false;
+    return 0;
   }
 
   if (io_read_handler->this_ptr != this_ptr) {
     BX_ERROR((">>> NOT THE SAME IO_READ_HANDLER THIS_PTR <<<"));
-    return false;
+    return 0;
   }
 
   if (io_read_handler->mask != mask) {
     BX_ERROR((">>> NOT THE SAME IO_READ_HANDLER MASK <<<"));
-    return false;
+    return 0;
   }
 
   read_port_to_handler[addr] = &io_read_handlers; // reset to default
@@ -795,9 +855,10 @@ bx_bool bx_devices_c::unregister_io_read_handler(void *this_ptr, bx_read_handler
   if (!io_read_handler->usage_count) { // kill this handler entry
     io_read_handler->prev->next = io_read_handler->next;
     io_read_handler->next->prev = io_read_handler->prev;
+    delete [] io_read_handler->handler_name;
     delete io_read_handler;
   }
-  return true;
+  return 1;
 }
 
 bx_bool bx_devices_c::unregister_io_write_handler(void *this_ptr, bx_write_handler_t f,
@@ -808,19 +869,19 @@ bx_bool bx_devices_c::unregister_io_write_handler(void *this_ptr, bx_write_handl
   struct io_handler_struct *io_write_handler = write_port_to_handler[addr];
 
   if (!io_write_handler)
-    return false;
+    return 0;
 
   if (io_write_handler == &io_write_handlers)
-    return false; // cannot unregister the default handler
+    return 0; // cannot unregister the default handler
 
   if (io_write_handler->funct != f)
-    return false;
+    return 0;
 
   if (io_write_handler->this_ptr != this_ptr)
-    return false;
+    return 0;
 
   if (io_write_handler->mask != mask)
-    return false;
+    return 0;
 
   write_port_to_handler[addr] = &io_write_handlers; // reset to default
   io_write_handler->usage_count--;
@@ -828,9 +889,10 @@ bx_bool bx_devices_c::unregister_io_write_handler(void *this_ptr, bx_write_handl
   if (!io_write_handler->usage_count) { // kill this handler entry
     io_write_handler->prev->next = io_write_handler->next;
     io_write_handler->next->prev = io_write_handler->prev;
+    delete [] io_write_handler->handler_name;
     delete io_write_handler;
   }
-  return true;
+  return 1;
 }
 
 bx_bool bx_devices_c::unregister_io_read_handler_range(void *this_ptr, bx_read_handler_t f,
@@ -839,14 +901,14 @@ bx_bool bx_devices_c::unregister_io_read_handler_range(void *this_ptr, bx_read_h
   begin &= 0x0000ffff;
   end &= 0x0000ffff;
   Bit32u addr;
-  bx_bool ret = true;
+  bx_bool ret = 1;
 
   /*
    * the easy way this time
    */
   for (addr = begin; addr <= end; addr++)
     if (!unregister_io_read_handler(this_ptr, f, addr, mask))
-      ret = false;
+      ret = 0;
 
   return ret;
 }
@@ -857,14 +919,14 @@ bx_bool bx_devices_c::unregister_io_write_handler_range(void *this_ptr, bx_write
   begin &= 0x0000ffff;
   end &= 0x0000ffff;
   Bit32u addr;
-  bx_bool ret = true;
+  bx_bool ret = 1;
 
   /*
    * the easy way this time
    */
   for (addr = begin; addr <= end; addr++)
     if (!unregister_io_write_handler(this_ptr, f, addr, mask))
-      ret = false;
+      ret = 0;
 
   return ret;
 }
@@ -958,3 +1020,15 @@ bx_bool bx_devices_c::is_parallel_enabled()
   }
   return false;
 }
+
+void bx_pci_device_stub_c::register_pci_state(bx_list_c *list, Bit8u *pci_conf)
+{
+  unsigned i;
+  char name[6];
+
+  bx_list_c *pci = new bx_list_c(list, "pci_conf", 256);
+  for (i=0; i<256; i++) {
+    sprintf(name, "0x%02x", i);
+    new bx_shadow_num_c(pci, name, &pci_conf[i], BASE_HEX);
+  }
+}
