@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: dma.cc,v 1.40 2006/06/14 16:44:33 sshwarts Exp $
+// $Id: dma.cc,v 1.44 2007/10/09 19:49:23 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -52,6 +52,7 @@ int libdma_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *
 
 void libdma_LTX_plugin_fini(void)
 {
+  delete theDmaDevice;
 }
 
 bx_dma_c::bx_dma_c()
@@ -62,7 +63,7 @@ bx_dma_c::bx_dma_c()
 
 bx_dma_c::~bx_dma_c()
 {
-   BX_DEBUG(("Exit"));
+  BX_DEBUG(("Exit"));
 }
 
 unsigned bx_dma_c::registerDMA8Channel(unsigned channel,
@@ -122,7 +123,7 @@ unsigned bx_dma_c::get_TC(void)
 void bx_dma_c::init(void)
 {
   unsigned c, i, j;
-  BX_DEBUG(("Init $Id: dma.cc,v 1.40 2006/06/14 16:44:33 sshwarts Exp $"));
+  BX_DEBUG(("Init $Id: dma.cc,v 1.44 2007/10/09 19:49:23 sshwarts Exp $"));
 
   /* 8237 DMA controller */
 
@@ -190,12 +191,11 @@ void bx_dma_c::reset_controller(unsigned num)
   BX_DMA_THIS s[num].flip_flop = 0;
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_dma_c::register_state(void)
 {
   unsigned i, c;
   char name[6];
-  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "dma", "DMA State");
+  bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "dma", "DMA State", 3);
   for (i=0; i<2; i++) {
     sprintf(name, "%d", i);
     bx_list_c *ctrl = new bx_list_c(list, name, 8);
@@ -226,15 +226,12 @@ void bx_dma_c::register_state(void)
     new bx_shadow_num_c(extpg, name, &BX_DMA_THIS ext_page_reg[i], BASE_HEX);
   }
 }
-#endif
 
 // index to find channel from register number (only [0],[1],[2],[6] used)
 Bit8u channelindex[7] = {2, 3, 1, 0, 0, 0, 0};
 
-
 // static IO port read callback handler
 // redirects to non-static class handler to avoid virtual functions
-
 Bit32u bx_dma_c::read_handler(void *this_ptr, Bit32u address, unsigned io_len)
 {
 #if !BX_USE_DMA_SMF
@@ -679,7 +676,7 @@ void bx_dma_c::control_HRQ(bx_bool ma_sl)
 void bx_dma_c::raise_HLDA(void)
 {
   unsigned channel;
-  Bit32u phy_addr;
+  bx_phy_address phy_addr;
   bx_bool count_expired = 0;
   bx_bool ma_sl = 0;
 
