@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: floppy.cc,v 1.102 2006/08/12 11:22:40 vruppert Exp $
+// $Id: floppy.cc,v 1.108 2007/09/28 19:51:59 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -112,14 +112,16 @@ static Bit16u drate_in_k[4] = {
 
 int libfloppy_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
-  theFloppyController = new bx_floppy_ctrl_c ();
+  theFloppyController = new bx_floppy_ctrl_c();
   bx_devices.pluginFloppyDevice = theFloppyController;
   BX_REGISTER_DEVICE_DEVMODEL(plugin, type, theFloppyController, BX_PLUGIN_FLOPPY);
   return(0); // Success
 }
 
-void libfloppy_LTX_plugin_fini(void) {}
-
+void libfloppy_LTX_plugin_fini(void)
+{
+  delete theFloppyController;
+}
 
 bx_floppy_ctrl_c::bx_floppy_ctrl_c()
 {
@@ -130,14 +132,14 @@ bx_floppy_ctrl_c::bx_floppy_ctrl_c()
 
 bx_floppy_ctrl_c::~bx_floppy_ctrl_c()
 {
-  // nothing for now
+  BX_DEBUG(("Exit"));
 }
 
 void bx_floppy_ctrl_c::init(void)
 {
   Bit8u i;
 
-  BX_DEBUG(("Init $Id: floppy.cc,v 1.102 2006/08/12 11:22:40 vruppert Exp $"));
+  BX_DEBUG(("Init $Id: floppy.cc,v 1.108 2007/09/28 19:51:59 sshwarts Exp $"));
   DEV_dma_register_8bit_channel(2, dma_read, dma_write, "Floppy Drive");
   DEV_register_irq(6, "Floppy Drive");
   for (unsigned addr=0x03F2; addr<=0x03F7; addr++) {
@@ -324,11 +326,12 @@ void bx_floppy_ctrl_c::init(void)
     BX_FD_THIS s.floppy_timer_index =
       bx_pc_system.register_timer( this, timer_handler, 250, 0, 0, "floppy");
   }
+  /* phase out s.non_dma in favor of using FD_MS_NDMA, more like hardware */
+  BX_FD_THIS s.main_status_reg &= ~FD_MS_NDMA;  // enable DMA from start
   /* these registers are not cleared by reset */
   BX_FD_THIS s.SRT = 0;
   BX_FD_THIS s.HUT = 0;
   BX_FD_THIS s.HLT = 0;
-  BX_FD_THIS s.non_dma = 0;
 }
 
 void bx_floppy_ctrl_c::reset(unsigned type)
@@ -375,18 +378,19 @@ void bx_floppy_ctrl_c::reset(unsigned type)
   }
 
   DEV_pic_lower_irq(6);
-  DEV_dma_set_drq(FLOPPY_DMA_CHAN, 0);
+  if (!(BX_FD_THIS s.main_status_reg & FD_MS_NDMA)) {
+    DEV_dma_set_drq(FLOPPY_DMA_CHAN, 0);
+  }
   enter_idle_phase();
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_floppy_ctrl_c::register_state(void)
 {
   unsigned i;
   char name[8];
   bx_list_c *drive;
 
-  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "floppy", "Floppy State", 36);
+  bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "floppy", "Floppy State", 35);
   new bx_shadow_num_c(list, "data_rate", &BX_FD_THIS s.data_rate);
   bx_list_c *command = new bx_list_c(list, "command", 10);
   for (i=0; i<10; i++) {
@@ -418,7 +422,6 @@ void bx_floppy_ctrl_c::register_state(void)
   new bx_shadow_num_c(list, "status_reg2", &BX_FD_THIS s.status_reg2, BASE_HEX);
   new bx_shadow_num_c(list, "status_reg3", &BX_FD_THIS s.status_reg3, BASE_HEX);
   new bx_shadow_num_c(list, "floppy_buffer_index", &BX_FD_THIS s.floppy_buffer_index);
-  new bx_shadow_bool_c(list, "non_dma", &BX_FD_THIS s.non_dma);
   new bx_shadow_bool_c(list, "lock", &BX_FD_THIS s.lock);
   new bx_shadow_num_c(list, "SRT", &BX_FD_THIS s.SRT, BASE_HEX);
   new bx_shadow_num_c(list, "HUT", &BX_FD_THIS s.HUT, BASE_HEX);
@@ -429,7 +432,7 @@ void bx_floppy_ctrl_c::register_state(void)
   new bx_shadow_data_c(list, "buffer", BX_FD_THIS s.floppy_buffer, 512);
   for (i=0; i<4; i++) {
     sprintf(name, "drive%d", i);
-    drive = new bx_list_c(list, name);
+    drive = new bx_list_c(list, name, 6);
     new bx_shadow_num_c(drive, "cylinder", &BX_FD_THIS s.cylinder[i]);
     new bx_shadow_num_c(drive, "head", &BX_FD_THIS s.head[i]);
     new bx_shadow_num_c(drive, "sector", &BX_FD_THIS s.sector[i]);
@@ -438,7 +441,6 @@ void bx_floppy_ctrl_c::register_state(void)
     new bx_shadow_num_c(drive, "DIR", &BX_FD_THIS s.DIR[i], BASE_HEX);
   }
 }
-#endif
 
 // static IO port read callback handler
 // redirects to non-static class handler to avoid virtual functions
@@ -470,9 +472,15 @@ Bit32u bx_floppy_ctrl_c::read(Bit32u address, unsigned io_len)
       break;
 
     case 0x3F5: /* diskette controller data */
-      if (BX_FD_THIS s.result_size == 0) {
+      if ((BX_FD_THIS s.main_status_reg & FD_MS_NDMA) &&
+	((BX_FD_THIS s.pending_command & 0x4f) == 0x46)) {
+        dma_write(&value);
+        lower_interrupt();
+        // don't enter idle phase until we've given CPU last data byte
+        if (BX_FD_THIS s.TC) enter_idle_phase();
+      } else if (BX_FD_THIS s.result_size == 0) {
         BX_ERROR(("port 0x3f5: no results to read"));
-        BX_FD_THIS s.main_status_reg = 0;
+        BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
         value = BX_FD_THIS s.result[0];
       } else {
         value = BX_FD_THIS s.result[BX_FD_THIS s.result_index++];
@@ -537,7 +545,8 @@ Bit32u bx_floppy_ctrl_c::read(Bit32u address, unsigned io_len)
       return(0);
       break;
   }
-  BX_DEBUG(("read access to port %04x returns 0x%02x", address, value));
+  BX_DEBUG(("read(): during command 0x%02x, port %04x returns 0x%02x",
+               BX_FD_THIS s.pending_command, address, value));
   return (value);
 }
 
@@ -592,7 +601,7 @@ void bx_floppy_ctrl_c::write(Bit32u address, Bit32u value, unsigned io_len)
         bx_pc_system.activate_timer(BX_FD_THIS s.floppy_timer_index, 250, 0);
       } else if (prev_normal_operation && normal_operation==0) {
         // transition from NORMAL to RESET
-        BX_FD_THIS s.main_status_reg = 0;
+        BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
         BX_FD_THIS s.pending_command = 0xfe; // RESET pending
       }
       BX_DEBUG(("io_write: digital output register"));
@@ -612,7 +621,7 @@ void bx_floppy_ctrl_c::write(Bit32u address, Bit32u value, unsigned io_len)
     case 0x3f4: /* diskette controller data rate select register */
       BX_FD_THIS s.data_rate = value & 0x03;
       if (value & 0x80) {
-        BX_FD_THIS s.main_status_reg = 0;
+        BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
         BX_FD_THIS s.pending_command = 0xfe; // RESET pending
         bx_pc_system.activate_timer(BX_FD_THIS s.floppy_timer_index, 250, 0);
       }
@@ -623,7 +632,11 @@ void bx_floppy_ctrl_c::write(Bit32u address, Bit32u value, unsigned io_len)
 
     case 0x3F5: /* diskette controller data */
       BX_DEBUG(("command = %02x", (unsigned) value));
-      if (BX_FD_THIS s.command_complete) {
+      if ((BX_FD_THIS s.main_status_reg & FD_MS_NDMA) && ((BX_FD_THIS s.pending_command & 0x4f) == 0x45)) {
+        BX_FD_THIS dma_read((Bit8u *) &value);
+        BX_FD_THIS lower_interrupt();
+        break;
+      } else if (BX_FD_THIS s.command_complete) {
         if (BX_FD_THIS s.pending_command != 0)
           BX_PANIC(("write 0x03f5: receiving new command 0x%02x, old one (0x%02x) pending",
             value, BX_FD_THIS s.pending_command));
@@ -631,7 +644,7 @@ void bx_floppy_ctrl_c::write(Bit32u address, Bit32u value, unsigned io_len)
         BX_FD_THIS s.command_complete = 0;
         BX_FD_THIS s.command_index = 1;
         /* read/write command in progress */
-        BX_FD_THIS s.main_status_reg &= 0x0f; // leave drive status untouched
+        BX_FD_THIS s.main_status_reg &= ~FD_MS_DIO; // leave drive status untouched
         BX_FD_THIS s.main_status_reg |= FD_MS_MRQ | FD_MS_BUSY;
         switch (value) {
           case 0x03: /* specify */
@@ -769,9 +782,9 @@ void bx_floppy_ctrl_c::floppy_command(void)
       BX_FD_THIS s.SRT = BX_FD_THIS s.command[1] >> 4;
       BX_FD_THIS s.HUT = BX_FD_THIS s.command[1] & 0x0f;
       BX_FD_THIS s.HLT = BX_FD_THIS s.command[2] >> 1;
-      BX_FD_THIS s.non_dma = BX_FD_THIS s.command[2] & 0x01;
-      if (BX_FD_THIS s.non_dma)
-        BX_ERROR(("non DMA mode not implemented yet"));
+      BX_FD_THIS s.main_status_reg |= (BX_FD_THIS s.command[2] & 0x01) ? FD_MS_NDMA : 0;
+      if (BX_FD_THIS s.main_status_reg & FD_MS_NDMA)
+        BX_ERROR(("non DMA mode not fully implemented yet"));
       enter_idle_phase();
       return;
 
@@ -799,7 +812,8 @@ void bx_floppy_ctrl_c::floppy_command(void)
        * The last two are taken care of in timer().
        */
       BX_FD_THIS s.cylinder[drive] = 0;
-      BX_FD_THIS s.main_status_reg = (1 << drive);
+      BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
+      BX_FD_THIS s.main_status_reg |= (1 << drive);
       return;
 
     case 0x08: /* sense interrupt status */
@@ -841,8 +855,9 @@ void bx_floppy_ctrl_c::floppy_command(void)
       bx_pc_system.activate_timer(BX_FD_THIS s.floppy_timer_index, step_delay, 0);
       /* ??? should also check cylinder validity */
       BX_FD_THIS s.cylinder[drive] = BX_FD_THIS s.command[2];
-      /* data reg not ready, drive busy */
-      BX_FD_THIS s.main_status_reg = (1 << drive);
+      /* data reg not ready, drive not busy */
+      BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
+      BX_FD_THIS s.main_status_reg |= (1 << drive);
       return;
 
     case 0x13: // Configure
@@ -865,17 +880,20 @@ void bx_floppy_ctrl_c::floppy_command(void)
       motor_on = (BX_FD_THIS s.DOR>>(drive+4)) & 0x01;
       if (motor_on == 0) {
         BX_ERROR(("floppy_command(): read ID: motor not on"));
-        BX_FD_THIS s.main_status_reg = FD_MS_BUSY;
+        BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
+        BX_FD_THIS s.main_status_reg |= FD_MS_BUSY;
         return; // Hang controller
       }
       if (BX_FD_THIS s.device_type[drive] == FDRIVE_NONE) {
         BX_ERROR(("floppy_command(): read ID: bad drive #%d", drive));
-        BX_FD_THIS s.main_status_reg = FD_MS_BUSY;
+        BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
+        BX_FD_THIS s.main_status_reg |= FD_MS_BUSY;
         return; // Hang controller
       }
       if (BX_FD_THIS s.media_present[drive] == 0) {
         BX_INFO(("attempt to read sector ID with media not present"));
-        BX_FD_THIS s.main_status_reg = FD_MS_BUSY;
+        BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
+        BX_FD_THIS s.main_status_reg |= FD_MS_BUSY;
         return; // Hang controller
       }
       BX_FD_THIS s.status_reg0 = (BX_FD_THIS s.head[drive]<<2) | drive;
@@ -883,7 +901,8 @@ void bx_floppy_ctrl_c::floppy_command(void)
       sector_time = 200000 / BX_FD_THIS s.media[drive].sectors_per_track;
       bx_pc_system.activate_timer(BX_FD_THIS s.floppy_timer_index, sector_time, 0);
       /* data reg not ready, controller busy */
-      BX_FD_THIS s.main_status_reg = FD_MS_BUSY;
+      BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
+      BX_FD_THIS s.main_status_reg |= FD_MS_BUSY;
       return;
 
     case 0x4d: // format track
@@ -925,10 +944,14 @@ void bx_floppy_ctrl_c::floppy_command(void)
       /* 4 header bytes per sector are required */
       BX_FD_THIS s.format_count <<= 2;
 
-      DEV_dma_set_drq(FLOPPY_DMA_CHAN, 1);
-
+      if (BX_FD_THIS s.main_status_reg & FD_MS_NDMA) {
+        BX_DEBUG(("non-DMA floppy format unimplemented"));
+      } else {
+        DEV_dma_set_drq(FLOPPY_DMA_CHAN, 1);
+      }
       /* data reg not ready, controller busy */
-      BX_FD_THIS s.main_status_reg = FD_MS_BUSY;
+      BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
+      BX_FD_THIS s.main_status_reg |= FD_MS_BUSY;
       BX_DEBUG(("format track"));
       return;
 
@@ -1032,16 +1055,25 @@ void bx_floppy_ctrl_c::floppy_command(void)
       if ((BX_FD_THIS s.command[0] & 0x4f) == 0x46) { // read
         floppy_xfer(drive, logical_sector*512, BX_FD_THIS s.floppy_buffer,
                     512, FROM_FLOPPY);
-        /* data reg not ready, controller busy */
-        BX_FD_THIS s.main_status_reg = FD_MS_BUSY;
+        /* controller busy; if DMA mode, data reg not ready */
+        BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
+        BX_FD_THIS s.main_status_reg |= FD_MS_BUSY;
+        if (BX_FD_THIS s.main_status_reg & FD_MS_NDMA) {
+          BX_FD_THIS s.main_status_reg |= (FD_MS_MRQ | FD_MS_DIO);
+        }
         // time to read one sector at 300 rpm
         sector_time = 200000 / BX_FD_THIS s.media[drive].sectors_per_track;
         bx_pc_system.activate_timer(BX_FD_THIS s.floppy_timer_index,
                                     sector_time , 0);
       } else if ((BX_FD_THIS s.command[0] & 0x7f) == 0x45) { // write
-        /* data reg not ready, controller busy */
-        BX_FD_THIS s.main_status_reg = FD_MS_BUSY;
-        DEV_dma_set_drq(FLOPPY_DMA_CHAN, 1);
+        /* controller busy; if DMA mode, data reg not ready */
+        BX_FD_THIS s.main_status_reg &= FD_MS_NDMA;
+        BX_FD_THIS s.main_status_reg |= FD_MS_BUSY;
+        if (BX_FD_THIS s.main_status_reg & FD_MS_NDMA) {
+          BX_FD_THIS s.main_status_reg |= FD_MS_MRQ;
+        } else {
+          DEV_dma_set_drq(FLOPPY_DMA_CHAN, 1);
+        }
       } else {
         BX_PANIC(("floppy_command(): unknown read/write command"));
         return;
@@ -1222,7 +1254,9 @@ void bx_floppy_ctrl_c::timer()
         enter_result_phase();
       } else {
         // transfer next sector
-        DEV_dma_set_drq(FLOPPY_DMA_CHAN, 1);
+        if (!(BX_FD_THIS s.main_status_reg & FD_MS_NDMA)) {
+          DEV_dma_set_drq(FLOPPY_DMA_CHAN, 1);
+        }
       }
       break;
 
@@ -1231,7 +1265,12 @@ void bx_floppy_ctrl_c::timer()
     case 0xc6:
     case 0xe6:
       // transfer next sector
-      DEV_dma_set_drq(FLOPPY_DMA_CHAN, 1);
+      if (BX_FD_THIS s.main_status_reg & FD_MS_NDMA) {
+        BX_FD_THIS s.main_status_reg &= ~FD_MS_BUSY;  // clear busy bit
+        BX_FD_THIS s.main_status_reg |= FD_MS_MRQ | FD_MS_DIO;  // data byte waiting
+      } else {
+        DEV_dma_set_drq(FLOPPY_DMA_CHAN, 1);
+      }
       break;
 
     case 0x4d: /* format track */
@@ -1241,7 +1280,9 @@ void bx_floppy_ctrl_c::timer()
         enter_result_phase();
       } else {
         // transfer next sector
-        DEV_dma_set_drq(FLOPPY_DMA_CHAN, 1);
+        if (!(BX_FD_THIS s.main_status_reg & FD_MS_NDMA)) {
+          DEV_dma_set_drq(FLOPPY_DMA_CHAN, 1);
+        }
       }
       break;
 
@@ -1265,16 +1306,17 @@ void bx_floppy_ctrl_c::timer()
 void bx_floppy_ctrl_c::dma_write(Bit8u *data_byte)
 {
   // A DMA write is from I/O to Memory
-  // We need to return then next data byte from the floppy buffer
+  // We need to return the next data byte from the floppy buffer
   // to be transfered via the DMA to memory. (read block from floppy)
 
+  Bit8u drive;
+
+  drive = BX_FD_THIS s.DOR & 0x03;
   *data_byte = BX_FD_THIS s.floppy_buffer[BX_FD_THIS s.floppy_buffer_index++];
 
-  BX_FD_THIS s.TC = DEV_dma_get_tc();
+  BX_FD_THIS s.TC = get_tc();
   if ((BX_FD_THIS s.floppy_buffer_index >= 512) || (BX_FD_THIS s.TC)) {
-    Bit8u drive;
 
-    drive = BX_FD_THIS s.DOR & 0x03;
     if (BX_FD_THIS s.floppy_buffer_index >= 512) {
       increment_sector(); // increment to next sector before retrieving next one
       BX_FD_THIS s.floppy_buffer_index = 0;
@@ -1293,12 +1335,14 @@ void bx_floppy_ctrl_c::dma_write(Bit8u *data_byte)
         BX_INFO(("  sector   = %u", (unsigned) BX_FD_THIS s.sector[drive]));
       }
 
-      DEV_dma_set_drq(FLOPPY_DMA_CHAN, 0);
+      if (!(BX_FD_THIS s.main_status_reg & FD_MS_NDMA)) {
+        DEV_dma_set_drq(FLOPPY_DMA_CHAN, 0);
+      }
       enter_result_phase();
     } else { // more data to transfer
       Bit32u logical_sector, sector_time;
 
-      // original assumed all floppies had two sides...now it does not  *delete this comment line*
+      // remember that not all floppies have two sides, multiply by s.head[drive]
       logical_sector = (BX_FD_THIS s.cylinder[drive] * BX_FD_THIS s.media[drive].heads *
                         BX_FD_THIS s.media[drive].sectors_per_track) +
                        (BX_FD_THIS s.head[drive] *
@@ -1307,7 +1351,9 @@ void bx_floppy_ctrl_c::dma_write(Bit8u *data_byte)
 
       floppy_xfer(drive, logical_sector*512, BX_FD_THIS s.floppy_buffer,
                   512, FROM_FLOPPY);
-      DEV_dma_set_drq(FLOPPY_DMA_CHAN, 0);
+      if (!(BX_FD_THIS s.main_status_reg & FD_MS_NDMA)) {
+        DEV_dma_set_drq(FLOPPY_DMA_CHAN, 0);
+      }
       // time to read one sector at 300 rpm
       sector_time = 200000 / BX_FD_THIS s.media[drive].sectors_per_track;
       bx_pc_system.activate_timer(BX_FD_THIS s.floppy_timer_index,
@@ -1325,7 +1371,6 @@ void bx_floppy_ctrl_c::dma_read(Bit8u *data_byte)
   Bit8u drive;
   Bit32u logical_sector, sector_time;
 
-  BX_FD_THIS s.TC = DEV_dma_get_tc();
   drive = BX_FD_THIS s.DOR & 0x03;
   if (BX_FD_THIS s.pending_command == 0x4d) { // format track in progress
     BX_FD_THIS s.format_count--;
@@ -1353,7 +1398,9 @@ void bx_floppy_ctrl_c::dma_read(Bit8u *data_byte)
                          (BX_FD_THIS s.sector[drive] - 1);
         floppy_xfer(drive, logical_sector*512, BX_FD_THIS s.floppy_buffer,
                     512, TO_FLOPPY);
-        DEV_dma_set_drq(FLOPPY_DMA_CHAN, 0);
+        if (!(BX_FD_THIS s.main_status_reg & FD_MS_NDMA)) {
+          DEV_dma_set_drq(FLOPPY_DMA_CHAN, 0);
+        }
         // time to write one sector at 300 rpm
         sector_time = 200000 / BX_FD_THIS s.media[drive].sectors_per_track;
         bx_pc_system.activate_timer(BX_FD_THIS s.floppy_timer_index,
@@ -1363,6 +1410,7 @@ void bx_floppy_ctrl_c::dma_read(Bit8u *data_byte)
   } else { // write normal data
     BX_FD_THIS s.floppy_buffer[BX_FD_THIS s.floppy_buffer_index++] = *data_byte;
 
+    BX_FD_THIS s.TC = get_tc();
     if ((BX_FD_THIS s.floppy_buffer_index >= 512) || (BX_FD_THIS s.TC)) {
       logical_sector = (BX_FD_THIS s.cylinder[drive] * BX_FD_THIS s.media[drive].heads * BX_FD_THIS s.media[drive].sectors_per_track) +
                        (BX_FD_THIS s.head[drive] * BX_FD_THIS s.media[drive].sectors_per_track) +
@@ -1383,11 +1431,17 @@ void bx_floppy_ctrl_c::dma_read(Bit8u *data_byte)
                   512, TO_FLOPPY);
       increment_sector(); // increment to next sector after writing current one
       BX_FD_THIS s.floppy_buffer_index = 0;
-      DEV_dma_set_drq(FLOPPY_DMA_CHAN, 0);
+      if (!(BX_FD_THIS s.main_status_reg & FD_MS_NDMA)) {
+        DEV_dma_set_drq(FLOPPY_DMA_CHAN, 0);
+      }
       // time to write one sector at 300 rpm
       sector_time = 200000 / BX_FD_THIS s.media[drive].sectors_per_track;
       bx_pc_system.activate_timer(BX_FD_THIS s.floppy_timer_index,
                                   sector_time , 0);
+      // the following is a kludge; i (jc) don't know how to work with the timer
+      if ((BX_FD_THIS s.main_status_reg & FD_MS_NDMA) && BX_FD_THIS s.TC) {
+        enter_result_phase();
+      }
     }
   }
 }
@@ -1729,7 +1783,7 @@ bx_bool bx_floppy_ctrl_c::evaluate_media(Bit8u devtype, Bit8u type, char *path,
         }
         media->sectors = media->heads * media->tracks * media->sectors_per_track;
     }
-    return(1); // success
+    return (media->sectors > 0); // success
   }
 
   else if ( S_ISCHR(stat_buf.st_mode)
@@ -1749,7 +1803,7 @@ bx_bool bx_floppy_ctrl_c::evaluate_media(Bit8u devtype, Bit8u type, char *path,
       media->heads             = floppy_type[type_idx].hd;
       media->sectors_per_track = floppy_type[type_idx].spt;
       media->sectors           = floppy_type[type_idx].sectors;
-      return 1;
+      return (media->sectors > 0);
     }
     media->tracks            = floppy_geom.track;
     media->heads             = floppy_geom.head;
@@ -1766,7 +1820,7 @@ bx_bool bx_floppy_ctrl_c::evaluate_media(Bit8u devtype, Bit8u type, char *path,
     media->sectors_per_track = floppy_type[type_idx].spt;
     media->sectors           = floppy_type[type_idx].sectors;
 #endif
-    return 1; // success
+    return (media->sectors > 0); // success
   } else {
     // unknown file type
     BX_ERROR(("unknown mode type"));
@@ -1783,7 +1837,7 @@ void bx_floppy_ctrl_c::enter_result_phase(void)
 
   /* these are always the same */
   BX_FD_THIS s.result_index = 0;
-  BX_FD_THIS s.main_status_reg &= 0x0f; // leave drive status untouched
+  // not necessary to clear any status bits, we're about to set them all
   BX_FD_THIS s.main_status_reg |= FD_MS_MRQ | FD_MS_DIO | FD_MS_BUSY;
 
   /* invalid command */
@@ -1809,7 +1863,7 @@ void bx_floppy_ctrl_c::enter_result_phase(void)
         BX_FD_THIS s.result[i] = BX_FD_THIS s.cylinder[i];
       }
       BX_FD_THIS s.result[4] = (BX_FD_THIS s.SRT << 4) | BX_FD_THIS s.HUT;
-      BX_FD_THIS s.result[5] = (BX_FD_THIS s.HLT << 1) | BX_FD_THIS s.non_dma;
+      BX_FD_THIS s.result[5] = (BX_FD_THIS s.HLT << 1) | ((BX_FD_THIS s.main_status_reg & FD_MS_NDMA) ? 1 : 0);
       BX_FD_THIS s.result[6] = BX_FD_THIS s.eot[drive];
       BX_FD_THIS s.result[7] = (BX_FD_THIS s.lock << 7) | (BX_FD_THIS s.perp_mode & 0x7f);
       BX_FD_THIS s.result[8] = BX_FD_THIS s.config;
@@ -1856,7 +1910,7 @@ void bx_floppy_ctrl_c::enter_result_phase(void)
 
 void bx_floppy_ctrl_c::enter_idle_phase(void)
 {
-  BX_FD_THIS s.main_status_reg &= 0x0f;      // leave drive status untouched
+  BX_FD_THIS s.main_status_reg &= (FD_MS_NDMA | 0x0f);  // leave drive status untouched
   BX_FD_THIS s.main_status_reg |= FD_MS_MRQ; // data register ready
 
   BX_FD_THIS s.command_complete = 1; /* waiting for new command */
@@ -1888,3 +1942,27 @@ void bx_floppy_ctrl_c::reset_changeline(void)
   if (BX_FD_THIS s.media_present[drive])
     BX_FD_THIS s.DIR[drive] &= ~0x80;
 }
+
+bx_bool bx_floppy_ctrl_c::get_tc(void)
+{
+  Bit8u drive;
+  bx_bool terminal_count;
+  if (BX_FD_THIS s.main_status_reg & FD_MS_NDMA) {
+    drive = BX_FD_THIS s.DOR & 0x03;
+    /* figure out if we've sent all the data, in non-DMA mode...
+     * the drive stays on the same cylinder for a read or write, so that's
+     * not going to be an issue. EOT stands for the last sector to be I/Od.
+     * it does all the head 0 sectors first, then the second if any.
+     * now, regarding reaching the end of the sector:
+     *  == 512 would make it more precise, allowing one to spot bugs...
+     *  >= 512 makes it more robust, but allows for sloppy code...
+     *  pick your poison?
+     * note: byte and head are 0-based; eot, sector, and heads are 1-based. */
+    terminal_count = ((BX_FD_THIS s.floppy_buffer_index == 512) &&
+     (BX_FD_THIS s.sector[drive] == BX_FD_THIS s.eot[drive]) &&
+     (BX_FD_THIS s.head[drive] == (BX_FD_THIS s.media[drive].heads - 1)));
+  } else {
+    terminal_count = DEV_dma_get_tc();
+  }
+  return terminal_count;
+}
