@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: gameport.cc,v 1.13 2007/04/03 22:38:48 sshwarts Exp $
+// $Id: gameport.cc,v 1.14 2007/09/28 19:51:59 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2003  MandrakeSoft S.A.
@@ -120,10 +120,9 @@ void bx_gameport_c::reset(unsigned type)
   // nothing for now
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_gameport_c::register_state(void)
 {
-  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "gameport", "Gameport State", 6);
+  bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "gameport", "Gameport State", 6);
   BXRS_HEX_PARAM_FIELD(list, port, BX_GAMEPORT_THIS port);
   BXRS_DEC_PARAM_FIELD(list, delay_x, BX_GAMEPORT_THIS delay_x);
   BXRS_DEC_PARAM_FIELD(list, delay_y, BX_GAMEPORT_THIS delay_y);
@@ -131,7 +130,6 @@ void bx_gameport_c::register_state(void)
   BXRS_PARAM_BOOL(list, timer_y, BX_GAMEPORT_THIS timer_y);
   BXRS_DEC_PARAM_FIELD(list, write_usec, BX_GAMEPORT_THIS write_usec);
 }
-#endif
 
 void bx_gameport_c::poll_joydev(void)
 {
