@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: harddrv.h,v 1.44 2006/08/05 07:49:31 vruppert Exp $
+// $Id: harddrv.h,v 1.50 2007/09/28 19:52:01 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -56,7 +56,7 @@ typedef struct {
     bx_bool index_pulse;
     unsigned index_pulse_count;
     bx_bool err;
-    } status;
+  } status;
   Bit8u    error_register;
   Bit8u    head_no;
   union {
@@ -89,13 +89,23 @@ typedef struct {
   Bit8u    lba_mode;
   bx_bool  packet_dma;
   Bit8u    mdma_mode;
+  Bit8u    udma_mode;
   struct {
     bx_bool reset;       // 0=normal, 1=reset controller
     bx_bool disable_irq; // 0=allow irq, 1=disable irq
-    } control;
+  } control;
   Bit8u    reset_in_progress;
   Bit8u    features;
-  } controller_t;
+  struct {
+    Bit8u  feature;
+    Bit8u  nsector;
+    Bit8u  sector;
+    Bit8u  lcyl;
+    Bit8u  hcyl;
+  } hob;
+  Bit32u   num_sectors;
+  bx_bool  lba48;
+} controller_t;
 
 struct sense_info_t {
   sense_t sense_key;
@@ -161,7 +171,6 @@ class bx_hard_drive_c : public bx_hard_drive_stub_c {
 public:
   bx_hard_drive_c();
   virtual ~bx_hard_drive_c();
-  virtual void   close_harddrive(void);
   virtual void   init();
   virtual void   reset(unsigned type);
   virtual Bit32u   get_device_handle(Bit8u channel, Bit8u device);
@@ -173,9 +182,7 @@ public:
   virtual bx_bool  bmdma_write_sector(Bit8u channel, Bit8u *buffer);
   virtual void     bmdma_complete(Bit8u channel);
 #endif
-#if BX_SUPPORT_SAVE_RESTORE
   virtual void     register_state(void);
-#endif
 
   virtual Bit32u virt_read_handler(Bit32u address, unsigned io_len) {
     return read_handler (this, address, io_len);
@@ -198,7 +205,7 @@ public:
 
 private:
 
-  BX_HD_SMF bx_bool calculate_logical_address(Bit8u channel, off_t *sector) BX_CPP_AttrRegparmN(2);
+  BX_HD_SMF bx_bool calculate_logical_address(Bit8u channel, Bit64s *sector) BX_CPP_AttrRegparmN(2);
   BX_HD_SMF void increment_address(Bit8u channel) BX_CPP_AttrRegparmN(1);
   BX_HD_SMF void identify_drive(Bit8u channel);
   BX_HD_SMF void identify_ATAPI_drive(Bit8u channel);
@@ -214,6 +221,7 @@ private:
   BX_HD_SMF void set_signature(Bit8u channel, Bit8u id);
   BX_HD_SMF bx_bool ide_read_sector(Bit8u channel, Bit8u *buffer, Bit32u buffer_size);
   BX_HD_SMF bx_bool ide_write_sector(Bit8u channel, Bit8u *buffer, Bit32u buffer_size);
+  BX_HD_SMF void lba48_transform(Bit8u channel, bx_bool lba48);
 
   // FIXME:
   // For each ATA channel we should have one controller struct
@@ -227,6 +235,7 @@ private:
       // they are fetched and returned via a return(), so
       // there's no need to keep them in x86 endian format.
       Bit16u id_drive[256];
+      bx_bool identify_set;
 
       controller_t controller;
       cdrom_t cdrom;
