@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: hdimage.h,v 1.6 2006/06/16 07:29:33 vruppert Exp $
+// $Id: hdimage.h,v 1.9 2007/03/10 12:53:54 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2005  MandrakeSoft S.A.
@@ -131,24 +131,25 @@ class device_image_t
   public:
       // Default constructor
       device_image_t();
+      virtual ~device_image_t() {}
 
       // Open a image. Returns non-negative if successful.
-      virtual int open (const char* pathname) = 0;
+      virtual int open(const char* pathname) = 0;
 
       // Close the image.
-      virtual void close () = 0;
+      virtual void close() = 0;
 
       // Position ourselves. Return the resulting offset from the
       // beginning of the file.
-      virtual off_t lseek (off_t offset, int whence) = 0;
+      virtual Bit64s lseek(Bit64s offset, int whence) = 0;
 
       // Read count bytes to the buffer buf. Return the number of
       // bytes read (count).
-      virtual ssize_t read (void* buf, size_t count) = 0;
+      virtual ssize_t read(void* buf, size_t count) = 0;
 
       // Write count bytes from buf. Return the number of bytes
       // written (count).
-      virtual ssize_t write (const void* buf, size_t count) = 0;
+      virtual ssize_t write(const void* buf, size_t count) = 0;
 
       unsigned cylinders;
       unsigned heads;
@@ -161,25 +162,25 @@ class default_image_t : public device_image_t
 {
   public:
       // Open a image. Returns non-negative if successful.
-      int open (const char* pathname);
+      int open(const char* pathname);
 
       // Open an image with specific flags. Returns non-negative if successful.
-      int open (const char* pathname, int flags);
+      int open(const char* pathname, int flags);
 
       // Close the image.
-      void close ();
+      void close();
 
       // Position ourselves. Return the resulting offset from the
       // beginning of the file.
-      off_t lseek (off_t offset, int whence);
+      Bit64s lseek(Bit64s offset, int whence);
 
       // Read count bytes to the buffer buf. Return the number of
       // bytes read (count).
-      ssize_t read (void* buf, size_t count);
+      ssize_t read(void* buf, size_t count);
 
       // Write count bytes from buf. Return the number of bytes
       // written (count).
-      ssize_t write (const void* buf, size_t count);
+      ssize_t write(const void* buf, size_t count);
 
   private:
       int fd;
@@ -246,22 +247,22 @@ class sparse_image_t : public device_image_t
       sparse_image_t();
 
       // Open a image. Returns non-negative if successful.
-      int open (const char* pathname);
+      int open(const char* pathname);
 
       // Close the image.
-      void close ();
+      void close();
 
       // Position ourselves. Return the resulting offset from the
       // beginning of the file.
-      off_t lseek (off_t offset, int whence);
+      Bit64s lseek(Bit64s offset, int whence);
 
       // Read count bytes to the buffer buf. Return the number of
       // bytes read (count).
-      ssize_t read (void* buf, size_t count);
+      ssize_t read(void* buf, size_t count);
 
       // Write count bytes from buf. Return the number of bytes
       // written (count).
-      ssize_t write (const void* buf, size_t count);
+      ssize_t write(const void* buf, size_t count);
 
   private:
  int fd;
@@ -283,23 +284,23 @@ class sparse_image_t : public device_image_t
  int     pagesize_shift;
  Bit32u  pagesize_mask;
 
- off_t   data_start;
- off_t   underlying_filesize;
+ Bit64s  data_start;
+ Bit64s  underlying_filesize;
 
  char *  pathname;
 
- off_t position;
+ Bit64s position;
 
  Bit32u position_virtual_page;
  Bit32u position_physical_page;
  Bit32u position_page_offset;
 
- off_t underlying_current_filepos;
+ Bit64s underlying_current_filepos;
 
- off_t total_size;
+ Bit64s total_size;
 
  void panic(const char * message);
- off_t get_physical_offset();
+ Bit64s get_physical_offset();
  void set_virtual_page(Bit32u new_virtual_page);
  void read_header();
  ssize_t read_page_fragment(Bit32u read_virtual_page, Bit32u read_page_offset, size_t read_size, void * buf);
@@ -316,22 +317,22 @@ class dll_image_t : public device_image_t
 {
   public:
       // Open a image. Returns non-negative if successful.
-      int open (const char* pathname);
+      int open(const char* pathname);
 
       // Close the image.
-      void close ();
+      void close();
 
       // Position ourselves. Return the resulting offset from the
       // beginning of the file.
-      off_t lseek (off_t offset, int whence);
+      Bit64s lseek(Bit64s offset, int whence);
 
       // Read count bytes to the buffer buf. Return the number of
       // bytes read (count).
-      ssize_t read (void* buf, size_t count);
+      ssize_t read(void* buf, size_t count);
 
       // Write count bytes from buf. Return the number of bytes
       // written (count).
-      ssize_t write (const void* buf, size_t count);
+      ssize_t write(const void* buf, size_t count);
 
   private:
       int vunit,vblk;
@@ -375,6 +376,7 @@ class growing_image_t : public device_image_t
   public:
       // Contructor
       growing_image_t();
+      virtual ~growing_image_t();
 
       // Open a image. Returns non-negative if successful.
       int open(const char* pathname);
@@ -404,6 +406,7 @@ class undoable_image_t : public device_image_t
   public:
       // Contructor
       undoable_image_t(const char* redolog_name);
+      virtual ~undoable_image_t();
 
       // Open a image. Returns non-negative if successful.
       int open(const char* pathname);
@@ -436,6 +439,7 @@ class volatile_image_t : public device_image_t
   public:
       // Contructor
       volatile_image_t(const char* redolog_name);
+      virtual ~volatile_image_t();
 
       // Open a image. Returns non-negative if successful.
       int open(const char* pathname);
@@ -476,25 +480,25 @@ class z_ro_image_t : public device_image_t
       z_ro_image_t();
 
       // Open a image. Returns non-negative if successful.
-      int open (const char* pathname);
+      int open(const char* pathname);
 
       // Close the image.
-      void close ();
+      void close();
 
       // Position ourselves. Return the resulting offset from the
       // beginning of the file.
-      off_t lseek (off_t offset, int whence);
+      Bit64s lseek(Bit64s offset, int whence);
 
       // Read count bytes to the buffer buf. Return the number of
       // bytes read (count).
-      ssize_t read (void* buf, size_t count);
+      ssize_t read(void* buf, size_t count);
 
       // Write count bytes from buf. Return the number of bytes
       // written (count).
-      ssize_t write (const void* buf, size_t count);
+      ssize_t write(const void* buf, size_t count);
 
   private:
-      off_t offset;
+      Bit64s offset;
       int fd;
       gzFile gzfile;
 
@@ -506,24 +510,25 @@ class z_undoable_image_t : public device_image_t
   public:
       // Contructor
       z_undoable_image_t(Bit64u size, const char* redolog_name);
+      virtual ~z_undoable_image_t();
 
       // Open a image. Returns non-negative if successful.
-      int open (const char* pathname);
+      int open(const char* pathname);
 
       // Close the image.
-      void close ();
+      void close();
 
       // Position ourselves. Return the resulting offset from the
       // beginning of the file.
-      off_t lseek (off_t offset, int whence);
+      Bit64s lseek(Bit64s offset, int whence);
 
       // Read count bytes to the buffer buf. Return the number of
       // bytes read (count).
-      ssize_t read (void* buf, size_t count);
+      ssize_t read(void* buf, size_t count);
 
       // Write count bytes from buf. Return the number of bytes
       // written (count).
-      ssize_t write (const void* buf, size_t count);
+      ssize_t write(const void* buf, size_t count);
 
   private:
       redolog_t       *redolog;       // Redolog instance
@@ -538,24 +543,25 @@ class z_volatile_image_t : public device_image_t
   public:
       // Contructor
       z_volatile_image_t(Bit64u size, const char* redolog_name);
+      virtual ~z_volatile_image_t();
 
       // Open a image. Returns non-negative if successful.
-      int open (const char* pathname);
+      int open(const char* pathname);
 
       // Close the image.
-      void close ();
+      void close();
 
       // Position ourselves. Return the resulting offset from the
       // beginning of the file.
-      off_t lseek (off_t offset, int whence);
+      Bit64s lseek(Bit64s offset, int whence);
 
       // Read count bytes to the buffer buf. Return the number of
       // bytes read (count).
-      ssize_t read (void* buf, size_t count);
+      ssize_t read(void* buf, size_t count);
 
       // Write count bytes from buf. Return the number of bytes
       // written (count).
-      ssize_t write (const void* buf, size_t count);
+      ssize_t write(const void* buf, size_t count);
 
   private:
       redolog_t       *redolog;       // Redolog instance
