@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: ioapic.cc,v 1.32 2006/06/05 05:39:21 sshwarts Exp $
+// $Id: ioapic.cc,v 1.35 2007/09/28 19:52:02 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -42,7 +42,7 @@ static bx_bool ioapic_read(unsigned long a20addr, unsigned long len, void *data,
 static bx_bool ioapic_write(unsigned long a20addr, unsigned long len, void *data, void *param)
 {
   if (len != 4) {
-    BX_PANIC (("I/O apic write with len=%d (should be 4)", len));
+    BX_PANIC (("I/O apic write with len=%ld (should be 4)", len));
   }
   bx_ioapic.write(a20addr, (Bit32u*) data, len);
   return 1;
@@ -62,13 +62,11 @@ void bx_io_redirect_entry_t::sprintf_self(char *buf)
      (unsigned) vector());
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_io_redirect_entry_t::register_state(bx_param_c *parent)
 {
   BXRS_HEX_PARAM_SIMPLE(parent, lo);
   BXRS_HEX_PARAM_SIMPLE(parent, hi);
 }
-#endif
 
 #define BX_IOAPIC_BASE_ADDR (0xfec00000)
 
@@ -217,6 +215,7 @@ void bx_ioapic_c::receive_eoi(Bit8u vector)
 void bx_ioapic_c::service_ioapic()
 {
   static unsigned int stuck = 0;
+  Bit8u vector = 0;
   // look in IRR and deliver any interrupts that are not masked.
   BX_DEBUG(("IOAPIC: servicing"));
   for (unsigned bit=0; bit < BX_IOAPIC_NUM_PINS; bit++) {
@@ -226,9 +225,11 @@ void bx_ioapic_c::service_ioapic()
       if (! entry->is_masked()) {
         // clear irr bit and deliver
         if (entry->delivery_mode() == 7) {
-          BX_PANIC(("ExtINT not implemented yet"));
+          vector = DEV_pic_iac();
+        } else {
+          vector = entry->vector();
         }
-        bx_bool done = apic_bus_deliver_interrupt(entry->vector(), entry->destination(), entry->delivery_mode(), entry->destination_mode(), entry->pin_polarity(), entry->trigger_mode());
+        bx_bool done = apic_bus_deliver_interrupt(vector, entry->destination(), entry->delivery_mode(), entry->destination_mode(), entry->pin_polarity(), entry->trigger_mode());
         if (done) {
           if (! entry->trigger_mode())
             irr &= ~mask;
@@ -238,7 +239,7 @@ void bx_ioapic_c::service_ioapic()
           entry->set_delivery_status();
           stuck++;
           if (stuck > 5)
-            BX_INFO(("vector %#x stuck?", entry->vector()));
+            BX_INFO(("vector %#x stuck?", vector));
         }
       }
       else {
@@ -248,10 +249,9 @@ void bx_ioapic_c::service_ioapic()
   }
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_ioapic_c::register_state(void)
 {
-  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "ioapic", "IOAPIC State");
+  bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "ioapic", "IOAPIC State", 4);
   BXRS_HEX_PARAM_SIMPLE(list, ioregsel);
   BXRS_HEX_PARAM_SIMPLE(list, intin);
   BXRS_HEX_PARAM_SIMPLE(list, irr);
@@ -264,6 +264,5 @@ void bx_ioapic_c::register_state(void)
     ioredtbl[i].register_state(entry);
   }
 }
-#endif
 
 #endif /* if BX_SUPPORT_APIC */
