@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: iodev.h,v 1.69 2006/01/17 20:16:59 vruppert Exp $
+// $Id: iodev.h,v 1.89 2007/09/28 19:52:02 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -58,7 +58,6 @@ class bx_g2h_c;
 #endif
 
 
-
 typedef Bit32u (*bx_read_handler_t)(void *, Bit32u, unsigned);
 typedef void   (*bx_write_handler_t)(void *, Bit32u, Bit32u, unsigned);
 
@@ -81,24 +80,47 @@ typedef void   (*bx_write_handler_t)(void *, Bit32u, Bit32u, unsigned);
 // virtual (= 0).
 class BOCHSAPI bx_devmodel_c : public logfunctions {
   public:
-  virtual ~bx_devmodel_c () {}
+  virtual ~bx_devmodel_c() {}
   virtual void init_mem(BX_MEM_C *) {}
   virtual void init(void) {}
   virtual void reset(unsigned type) {}
-  virtual void device_load_state () {}
-  virtual void device_save_state () {}
+  virtual void register_state(void) {}
+  virtual void after_restore_state(void) {}
+};
+
+//////////////////////////////////////////////////////////////////////
+// declare stubs for PCI devices
+//////////////////////////////////////////////////////////////////////
+
+class bx_list_c;
+
+// the best should be deriving of bx_pci_device_stub_c from bx_devmodel_c
+// but it make serious problems for cirrus_svga device
+class BOCHSAPI bx_pci_device_stub_c {
+public:
+  virtual ~bx_pci_device_stub_c() {}
+  
+  virtual Bit32u pci_read_handler(Bit8u address, unsigned io_len) {
+    return 0;
+  }
+
+  virtual void pci_write_handler(Bit8u address, Bit32u value, unsigned io_len) {}
+
+  void register_pci_state(bx_list_c *list, Bit8u *pci_conf);
 };
 
 //////////////////////////////////////////////////////////////////////
 // declare stubs for devices
 //////////////////////////////////////////////////////////////////////
 
+//////////////////////////////////////////////////////////////////////
 #define STUBFUNC(dev,method) \
    pluginlog->panic("%s called in %s stub. you must not have loaded the %s plugin", #dev, #method, #dev )
+//////////////////////////////////////////////////////////////////////
 
 class BOCHSAPI bx_keyb_stub_c : public bx_devmodel_c {
-  public:
-  virtual ~bx_keyb_stub_c () {}
+public:
+  virtual ~bx_keyb_stub_c() {}
   // stubs for bx_keyb_c methods
   virtual void mouse_motion(int delta_x, int delta_y, int delta_z, unsigned button_state) {
     STUBFUNC(keyboard, mouse_motion);
@@ -112,10 +134,7 @@ class BOCHSAPI bx_keyb_stub_c : public bx_devmodel_c {
 };
 
 class BOCHSAPI bx_hard_drive_stub_c : public bx_devmodel_c {
-  public:
-  virtual void   close_harddrive(void) {
-    STUBFUNC(HD, close_harddrive);
-  }
+public:
   virtual void   init() {
     STUBFUNC(HD, init);
   }
@@ -155,7 +174,7 @@ class BOCHSAPI bx_hard_drive_stub_c : public bx_devmodel_c {
 };
 
 class BOCHSAPI bx_floppy_stub_c : public bx_devmodel_c {
-  public:
+public:
   virtual unsigned get_media_status(unsigned drive) {
     STUBFUNC(floppy,  get_media_status); return 0;
   }
@@ -165,7 +184,7 @@ class BOCHSAPI bx_floppy_stub_c : public bx_devmodel_c {
 };
 
 class BOCHSAPI bx_cmos_stub_c : public bx_devmodel_c {
-  public:
+public:
   virtual Bit32u get_reg(unsigned reg) {
     STUBFUNC(cmos, get_reg); return 0;
   }
@@ -178,27 +197,24 @@ class BOCHSAPI bx_cmos_stub_c : public bx_devmodel_c {
   virtual void checksum_cmos(void) {
     STUBFUNC(cmos, checksum);
   }
-  virtual void save_image(void) {
-    STUBFUNC(cmos, save_image);
-  }
 };
 
 class BOCHSAPI bx_dma_stub_c : public bx_devmodel_c {
-  public:
+public:
   virtual unsigned registerDMA8Channel(
     unsigned channel,
     void (* dmaRead)(Bit8u *data_byte),
     void (* dmaWrite)(Bit8u *data_byte),
-    const char *name
-    ) {
+    const char *name)
+  {
     STUBFUNC(dma, registerDMA8Channel); return 0;
   }
   virtual unsigned registerDMA16Channel(
     unsigned channel,
     void (* dmaRead)(Bit16u *data_word),
     void (* dmaWrite)(Bit16u *data_word),   
-    const char *name
-    ) {
+    const char *name)
+  {
     STUBFUNC(dma, registerDMA16Channel); return 0;
   }
   virtual unsigned unregisterDMAChannel(unsigned channel) {
@@ -216,7 +232,7 @@ class BOCHSAPI bx_dma_stub_c : public bx_devmodel_c {
 };
 
 class BOCHSAPI bx_pic_stub_c : public bx_devmodel_c {
-  public:
+public:
   virtual void raise_irq(unsigned irq_no) {
     STUBFUNC(pic, raise_irq); 
   }
@@ -235,7 +251,7 @@ class BOCHSAPI bx_pic_stub_c : public bx_devmodel_c {
 };
 
 class BOCHSAPI bx_vga_stub_c : public bx_devmodel_c {
-  public:
+public:
   virtual void redraw_area(unsigned x0, unsigned y0, 
                            unsigned width, unsigned height) {
     STUBFUNC(vga, redraw_area);  
@@ -259,13 +275,12 @@ class BOCHSAPI bx_vga_stub_c : public bx_devmodel_c {
   virtual void dump_status(void) {}
 };
 
-class BOCHSAPI bx_pci_stub_c : public bx_devmodel_c {
-  public:
-  virtual bx_bool register_pci_handlers(void *this_ptr,
-                                        Bit32u (*bx_pci_read_handler)(void *, Bit8u, unsigned),
-                                        void(*bx_pci_write_handler)(void *, Bit8u, Bit32u, unsigned),
+class BOCHSAPI bx_pci_bridge_stub_c : public bx_devmodel_c, public bx_pci_device_stub_c {
+public:
+  virtual bx_bool register_pci_handlers(bx_pci_device_stub_c *device,
                                         Bit8u *devfunc, const char *name,
-                                        const char *descr) {
+                                        const char *descr)
+  {
     STUBFUNC(pci, register_pci_handlers); return 0;
   }
   virtual bx_bool is_pci_device (const char *name) {
@@ -293,15 +308,15 @@ class BOCHSAPI bx_pci_stub_c : public bx_devmodel_c {
   virtual void print_i440fx_state(void) {}
 };
 
-class BOCHSAPI bx_pci2isa_stub_c : public bx_devmodel_c {
-  public:
+class BOCHSAPI bx_pci2isa_stub_c : public bx_devmodel_c, public bx_pci_device_stub_c {
+public:
   virtual void pci_set_irq (Bit8u devfunc, unsigned line, bx_bool level) {
     STUBFUNC(pci2isa, pci_set_irq);
   }
 };
 
-class BOCHSAPI bx_pci_ide_stub_c : public bx_devmodel_c {
-  public:
+class BOCHSAPI bx_pci_ide_stub_c : public bx_devmodel_c, public bx_pci_device_stub_c {
+public:
   virtual bx_bool bmdma_present(void) {
     return 0;
   }
@@ -309,31 +324,31 @@ class BOCHSAPI bx_pci_ide_stub_c : public bx_devmodel_c {
 };
 
 class BOCHSAPI bx_ne2k_stub_c : public bx_devmodel_c {
-  public:
+public:
   virtual void print_info(FILE *file, int page, int reg, int nodups) {}
 };
 
 class BOCHSAPI bx_speaker_stub_c : public bx_devmodel_c {
-  public:
+public:
   virtual void beep_on(float frequency) {}
   virtual void beep_off() {}
 };
 
 class BOCHSAPI bx_serial_stub_c : public bx_devmodel_c {
-  public:
+public:
   virtual void serial_mouse_enq(int delta_x, int delta_y, int delta_z, unsigned button_state) {
     STUBFUNC(serial, serial_mouse_enq);
   }
 };
 
 #if BX_SUPPORT_PCIUSB
-class BOCHSAPI bx_usb_stub_c : public bx_devmodel_c {
-  public:
+class BOCHSAPI bx_pci_usb_stub_c : public bx_devmodel_c, public bx_pci_device_stub_c {
+public:
   virtual void usb_mouse_enq(int delta_x, int delta_y, int delta_z, unsigned button_state) {
     STUBFUNC(pciusb, usb_mouse_enq);
   }
-  virtual void usb_mouse_enable(bx_bool enable) {
-    STUBFUNC(pciusb, usb_mouse_enable);
+  virtual void usb_mouse_enabled_changed(bx_bool enable) {
+    STUBFUNC(pciusb, usb_mouse_enabled_changed);
   }
   virtual bx_bool usb_key_enq(Bit8u *scan_code) {
     STUBFUNC(pciusb, usb_key_enq);
@@ -348,9 +363,16 @@ class BOCHSAPI bx_usb_stub_c : public bx_devmodel_c {
 };
 #endif
 
+#if BX_SUPPORT_ACPI
+class BOCHSAPI bx_acpi_ctrl_stub_c : public bx_devmodel_c, public bx_pci_device_stub_c {
+public:
+  virtual void generate_smi(Bit8u value) {}
+};
+#endif
+
 #if BX_SUPPORT_BUSMOUSE
 class BOCHSAPI bx_busm_stub_c : public bx_devmodel_c {
-  public:
+public:
   virtual void bus_mouse_enq(int delta_x, int delta_y, int delta_z, unsigned button_state) {
     STUBFUNC(busmouse, bus_mouse_enq);
   }
@@ -360,7 +382,9 @@ class BOCHSAPI bx_busm_stub_c : public bx_devmodel_c {
 class BOCHSAPI bx_devices_c : public logfunctions {
 public:
   bx_devices_c(void);
-  ~bx_devices_c(void);
+ ~bx_devices_c(void);
+  // Initialize the device stubs (in constructur and exit())
+  void init_stubs(void);
   // Register I/O addresses and IRQ lines. Initialize any internal
   // structures.  init() is called only once, even if the simulator
   // reboots or is restarted.
@@ -369,29 +393,33 @@ public:
   // The types of reset conditions are defined in bochs.h:
   // power-on, hardware, or software.
   void reset(unsigned type);
+  // Cleanup the devices when the simulation quits.
+  void exit(void);
+  void register_state(void);
+  void after_restore_state(void);
   BX_MEM_C *mem;  // address space associated with these devices
-  bx_bool register_io_read_handler(void *this_ptr, bx_read_handler_t f, 
-		  Bit32u addr, const char *name, Bit8u mask );
-  bx_bool unregister_io_read_handler( void *this_ptr, bx_read_handler_t f,
-                                        Bit32u addr, Bit8u mask );
-  bx_bool register_io_write_handler(void *this_ptr, bx_write_handler_t f, Bit32u addr, const char *name, Bit8u mask );
-  bx_bool unregister_io_write_handler( void *this_ptr, bx_write_handler_t f,
-                                        Bit32u addr, Bit8u mask );
-  bx_bool register_io_read_handler_range( void *this_ptr, bx_read_handler_t f,
-		  Bit32u begin_addr, Bit32u end_addr, 
-		  const char *name, Bit8u mask );
-  bx_bool register_io_write_handler_range( void *this_ptr, bx_write_handler_t f,
-		  Bit32u begin_addr, Bit32u end_addr, 
-		  const char *name, Bit8u mask );
-  bx_bool unregister_io_read_handler_range( void *this_ptr, bx_read_handler_t f,
-                                        Bit32u begin, Bit32u end, Bit8u mask );
-  bx_bool unregister_io_write_handler_range( void *this_ptr, bx_write_handler_t f,
-                                        Bit32u begin, Bit32u end, Bit8u mask );
-  bx_bool register_default_io_read_handler(void *this_ptr, bx_read_handler_t f, const char *name, Bit8u mask );
-  bx_bool register_default_io_write_handler(void *this_ptr, bx_write_handler_t f, const char *name, Bit8u mask );
+  bx_bool register_io_read_handler(void *this_ptr, bx_read_handler_t f,
+                                   Bit32u addr, const char *name, Bit8u mask);
+  bx_bool unregister_io_read_handler(void *this_ptr, bx_read_handler_t f,
+                                     Bit32u addr, Bit8u mask);
+  bx_bool register_io_write_handler(void *this_ptr, bx_write_handler_t f,
+                                    Bit32u addr, const char *name, Bit8u mask);
+  bx_bool unregister_io_write_handler(void *this_ptr, bx_write_handler_t f,
+                                      Bit32u addr, Bit8u mask );
+  bx_bool register_io_read_handler_range(void *this_ptr, bx_read_handler_t f,
+                                         Bit32u begin_addr, Bit32u end_addr,
+                                         const char *name, Bit8u mask);
+  bx_bool register_io_write_handler_range(void *this_ptr, bx_write_handler_t f,
+                                          Bit32u begin_addr, Bit32u end_addr,
+                                          const char *name, Bit8u mask );
+  bx_bool unregister_io_read_handler_range(void *this_ptr, bx_read_handler_t f,
+                                           Bit32u begin, Bit32u end, Bit8u mask);
+  bx_bool unregister_io_write_handler_range(void *this_ptr, bx_write_handler_t f,
+                                            Bit32u begin, Bit32u end, Bit8u mask);
+  bx_bool register_default_io_read_handler(void *this_ptr, bx_read_handler_t f, const char *name, Bit8u mask);
+  bx_bool register_default_io_write_handler(void *this_ptr, bx_write_handler_t f, const char *name, Bit8u mask);
   bx_bool register_irq(unsigned irq, const char *name);
   bx_bool unregister_irq(unsigned irq, const char *name);
-  void iodev_init(void);
   Bit32u inp(Bit16u addr, unsigned io_len) BX_CPP_AttrRegparmN(2);
   void   outp(Bit16u addr, Bit32u value, unsigned io_len) BX_CPP_AttrRegparmN(3);
 
@@ -402,9 +430,12 @@ public:
 #if BX_SUPPORT_APIC
   bx_ioapic_c       *ioapic;
 #endif
-  bx_pci_stub_c     *pluginPciBridge;
+  bx_pci_bridge_stub_c *pluginPciBridge;
   bx_pci2isa_stub_c *pluginPci2IsaBridge;
   bx_pci_ide_stub_c *pluginPciIdeController;
+#if BX_SUPPORT_ACPI
+  bx_acpi_ctrl_stub_c *pluginACPIController;
+#endif
   bx_devmodel_c     *pluginPciVgaAdapter;
   bx_devmodel_c     *pluginPciDevAdapter;
   bx_devmodel_c     *pluginPciPNicAdapter;
@@ -415,7 +446,7 @@ public:
   bx_cmos_stub_c    *pluginCmosDevice;
   bx_serial_stub_c  *pluginSerialDevice;
 #if BX_SUPPORT_PCIUSB
-  bx_usb_stub_c     *pluginPciUSBAdapter;
+  bx_pci_usb_stub_c *pluginPciUSBAdapter;
 #endif
   bx_devmodel_c     *pluginParallelDevice;
   bx_devmodel_c     *pluginUnmapped;
@@ -449,14 +480,17 @@ public:
   bx_pic_stub_c  stubPic;
   bx_floppy_stub_c  stubFloppy;
   bx_vga_stub_c  stubVga;
-  bx_pci_stub_c  stubPci;
+  bx_pci_bridge_stub_c  stubPci;
   bx_pci2isa_stub_c stubPci2Isa;
   bx_pci_ide_stub_c stubPciIde;
   bx_ne2k_stub_c    stubNE2k;
   bx_speaker_stub_c stubSpeaker;
   bx_serial_stub_c  stubSerial;
 #if BX_SUPPORT_PCIUSB
-  bx_usb_stub_c     stubUsbAdapter;
+  bx_pci_usb_stub_c stubUsbAdapter;
+#endif
+#if BX_SUPPORT_ACPI
+  bx_acpi_ctrl_stub_c stubACPIController;
 #endif
 
   // Some info to pass to devices which can handled bulk IO.  This allows
@@ -476,7 +510,7 @@ private:
 	struct io_handler_struct *prev;	
 	void *funct; // C++ type checking is great, but annoying
 	void *this_ptr;
-	const char *handler_name;  // name of device
+	char *handler_name;  // name of device
 	int usage_count;
 	Bit8u mask;          // io_len mask
   };
@@ -488,7 +522,7 @@ private:
   
   // more for informative purposes, the names of the devices which
   // are use each of the IRQ 0..15 lines are stored here
-  const char *irq_handler_name[BX_MAX_IRQS];
+  char *irq_handler_name[BX_MAX_IRQS];
 
   static Bit32u read_handler(void *this_ptr, Bit32u address, unsigned io_len);
   static void   write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len);
@@ -502,7 +536,12 @@ private:
   bx_bool is_serial_enabled ();
   bx_bool is_usb_enabled ();
   bx_bool is_parallel_enabled ();
-  };
+};
+
+#define DEV_MEM_READ_PHYSICAL(phy_addr, len, ptr) \
+  BX_MEM(0)->readPhysicalPage(NULL, phy_addr, len, ptr)
+#define DEV_MEM_WRITE_PHYSICAL(phy_addr, len, ptr) \
+  BX_MEM(0)->writePhysicalPage(NULL, phy_addr, len, ptr)
 
 #ifndef NO_DEVICE_INCLUDES
 
@@ -510,6 +549,9 @@ private:
 #include "iodev/pci.h"
 #include "iodev/pci2isa.h"
 #include "iodev/pci_ide.h"
+#if BX_SUPPORT_ACPI
+#include "iodev/acpi.h"
+#endif
 #if BX_SUPPORT_PCIVGA
 #include "iodev/pcivga.h"
 #endif
@@ -538,7 +580,6 @@ private:
 #endif
 #include "iodev/parallel.h"
 #include "iodev/pic.h"
-#include "iodev/pit.h"
 #include "iodev/pit_wrap.h"
 #include "iodev/virt_timer.h"
 #include "iodev/serial.h"
