@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: parallel.cc,v 1.30 2006/05/29 22:33:38 sshwarts Exp $
+// $Id: parallel.cc,v 1.33 2007/09/28 19:52:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -41,7 +41,7 @@ bx_parallel_c *theParallelDevice = NULL;
 
 int libparallel_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
-  theParallelDevice = new bx_parallel_c ();
+  theParallelDevice = new bx_parallel_c();
   bx_devices.pluginParallelDevice = theParallelDevice;
   BX_REGISTER_DEVICE_DEVMODEL(plugin, type, theParallelDevice, BX_PLUGIN_PARALLEL);
   return(0); // Success
@@ -49,6 +49,7 @@ int libparallel_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, c
 
 void libparallel_LTX_plugin_fini(void)
 {
+  delete theParallelDevice;
 }
 
 bx_parallel_c::bx_parallel_c()
@@ -66,6 +67,7 @@ bx_parallel_c::~bx_parallel_c()
     if (s[i].output != NULL)
       fclose(s[i].output);
   }
+  BX_DEBUG(("Exit"));
 }
 
 void bx_parallel_c::init(void)
@@ -75,7 +77,7 @@ void bx_parallel_c::init(void)
   char name[16], pname[20];
   bx_list_c *base;
 
-  BX_DEBUG(("Init $Id: parallel.cc,v 1.30 2006/05/29 22:33:38 sshwarts Exp $"));
+  BX_DEBUG(("Init $Id: parallel.cc,v 1.33 2007/09/28 19:52:03 sshwarts Exp $"));
 
   for (unsigned i=0; i<BX_N_PARALLEL_PORTS; i++) {
     sprintf(pname, "ports.parallel.%d", i+1);
@@ -121,14 +123,13 @@ void bx_parallel_c::reset(unsigned type)
 {
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_parallel_c::register_state(void)
 {
   unsigned i;
   char name[4], pname[20];
   bx_list_c *base, *port;
 
-  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "parallel", "Parallel Port State");
+  bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "parallel", "Parallel Port State", BX_N_PARALLEL_PORTS);
   for (i=0; i<BX_N_PARALLEL_PORTS; i++) {
     sprintf(pname, "ports.parallel.%d", i+1);
     base = (bx_list_c*) SIM->get_param(pname);
@@ -149,7 +150,6 @@ void bx_parallel_c::register_state(void)
     }
   }
 }
-#endif
 
 void bx_parallel_c::virtual_printer(Bit8u port)
 {
