@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pci.cc,v 1.54 2007/04/03 22:38:48 sshwarts Exp $
+// $Id: pci.cc,v 1.55 2007/09/28 19:52:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -150,10 +150,9 @@ bx_pci_bridge_c::reset(unsigned type)
   BX_PCI_THIS s.i440fx.pci_conf[0x72] = 0x02;
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_pci_bridge_c::register_state(void)
 {
-  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "pci_bridge", "PCI Bridge State", 3);
+  bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "pci_bridge", "PCI Bridge State", 3);
   BXRS_HEX_PARAM_FIELD(list, confAddr, BX_PCI_THIS s.i440fx.confAddr);
   BXRS_HEX_PARAM_FIELD(list, confData, BX_PCI_THIS s.i440fx.confData);
   bx_list_c *pci_conf = new bx_list_c(list, "pci_conf", 256);
@@ -168,7 +167,6 @@ void bx_pci_bridge_c::after_restore_state(void)
 {
   BX_PCI_THIS smram_control(BX_PCI_THIS s.i440fx.pci_conf[0x72]);
 }
-#endif
 
 // static IO port read callback handler
 // redirects to non-static class handler to avoid virtual functions
