@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pci_ide.cc,v 1.26 2006/06/14 16:44:33 sshwarts Exp $
+// $Id: pci_ide.cc,v 1.30 2007/09/28 19:52:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -42,10 +42,9 @@ bx_pci_ide_c *thePciIdeController = NULL;
 
 const Bit8u bmdma_iomask[16] = {1, 0, 1, 0, 4, 0, 0, 0, 1, 0, 1, 0, 4, 0, 0, 0};
 
-  int
-libpci_ide_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
+int libpci_ide_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
-  thePciIdeController = new bx_pci_ide_c ();
+  thePciIdeController = new bx_pci_ide_c();
   bx_devices.pluginPciIdeController = thePciIdeController;
   BX_REGISTER_DEVICE_DEVMODEL(plugin, type, thePciIdeController, BX_PLUGIN_PCI_IDE);
   return(0); // Success
@@ -53,6 +52,7 @@ libpci_ide_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *
 
 void libpci_ide_LTX_plugin_fini(void)
 {
+  delete thePciIdeController;
 }
 
 bx_pci_ide_c::bx_pci_ide_c()
@@ -73,7 +73,7 @@ bx_pci_ide_c::~bx_pci_ide_c()
   if (s.bmdma[1].buffer != NULL) {
     delete [] s.bmdma[1].buffer;
   }
-  BX_DEBUG(("Exit."));
+  BX_DEBUG(("Exit"));
 }
 
 void bx_pci_ide_c::init(void)
@@ -135,24 +135,19 @@ void bx_pci_ide_c::reset(unsigned type)
   }
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
+// save/restore code begin
 void bx_pci_ide_c::register_state(void)
 {
   char name[6];
-  unsigned i;
 
-  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "pci_ide", "PCI IDE Controller State");
+  bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "pci_ide", "PCI IDE Controller State", 5);
 
-  bx_list_c *pci_conf = new bx_list_c(list, "pci_conf", 256);
-  for (i=0; i<256; i++) {
-    sprintf(name, "0x%02x", i);
-    new bx_shadow_num_c(pci_conf, name, &BX_PIDE_THIS s.pci_conf[i], BASE_HEX);
-  }
+  register_pci_state(list, BX_PIDE_THIS s.pci_conf);
 
   new bx_shadow_data_c(list, "buffer0", BX_PIDE_THIS s.bmdma[0].buffer, 0x20000);
   new bx_shadow_data_c(list, "buffer1", BX_PIDE_THIS s.bmdma[1].buffer, 0x20000);
 
-  for (i=0; i<2; i++) {
+  for (unsigned i=0; i<2; i++) {
     sprintf(name, "%d", i);
     bx_list_c *ctrl = new bx_list_c(list, name, 7);
     BXRS_PARAM_BOOL(ctrl, cmd_ssbm, BX_PIDE_THIS s.bmdma[i].cmd_ssbm);
@@ -218,7 +213,7 @@ Bit64s bx_pci_ide_c::param_restore(bx_param_c *param, Bit64s val)
   }
   return val;
 }
-#endif
+// save/restore code end
 
 bx_bool bx_pci_ide_c::bmdma_present(void)
 {
