@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pcipnic.cc,v 1.21 2006/05/29 22:33:38 sshwarts Exp $
+// $Id: pcipnic.cc,v 1.25 2007/11/01 18:14:28 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2003  Fen Systems Ltd.
@@ -36,16 +36,18 @@ bx_pcipnic_c* thePNICDevice = NULL;
 
 const Bit8u pnic_iomask[16] = {2, 0, 2, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
 
-  int
-libpcipnic_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
+int libpcipnic_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
-  thePNICDevice = new bx_pcipnic_c ();
+  thePNICDevice = new bx_pcipnic_c();
   bx_devices.pluginPciPNicAdapter = thePNICDevice;
   BX_REGISTER_DEVICE_DEVMODEL(plugin, type, thePNICDevice, BX_PLUGIN_PCIPNIC);
   return 0; // Success
 }
 
-void libpcipnic_LTX_plugin_fini(void) {}
+void libpcipnic_LTX_plugin_fini(void)
+{
+  delete thePNICDevice;
+}
 
 bx_pcipnic_c::bx_pcipnic_c()
 {
@@ -55,7 +57,7 @@ bx_pcipnic_c::bx_pcipnic_c()
 
 bx_pcipnic_c::~bx_pcipnic_c()
 {
-  // nothing for now
+  BX_DEBUG(("Exit"));
 }
 
 void bx_pcipnic_c::init(void)
@@ -76,7 +78,7 @@ void bx_pcipnic_c::init(void)
 
   // This code ripped wholesale from ne2k.cc:
   // Attach to the simulated ethernet dev
-  char *ethmod = SIM->get_param_enum("ethmod", base)->get_selected();
+  const char *ethmod = SIM->get_param_enum("ethmod", base)->get_selected();
   BX_PNIC_THIS ethdev = eth_locator_c::create(ethmod,
                                               SIM->get_param_string("ethdev", base)->getptr(),
                                               (const char *) SIM->get_param_string("macaddr", base)->getptr(),
@@ -148,13 +150,12 @@ void bx_pcipnic_c::reset(unsigned type)
   set_irq_level(0);
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
 void bx_pcipnic_c::register_state(void)
 {
   unsigned i;
   char name[6];
 
-  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "pcipnic", "PCI Pseudo NIC State", 11);
+  bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "pcipnic", "PCI Pseudo NIC State", 11);
   new bx_shadow_num_c(list, "irqEnabled", &BX_PNIC_THIS s.irqEnabled);
   new bx_shadow_num_c(list, "rCmd", &BX_PNIC_THIS s.rCmd);
   new bx_shadow_num_c(list, "rStatus", &BX_PNIC_THIS s.rStatus);
@@ -169,11 +170,7 @@ void bx_pcipnic_c::register_state(void)
   }
   new bx_shadow_data_c(list, "rData", BX_PNIC_THIS s.rData, PNIC_DATA_SIZE);
   new bx_shadow_data_c(list, "recvRing", (Bit8u*)BX_PNIC_THIS s.recvRing, PNIC_RECV_RINGS*PNIC_DATA_SIZE);
-  bx_list_c *pci_conf = new bx_list_c(list, "pci_conf", 256);
-  for (i=0; i<256; i++) {
-    sprintf(name, "0x%02x", i);
-    new bx_shadow_num_c(pci_conf, name, &BX_PNIC_THIS s.pci_conf[i], BASE_HEX);
-  }
+  register_pci_state(list, BX_PNIC_THIS s.pci_conf);
 }
 
 void bx_pcipnic_c::after_restore_state(void)
@@ -185,7 +182,6 @@ void bx_pcipnic_c::after_restore_state(void)
     BX_INFO(("new base address: 0x%04x", BX_PNIC_THIS s.base_ioaddr));
   }
 }
-#endif
 
 void bx_pcipnic_c::set_irq_level(bx_bool level)
 {
