@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pciusb.h,v 1.18 2006/05/27 15:54:48 sshwarts Exp $
+// $Id: pciusb.h,v 1.30 2007/09/28 19:52:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2004  MandrakeSoft S.A.
@@ -41,147 +41,142 @@
 #define BX_USB_CONFDEV  1   /* only 1 USB hub currently */
 
 #define USB_NUM_PORTS   2   /* UHCI supports 2 ports per root hub */
-#define USB_CUR_DEVS    3
 
-#define TOKEN_IN    0x69
-#define TOKEN_OUT   0xE1
-#define TOKEN_SETUP 0x2D
+#define USB_TOKEN_IN    0x69
+#define USB_TOKEN_OUT   0xE1
+#define USB_TOKEN_SETUP 0x2D
 
-// device requests
-enum { GET_STATUS=0, CLEAR_FEATURE, SET_FEATURE=3, SET_ADDRESS=5, GET_DESCRIPTOR=6, SET_DESCRIPTOR,
-       GET_CONFIGURATION, SET_CONFIGURATION,
-// interface requests
-       GET_INTERFACE, SET_INTERFACE,
-// standard endpoint requests
-       SYNCH_FRAME
+#define USB_MSG_ATTACH   0x100
+#define USB_MSG_DETACH   0x101
+#define USB_MSG_RESET    0x102
+
+#define USB_RET_NODEV  (-1) 
+#define USB_RET_NAK    (-2)
+#define USB_RET_STALL  (-3)
+#define USB_RET_BABBLE (-4)
+#define USB_RET_ASYNC  (-5)
+
+#define USB_SPEED_LOW   0
+#define USB_SPEED_FULL  1
+#define USB_SPEED_HIGH  2
+
+#define USB_STATE_NOTATTACHED 0
+#define USB_STATE_ATTACHED    1
+//#define USB_STATE_POWERED     2
+#define USB_STATE_DEFAULT     3
+//#define USB_STATE_ADDRESS     4
+//#define USB_STATE_CONFIGURED  5
+#define USB_STATE_SUSPENDED   6
+
+#define USB_DIR_OUT  0
+#define USB_DIR_IN   0x80
+
+#define USB_TYPE_MASK			(0x03 << 5)
+#define USB_TYPE_STANDARD		(0x00 << 5)
+#define USB_TYPE_CLASS			(0x01 << 5)
+#define USB_TYPE_VENDOR			(0x02 << 5)
+#define USB_TYPE_RESERVED		(0x03 << 5)
+
+#define USB_RECIP_MASK			0x1f
+#define USB_RECIP_DEVICE		0x00
+#define USB_RECIP_INTERFACE		0x01
+#define USB_RECIP_ENDPOINT		0x02
+#define USB_RECIP_OTHER			0x03
+
+#define DeviceRequest ((USB_DIR_IN|USB_TYPE_STANDARD|USB_RECIP_DEVICE)<<8)
+#define DeviceOutRequest ((USB_DIR_OUT|USB_TYPE_STANDARD|USB_RECIP_DEVICE)<<8)
+#define InterfaceRequest \
+        ((USB_DIR_IN|USB_TYPE_STANDARD|USB_RECIP_INTERFACE)<<8)
+#define InterfaceOutRequest \
+        ((USB_DIR_OUT|USB_TYPE_STANDARD|USB_RECIP_INTERFACE)<<8)
+#define EndpointRequest ((USB_DIR_IN|USB_TYPE_STANDARD|USB_RECIP_ENDPOINT)<<8)
+#define EndpointOutRequest \
+        ((USB_DIR_OUT|USB_TYPE_STANDARD|USB_RECIP_ENDPOINT)<<8)
+
+#define USB_REQ_GET_STATUS		0x00
+#define USB_REQ_CLEAR_FEATURE		0x01
+#define USB_REQ_SET_FEATURE		0x03
+#define USB_REQ_SET_ADDRESS		0x05
+#define USB_REQ_GET_DESCRIPTOR		0x06
+#define USB_REQ_SET_DESCRIPTOR		0x07
+#define USB_REQ_GET_CONFIGURATION	0x08
+#define USB_REQ_SET_CONFIGURATION	0x09
+#define USB_REQ_GET_INTERFACE		0x0A
+#define USB_REQ_SET_INTERFACE		0x0B
+#define USB_REQ_SYNCH_FRAME		0x0C
+
+#define USB_DEVICE_SELF_POWERED		0
+#define USB_DEVICE_REMOTE_WAKEUP	1
+
+// USB 1.1
+#define USB_DT_DEVICE			0x01
+#define USB_DT_CONFIG			0x02
+#define USB_DT_STRING			0x03
+#define USB_DT_INTERFACE		0x04
+#define USB_DT_ENDPOINT			0x05
+// USB 2.0
+#define USB_DT_DEVICE_QUALIFIER         0x06
+#define USB_DT_OTHER_SPEED_CONFIG       0x07
+#define USB_DT_INTERFACE_POWER          0x08
+
+struct USBPacket {
+  int pid;
+  Bit8u devaddr;
+  Bit8u devep;
+  Bit8u *data;
+  int len;
 };
 
-#define SET_FEATURE_TEST_MODE 0 /////////TODO: I don't know yet what this value is to be
-
-// Descriptor types
-enum {  DEVICE=1, CONFIG, STRING, INTERFACE, ENDPOINT,         // USB 1.1
-        DEVICE_QUALIFIER, OTHER_SPEED_CONFIG, INTERFACE_POWER  // USB 2.0
+enum usbdev_type {
+  USB_DEV_TYPE_NONE=0,
+  USB_DEV_TYPE_MOUSE,
+  USB_DEV_TYPE_TABLET,
+  USB_DEV_TYPE_KEYPAD,
+  USB_DEV_TYPE_DISK
 };
 
-#define STATE_DEFAULT    0
-#define STATE_ADDRESS    1
-#define STATE_CONFIGURED 2
+class usb_hid_device_t;
 
-// setup packets
-struct REQUEST_PACKET {
-  Bit8u  request_type;
-  Bit8u  request;
-  Bit16u value;
-  Bit16u index;
-  Bit16u length;
-};
+void usb_dump_packet(Bit8u *data, unsigned size);
+int set_usb_string(Bit8u *buf, const char *str);
 
-#define KEYPAD_LEN   128
-struct KEYPAD {
-  Bit8u  scan_code[8];
-  Bit8u  keypad_packet[8];
-};
 
-#define USB_DEV_TYPE_NONE    0
-#define USB_DEV_TYPE_MOUSE   1
-#define USB_DEV_TYPE_KEYPAD  2
-#define USB_DEV_TYPE_FLASH   3
+class usb_device_t : public logfunctions {
+public:
+  usb_device_t(void);
+  virtual ~usb_device_t(void) {}
 
-// set it to 1 (align on byte) and save so we can pop it
-#pragma pack(push, 1)
-struct USB_DEVICE {
-  int     dev_type;       // our device type
-  bx_bool connect_status; // is device connected
-  Bit8u   address;        // 7 bit address
-  Bit8u   config;         // which configuration to use
-  Bit8u   Interface;      // which interface to use
-  Bit8u   alt_interface;  // which alt interface to use
-  Bit8u   endpt;          // which endpt to use
-  unsigned state;         // the state the device is in.  DEFAULT, ADDRESS, or CONFIGURED
-  bx_bool low_speed;      // 1 = ls 
-  Bit32u  scratch;        // 32-bit scratch area
-  int     fd;             // if this device accesses a file, this is the handle.
-  bx_bool in_stall;       // is this device in a stall state?
-  Bit8u   stall_once;     // some devices stall on the first setup packet after powerup
+  virtual int handle_packet(USBPacket *p);
+  virtual void handle_reset() {}
+  virtual int handle_control(int request, int value, int index, int length, Bit8u *data) {return 0;}
+  virtual int handle_data(USBPacket *p) {return 0;}
+  void register_state(bx_list_c *parent);
+  virtual void register_state_specific(bx_list_c *parent) {}
+  virtual void after_restore_state() {}
+
+  bx_bool get_connected() {return d.connected;}
+  usbdev_type get_type() {return d.type;}
+  int get_speed() {return d.speed;}
+  Bit8u get_address() {return d.addr;}
+
+protected:
   struct {
-    Bit8u   direction;
-    Bit8u   *in;
-    Bit8u   *out;
-    Bit16u  in_cnt;
-    Bit16u  out_cnt;
-    unsigned configs; ///// this is the same as configs below??????
-    struct {
-      Bit8u  len;
-      Bit8u  type;
-      Bit16u usb_ver;
-      Bit8u  _class;
-      Bit8u  subclass;
-      Bit8u  protocol;
-      Bit8u  max_packet_size;
-      Bit16u vendorid;
-      Bit16u productid;
-      Bit16u device_rel;
-      Bit8u  manuf_indx;
-      Bit8u  prod_indx;
-      Bit8u  serial_indx;
-      Bit8u  configs;
-    } device_descr;
-    struct {
-      Bit8u  len;
-      Bit8u  type;
-      Bit16u tot_len;
-      Bit8u  interfaces;
-      Bit8u  config_val;
-      Bit8u  config_indx;
-      Bit8u  attrbs;
-      Bit8u  max_power;
-      struct {
-        Bit8u  size;
-        Bit8u  type;
-        Bit8u  interface_num;
-        Bit8u  alternate;
-        Bit8u  num_endpts;
-        Bit8u  iclass;
-        Bit8u  subclass;
-        Bit8u  protocol;
-        Bit8u  str_indx;
-        struct {
-          Bit8u  size;
-          Bit8u  type;
-          Bit8u  endpt;
-          Bit8u  attrib;
-          Bit16u max_size;
-          Bit8u  interval;
-        } endpts[4];
-        struct {
-          Bit8u  size;
-          Bit8u  type;
-          Bit16u HID_class;
-          Bit8u  country_code;
-          Bit8u  num_descriptors;
-          struct {
-            Bit8u  type;
-            Bit16u len;
-            Bit8u  dev_hid_descript_report[128];
-          } descriptor[16];
-        } dev_hid_descript;
-        int lookup_cnt;
-        struct KEYPAD lookup[KEYPAD_LEN];
-      } Interface[4];
-    } device_config[4];
-    struct {
-      Bit8u  size;
-      Bit8u  type;
-      Bit16u langid[3];
-    } str_descriptor;
-    struct {
-      Bit8u  size;
-      Bit8u  type;
-      Bit8u  unicode_str[64];
-    } string[6];
-  } function;     // currently, we only support 1 function
+    enum usbdev_type type;
+    bx_bool connected;
+    int speed;
+    Bit8u addr;
+    Bit8u config;
+    char devname[32];
+
+    int state;
+    Bit8u setup_buf[8];
+    Bit8u data_buf[1024];
+    int remote_wakeup;
+    int setup_state;
+    int setup_len;
+    int setup_index;
+  } d;
 };
-#pragma pack(pop)
 
 typedef struct {
   Bit32u base_ioaddr;
@@ -292,7 +287,7 @@ typedef struct {
   //  Can only write in WORD sizes (Read in byte sizes???)
   struct {
     // our data
-    int     device_num;     // device number on this hub
+    usb_device_t *device;   // device connected to this port
 
     // bit reps of actual port
     bx_bool suspend;
@@ -310,8 +305,6 @@ typedef struct {
   Bit8u pci_conf[256];
   Bit8u devfunc;
 
-  struct USB_DEVICE device[USB_CUR_DEVS];  // two ports per hub, 127 devices per port (max)
-
   int statusbar_id[2]; // IDs of the status LEDs
 
 } bx_usb_t;
@@ -342,14 +335,12 @@ public:
   virtual void init(void);
   virtual void reset(unsigned);
   virtual void usb_mouse_enq(int delta_x, int delta_y, int delta_z, unsigned button_state);
-  virtual void usb_mouse_enable(bx_bool enable);
+  virtual void usb_mouse_enabled_changed(bx_bool enable);
   virtual bx_bool usb_key_enq(Bit8u *scan_code);
   virtual bx_bool usb_keyboard_connected();
   virtual bx_bool usb_mouse_connected();
-#if BX_SUPPORT_SAVE_RESTORE
   virtual void register_state(void);
   virtual void after_restore_state(void);
-#endif
   virtual Bit32u  pci_read_handler(Bit8u address, unsigned io_len);
   virtual void    pci_write_handler(Bit8u address, Bit32u value, unsigned io_len);
 
@@ -361,38 +352,20 @@ private:
   bx_usb_t hub[BX_USB_CONFDEV];
   Bit8u    global_reset;
 
-  int      mouse_delayed_dx;
-  int      mouse_delayed_dy;
-  int      mouse_delayed_dz;
-  unsigned button_state;
-  Bit8s    mouse_x;
-  Bit8s    mouse_y;
-  Bit8s    mouse_z;
-  Bit8u    b_state;
-
-  Bit8u    saved_key[8];
-  Bit8u    key_pad_packet[8];
-
   static void set_irq_level(bx_bool level);
   Bit8u  *device_buffer;
 
-  unsigned set_address_stk;
-  Bit8u    set_address[128];
+  usb_hid_device_t *mousedev;
+  usb_hid_device_t *keybdev;
 
-  bx_bool  last_connect;
-  bx_bool  keyboard_connected;
-  bx_bool  mouse_connected;
-  bx_bool  flash_connected;
+  USBPacket usb_packet;
 
-  static void init_device(Bit8u port, char *devname);
+  static void init_device(Bit8u port, const char *devname);
   static void usb_set_connect_status(Bit8u port, int type, bx_bool connected);
 
   static void usb_timer_handler(void *);
   void usb_timer(void);
   bx_bool DoTransfer(Bit32u address, Bit32u queue_num, struct TD *);
-  void dump_packet(Bit8u *data, unsigned size);
-  bx_bool flash_stick(Bit8u *packet, Bit16u size, bx_bool out);
-  unsigned GetDescriptor(struct USB_DEVICE *, struct REQUEST_PACKET *);
   void set_status(struct TD *td, bx_bool stalled, bx_bool data_buffer_error, bx_bool babble,
     bx_bool nak, bx_bool crc_time_out, bx_bool bitstuff_error, Bit16u act_len);
 
@@ -402,6 +375,7 @@ private:
   Bit32u read(Bit32u address, unsigned io_len);
   void   write(Bit32u address, Bit32u value, unsigned io_len);
 #endif
+  void usb_send_msg(usb_device_t *dev, int msg);
 };
 
 #endif
