@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: scancodes.cc,v 1.5 2002/10/24 21:07:51 bdenney Exp $
+// $Id: scancodes.cc,v 1.6 2007/08/18 08:05:33 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -492,8 +492,8 @@ scancode scancodes[BX_KEY_NBKEYS][3] =
  },
 
  { // BX_KEY_PRINT ( ibm 124 ) 
-   { "\xE0\x37" , "\xE0\xB7" },
-   { "\xE0\x7C" , "\xE0\xF0\x7C" },
+   { "\xE0\x2A\xE0\x37" , "\xE0\xB7\xE0\xAA" },
+   { "\xE0\x12\xE0\x7C" , "\xE0\xF0\x7C\xE0\xF0\x12" },
    { "\x57" ,     "\xF0\x57" },
  },
 
