@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: slowdown_timer.cc,v 1.25 2006/05/27 15:54:49 sshwarts Exp $
+// $Id: slowdown_timer.cc,v 1.27 2007/09/28 19:52:06 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -84,17 +84,19 @@ void bx_slowdown_timer_c::init(void)
   bx_pc_system.activate_timer(s.timer_handle,(Bit32u)s.Q,0);
 }
 
-#if BX_SUPPORT_SAVE_RESTORE
+void bx_slowdown_timer_c::exit(void)
+{
+  s.timer_handle = BX_NULL_TIMER_HANDLE;
+}
+
 void bx_slowdown_timer_c::after_restore_state(void)
 {
   s.start_emulated_time = bx_pc_system.time_usec();
 }
-#endif
 
 void bx_slowdown_timer_c::timer_handler(void * this_ptr)
 {
   bx_slowdown_timer_c * class_ptr = (bx_slowdown_timer_c *) this_ptr;
-
   class_ptr->handle_timer();
 }
 
