@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: vmware3.cc,v 1.15 2006/06/15 09:44:37 vruppert Exp $
+// $Id: vmware3.cc,v 1.18 2007/10/24 23:17:30 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 
 /*
  * This file provides support for VMWare's virtual disk image
- * format.
+ * format version 3.
  *
  * Author: Sharvil Nanavati, for Net Integration Technologies, Inc.
  * Contact: snrrrub@yahoo.com
@@ -163,7 +163,7 @@ char * vmware3_image_t::generate_cow_name(const char * filename, unsigned chain)
 {
     char * name = new char[strlen(filename) + 4];
     if(name == 0)
-        BX_PANIC(("unable to allocate %d bytes for vmware3 COW file name (base: %s, chain: %d)", strlen(filename) + 4, filename, chain));
+        BX_PANIC(("unable to allocate %lu bytes for vmware3 COW file name (base: %s, chain: %u)", strlen(filename) + 4, filename, chain));
     strcpy(name, filename);
     if(chain != 0)
     {
@@ -358,7 +358,7 @@ ssize_t vmware3_image_t::read(void * buf, size_t count)
         off_t offset = perform_seek();
         if(offset == INVALID_OFFSET)
         {
-            BX_DEBUG(("vmware3 COW read failed on %d bytes", count));
+            BX_DEBUG(("vmware3 COW read failed on %lu bytes", count));
             return -1;
         }
         unsigned bytes_remaining = (unsigned)(tlb_size - offset);
@@ -467,7 +467,7 @@ ssize_t vmware3_image_t::write(const void * buf, size_t count)
             memcpy(current->tlb + offset, buf, bytes_remaining);
             if(!sync())
             {
-                BX_DEBUG(("failed to sync when writing %d bytes", count));
+                BX_DEBUG(("failed to sync when writing %lu bytes", count));
                 return -1;
             }
             amount = bytes_remaining;
@@ -479,14 +479,14 @@ ssize_t vmware3_image_t::write(const void * buf, size_t count)
     return total;
 }
 
-off_t vmware3_image_t::lseek(off_t offset, int whence)
+Bit64s vmware3_image_t::lseek(Bit64s offset, int whence)
 {
     if(whence == SEEK_SET)
-        requested_offset = offset;
+        requested_offset = (off_t)offset;
     else if (whence == SEEK_CUR)
-        requested_offset += offset;
+        requested_offset += (off_t)offset;
     else if (whence == SEEK_END)
-        requested_offset = (off_t)(current->header.total_sectors_in_disk * 512) + offset;
+        requested_offset = (off_t)(current->header.total_sectors_in_disk * 512) + (off_t)offset;
     else
     {
         BX_DEBUG(("unknown 'whence' value (%d) when trying to seek vmware3 COW image", whence));
